@@ -67,8 +67,10 @@ typedef struct Avision_Connection {
 typedef struct Avision_HWEntry {
   char* scsi_mfg;
   char* scsi_model;
+
   int   usb_vendor;
   int   usb_product;
+
   const char* real_mfg;
   const char* real_model;
   
@@ -83,32 +85,35 @@ typedef struct Avision_HWEntry {
   /* feature overwrites */
   enum {
     /* force no calibration */
-    AV_NO_CALIB = 1,
+    AV_NO_CALIB = (1),
     
     /* force the special C5 calibration */
-    AV_C5_CALIB = 2,
+    AV_C5_CALIB = (1<<1),
     
     /* force all in one command calibration */
-    AV_ONE_CMD_CALIB = 4,
+    AV_ONE_CALIB_CMD = (1<<2),
     
     /* no gamma table */
-    AV_NO_GAMMA = 8,
+    AV_NO_GAMMA = (1<<3),
     
     /* light check is bogus */
-    AV_LIGHT_CHECK_BOGUS = 16,
+    AV_LIGHT_CHECK_BOGUS = (1<<4),
     
     /* do not use line packing even if line_difference */
-    AV_NO_LINE_DIFFERENCE = 32,
+    AV_NO_LINE_DIFFERENCE = (1<<5),
     
     /* limit the available resolutions */
-    AV_RES_HACK = 64,
+    AV_RES_HACK = (1<<6),
+    
+    /* some (?) USB devices like firmware */
+    AV_FIRMWARE = (1<<7),
     
     /* fujitsu adaption */
-    AV_FUJITSU = 128
+    AV_FUJITSU = (1<<8)
     
     /* maybe more ...*/
   } feature_type;
-  
+    
 } Avision_HWEntry;
 
 typedef enum {
@@ -125,6 +130,8 @@ typedef enum {
   AV_DITHERED,
   AV_GRAYSCALE,
   AV_TRUECOLOR,
+  AV_GRAYSCALE16,
+  AV_TRUECOLOR16,
   AV_COLOR_MODE_LAST
 } color_mode;
 
@@ -132,8 +139,8 @@ typedef enum {
   AV_NORMAL,
   AV_TRANSPARENT,
   AV_ADF,
-  AV_OPERATION_MODE_LAST
-} operation_mode;
+  AV_SOURCE_MODE_LAST
+} source_mode;
 
 enum Avision_Option
 {
@@ -141,12 +148,13 @@ enum Avision_Option
   
   OPT_MODE_GROUP,
   OPT_MODE,
-#define OPT_MODE_DEFAULT 3
   OPT_RESOLUTION,
 #define OPT_RESOLUTION_DEFAULT 300
   OPT_SPEED,
   OPT_PREVIEW,
   
+  OPT_SOURCE,            /* scan source normal, transparency, ADF */
+  
   OPT_GEOMETRY_GROUP,
   OPT_TL_X,	         /* top-left x */
   OPT_TL_Y,	         /* top-left y */
@@ -158,8 +166,6 @@ enum Avision_Option
   OPT_CONTRAST,
   OPT_QSCAN,
   OPT_QCALIB,
-  OPT_TRANS,             /* Transparency Mode */
-  OPT_ADF,               /* Use ADF unit */
   
   OPT_GAMMA_VECTOR,      /* first must be gray */
   OPT_GAMMA_VECTOR_R,    /* then r g b vector */
@@ -171,13 +177,6 @@ enum Avision_Option
   NUM_OPTIONS            /* must come last */
 };
 
-typedef union Option_Value
-{
-  SANE_Word w;
-  SANE_Word *wa;	 /* word array */
-  SANE_String s;
-} Option_Value;
-
 typedef struct Avision_Dimensions
 {
   /* in dpi */
@@ -219,32 +218,44 @@ typedef struct Avision_Device
   SANE_Bool inquiry_needs_software_colorpack;
   SANE_Bool inquiry_needs_line_pack;
   SANE_Bool inquiry_adf_need_mirror;
+  SANE_Bool inquiry_adf_bgr_order;
   SANE_Bool inquiry_light_detect;
   SANE_Bool inquiry_light_control;
   int       inquiry_max_shading_target;
   
+  /* the list of available color modes */
+  SANE_String_Const color_list[AV_COLOR_MODE_LAST];
+  color_mode color_list_num[AV_COLOR_MODE_LAST];
+  
+  /* the list of available source modes */
+  SANE_String_Const source_list[AV_SOURCE_MODE_LAST];
+  source_mode source_list_num[AV_SOURCE_MODE_LAST];
+  
   int inquiry_optical_res;        /* in dpi */
   int inquiry_max_res;            /* in dpi */
   
-  double inquiry_x_ranges  [AV_OPERATION_MODE_LAST]; /* in mm */
-  double inquiry_y_ranges  [AV_OPERATION_MODE_LAST]; /* in mm */
+  double inquiry_x_ranges  [AV_SOURCE_MODE_LAST]; /* in mm */
+  double inquiry_y_ranges  [AV_SOURCE_MODE_LAST]; /* in mm */
   
   int inquiry_color_boundary;
   int inquiry_gray_boundary;
   int inquiry_dithered_boundary;
   int inquiry_thresholded_boundary;
   int inquiry_line_difference; /* software color pack */
-  
-  /* int inquiry_bits_per_channel; */
+
+  int inquiry_channels_per_pixel;
+  int inquiry_bits_per_channel;
 
   int scsi_buffer_size; /* nice to have SCSI buffer size */
+
+  /* additional information - read delayed until sane_open() */
+  
+  SANE_Bool additional_probe;
   
   /* accessories */
   SANE_Bool acc_light_box;
   SANE_Bool acc_adf;
   
-  SANE_Bool is_adf; /* ADF scanner */
-  
   /* film scanner atributes - maybe these should be in the scanner struct? */
   SANE_Range frame_range;
   SANE_Word current_frame;
@@ -253,9 +264,6 @@ typedef struct Avision_Device
   /* some versin corrections */
   u_int16_t data_dq; /* was ox0A0D - but hangs some new scanners */
   
-  /* driver state */
-  SANE_Bool is_calibrated;
-  
   Avision_HWEntry* hw;
 } Avision_Device;
 
@@ -269,8 +277,10 @@ typedef struct Avision_Scanner
   Option_Value val [NUM_OPTIONS];
   SANE_Int gamma_table [4][256];
   
-  /* page (ADF) state */
-  int page;
+  /* we now save the calib data because we might need it for 16bit software
+     calibration :-( */
+  u_int8_t* dark_avg_data;
+  u_int8_t* white_avg_data;
   
   /* Parsed option values and variables that are valid only during
      the actual scan: */
@@ -280,7 +290,7 @@ typedef struct Avision_Scanner
   Avision_Dimensions avdimen;   /* scan window - detailed internals */
 
   color_mode c_mode;
-  operation_mode o_mode;
+  source_mode source_mode;
   
   /* Avision HW Access Connection (SCSI/USB abstraction) */
   Avision_Connection av_con;
@@ -297,6 +307,12 @@ typedef struct Avision_Scanner
 /* Some Avision driver internal defines */
 #define AV_WINID 0
 
+/* SCSI error codes */
+#define AVISION_SCSI_GOOD                   0x00
+#define AVISION_SCSI_CONDITION_GOOD         0x02
+#define AVISION_SCSI_INTERMEDIATE_GOOD      0x08
+#define AVISION_SCSI_INTERMEDIATE_C_GOOD    0x0a
+
 /* SCSI commands that the Avision scanners understand: */
 
 #define AVISION_SCSI_TEST_UNIT_READY        0x00
@@ -482,8 +498,8 @@ typedef struct calibration_format
 } calibration_format;
 
 
-/* set/get SCSI highended variables. Declare them as an array of chars */
-/* endianness-safe, int-size safe ... */
+/* set/get SCSI highended (big-endian) variables. Declare them as an array
+ * of chars endianness-safe, int-size safe ... */
 #define set_double(var,val) var[0] = ((val) >> 8) & 0xff;  \
                             var[1] = ((val)     ) & 0xff
 
@@ -505,7 +521,13 @@ typedef struct calibration_format
                          (*(var + 1) << 16) + \
                          (*(var + 2) << 8) + *(var + 3))
 
-#define BIT(n, p) ((n & ( 1 << p)) ? 1 : 0)
+/* set/get Avision lowended (little-endian) shading data */
+#define set_double_le(var,val) var[0] = ((val)     ) & 0xff;  \
+                               var[1] = ((val) >> 8) & 0xff
+
+#define get_double_le(var) ((*(var + 1) << 8) + *var)
+
+#define BIT(n, p) ((n & (1 << p)) ? 1 : 0)
 
 #define SET_BIT(n, p) (n |= (1 << p))
 
