@@ -9,16 +9,9 @@
 
 /* ------------------------------------------------------------------------- */
 
-static int num_devices;
+static int num_devices = 0;
 static struct fujitsu *first_dev;
 
-typedef union
-  {
-    SANE_Word w;
-    SANE_String s;
-  }
-Option_Value;
-
 /* ------------------------------------------------------------------------- 
  * This option list has to contain all options for all scanners supported by
  * this driver. If a certain scanner cannot handle a certain option, there's
@@ -52,6 +45,7 @@ enum fujitsu_Option
 
   OPT_RIF,
   OPT_COMPRESSION,
+  OPT_COMPRESSION_ARG,
 
   OPT_DTC_SELECTION,
   OPT_GAMMA,
@@ -140,6 +134,7 @@ struct fujitsu
   /* when scanning duplex                   */
 
   int can_read_alternate;       /* duplex transfer mode front/back/front/back... */
+  int read_mode;                /* constants READ_MODE_* and funcs convert_* */
   int has_adf;                  /* true if an ADF has been detected.       */
   int has_fb;
   int duplex_present;           /* true if a duplex unit has been detected. */
@@ -158,6 +153,9 @@ struct fujitsu
   int has_threshold;
   int has_brightness;
   int has_contrast;
+  int has_gamma;
+  int num_download_gamma;
+  int has_fixed_paper_size;
 
   SANE_Range adf_width_range;
   SANE_Range adf_height_range;
@@ -359,8 +357,6 @@ struct fujitsu
 #define MODEL_3091 1
 /** Fujitsu M3096 (400x400dpi grayscale, simplex, FB and optional ADF) */
 #define MODEL_3096 2
-/** Fujitsu ScanPartner 15C (....) */
-#define MODEL_SP15 3
 /** 3093 */
 #define MODEL_3093 4
 /** 4097 */
@@ -370,6 +366,8 @@ struct fujitsu
 #define MODEL_3097 7
 /** Fujitsu M3092DCd (300x600dpi, color, duplex, FB and ADF) */
 #define MODEL_3092 8
+/** ScanPartner fi-4120 and ScanParter fi-4220 scsi/usb */
+#define MODEL_FI4x20   9
 
 /* A note regarding the MODEL... constants. There's a place in
  * identifyScanner() where the INQUIRY data is parsed and the model
@@ -418,6 +416,11 @@ struct fujitsu
 #define mmToIlu(mm) ((mm) / length_quant)
 #define iluToMm(ilu) ((ilu) * length_quant)
 #define FUJITSU_CONFIG_FILE "fujitsu.conf"
+#define READ_MODE_PASS 1
+#define READ_MODE_RRGGBB 2
+#define READ_MODE_3091RGB 3
+#define READ_MODE_BGR 4
+
 
 #define FIXED_MM_TO_SCANNER_UNIT(number) SANE_UNFIX(number) * 1200 / MM_PER_INCH
 #define SCANNER_UNIT_TO_FIXED_MM(number) SANE_FIX(number * MM_PER_INCH / 1200)
@@ -534,17 +537,22 @@ static unsigned int reader3091ColorSimplex (struct fujitsu *scanner,
                                             FILE * fd);
 static unsigned int reader3091GrayDuplex (struct fujitsu *scanner, FILE * fd,
                                           FILE * fd2);
-static unsigned int reader3092ColorDuplex (struct fujitsu *scanner, FILE * fd,
-                                          FILE * fd2);
-static unsigned int reader3092ColorSimplex (struct fujitsu *scanner,
-                                           FILE * fd);
-static unsigned int reader3092GrayDuplex (struct fujitsu *scanner, FILE * fd,
-                                         FILE * fd2);
+static void convert_bgr_to_rgb(struct fujitsu *scanner, unsigned char * buffptr, unsigned int length);
+
+static void convert_rrggbb_to_rgb(struct fujitsu *scanner, unsigned char * buffptr, unsigned int length);
+
 static unsigned int reader_gray_duplex_sequential (struct fujitsu *scanner,
                                                    FILE * fd, FILE * fd2);
 static unsigned int reader_gray_duplex_alternate (struct fujitsu *scanner,
                                                   FILE * fd, FILE * fd2);
 
+static unsigned int reader_duplex_sequential (struct fujitsu *scanner,
+                                                   FILE * fd, FILE * fd2);
+static unsigned int reader_duplex_alternate (struct fujitsu *scanner,
+                                                  FILE * fd, FILE * fd2);
+static unsigned int reader_simplex (struct fujitsu *scanner, FILE * fd, 
+					int i_window_id);
+
 static unsigned int reader_generic_passthrough (struct fujitsu *scanner,
                                                 FILE * fd, int i_window_id);
 
@@ -558,14 +566,10 @@ static SANE_Status attachOne (const char *name);
 static int modelMatch (const char *product);
 
 static void setDefaults3091 (struct fujitsu *scanner);
-static void setDefaults3092 (struct fujitsu *scanner);
 static void setDefaults3096 (struct fujitsu *scanner);
-static void setDefaultsSP15 (struct fujitsu *scanner);
 
 static SANE_Status setMode3091 (struct fujitsu *scanner, int mode);
-static SANE_Status setMode3092 (struct fujitsu *scanner, int mode);
 static SANE_Status setMode3096 (struct fujitsu *scanner, int mode);
-static SANE_Status setModeSP15 (struct fujitsu *scanner, int mode);
 
 static void calculateDerivedValues (struct fujitsu *scanner);
 
