@@ -1,7 +1,7 @@
 /* sane - Scanner Access Now Easy.
 
    Copyright (C) 2002 Sergey Vlasov <vsu@altlinux.ru>
-   Copyright (C) 2002, 2003 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2002 - 2004 Henning Meier-Geinitz <henning@meier-geinitz.de>
    
    This file is part of the SANE package.
    
@@ -164,6 +164,44 @@ static GT68xx_Command_Set plustek_gt6801_command_set = {
   gt68xx_generic_get_id
 };
 
+static GT68xx_Command_Set plustek_u16b_command_set = {
+  "plustek-u16b",
+
+  0x40,				/* Request type */
+  0x04,				/* Request */
+
+  0x200a,			/* Memory read - wValue */
+  0x2009,			/* Memory write - wValue */
+
+  0x2010,			/* Send normal command - wValue */
+  0x3f40,			/* Send normal command - wIndex */
+  0x2011,			/* Receive normal result - wValue */
+  0x3f00,			/* Receive normal result - wIndex */
+
+  0x2012,			/* Send small command - wValue */
+  0x3f40,			/* Send small command - wIndex */
+  0x2013,			/* Receive small result - wValue */
+  0x3f00,			/* Receive small result - wIndex */
+
+  /* activate */ NULL,
+  /* deactivate */ NULL,
+  gt6801_check_plustek_firmware, /* ok */
+  gt6801_u16b_download_firmware, /* ok */
+  gt6801_get_power_status, /* not tested */
+  /* get_ta_status (FIXME: implement this) */ NULL,
+  gt6801_lamp_control, /* ok */
+  gt6816_is_moving, /* ok */
+  /* gt68xx_generic_move_relative *** to be tested */ NULL,
+  gt6801_carriage_home, /* ok */
+  gt68xx_generic_start_scan, /* ok */
+  gt68xx_generic_read_scanned_data, /* ok */
+  gt6801_u16b_stop_scan,  /* ok */
+  gt6816_setup_scan, /* ? */
+  gt68xx_generic_set_afe, /* ok */
+  /* set_exposure_time */ NULL,
+  gt68xx_generic_get_id /* not tested */
+};
+
 static GT68xx_Model mustek_2400ta_model = {
   "mustek-bearpaw-2400-ta",	/* Name */
   "Mustek",			/* Device vendor string */
@@ -261,6 +299,54 @@ static GT68xx_Model mustek_2400taplus_model = {
     /* Setup and tested */
 };
 
+static GT68xx_Model mustek_2448taplus_model = {
+  "mustek-bearpaw-2448-ta-plus",	/* Name */
+  "Mustek",			/* Device vendor string */
+  "BearPaw 2448 TA Plus",	/* Device model name */
+  "A2Nfw.usb",			/* Name of the firmware file */
+  SANE_FALSE,			/* Dynamic allocation flag */
+
+  &mustek_gt6816_command_set,	/* Command set used by this scanner */
+
+  1200,				/* maximum optical sensor resolution */
+  2400,				/* maximum motor resolution */
+  600,				/* base x-res used to calculate geometry */
+  1200,				/* base y-res used to calculate geometry */
+  1200,				/* if ydpi is equal or higher, use linemode */
+  SANE_FALSE,			/* Use base_ydpi for all resolutions */
+
+  {1200, 600, 300, 200, 100, 0},	/* possible x-resolutions */
+  {2400, 1200, 600, 300, 200, 100, 0},	/* possible y-resolutions */
+  {16, 12, 8, 0},			/* possible depths in gray mode */
+  {16, 12, 8, 0},		/* possible depths in color mode */
+
+  SANE_FIX (7.41),		/* Start of scan area in mm  (x) */
+  SANE_FIX (7.4),		/* Start of scan area in mm (y) */
+  SANE_FIX (216.3),		/* Size of scan area in mm (x) */
+  SANE_FIX (297.5),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (3.9),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (94.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (105.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (36.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (165.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (95.0),		/* Start of white strip in TA mode in mm (y) */
+
+  0, 48, 96,			/* RGB CCD Line-distance correction in pixel */
+  8,				/* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+  {0x2a, 0x0c, 0x2e, 0x06, 0x2d, 0x07},	/* Default offset/gain */
+  {0x157, 0x157, 0x157},	/* Default exposure parameters */
+  SANE_FIX (2.0),		/* Default gamma value */
+
+  SANE_FALSE,			/* Is this a CIS scanner? */
+  0				/* Which flags are needed for this scanner? */
+    /* Based on data from Jakub Dvo?k <xdvorak@chello.cz>. */
+};
 
 static GT68xx_Model mustek_1200ta_model = {
   "mustek-bearpaw-1200-ta",	/* Name */
@@ -679,12 +765,12 @@ static GT68xx_Model mustek_a3usb_model = {
   {12, 8, 0},			/* possible depths in color mode */
 
   SANE_FIX (6.0),		/* Start of scan area in mm  (x) */
-  SANE_FIX (14.0),		/* Start of scan area in mm (y) */
-  SANE_FIX (297.0),		/* Size of scan area in mm (x) */
-  SANE_FIX (431.0),		/* Size of scan area in mm (y) */
+  SANE_FIX (10.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (433.0),		/* Size of scan area in mm (y) */
 
-  SANE_FIX (5.0),		/* Start of white strip in mm (y) */
-  SANE_FIX (0.0),		/* Start of black mark in mm (x) */
+  SANE_FIX (2.4),		/* Start of white strip in mm (y) */
+  SANE_FIX (4.3),		/* Start of black mark in mm (x) */
 
   SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
   SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
@@ -693,18 +779,19 @@ static GT68xx_Model mustek_a3usb_model = {
 
   SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
 
-  0, 0, 0,			/* RGB CCD Line-distance correction in pixel */
+  0, 4, 4,			/* RGB CCD Line-distance correction in pixel */
   0,				/* CCD distcance for CCD with 6 lines) */
 
   COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
-  {0x14, 0x07, 0x14, 0x07, 0x14, 0x07},	/* Default offset/gain */
+  {0x14, 0x05, 0x12, 0x05, 0x17, 0x0c},	/* Default offset/gain */
   {0x157, 0x157, 0x157},	/* Default exposure parameters */
   SANE_FIX (2.0),		/* Default gamma value */
 
-  SANE_FALSE,			/* Is this a CIS scanner? */
-  0				/* Which flags are needed for this scanner? */
-    /* Tested by Pedro Morais <morais@inocam.com>, changes to CCD by hmg, white
-       strip/black mark untested */
+  SANE_TRUE,			/* Is this a CIS scanner? */
+  GT68XX_FLAG_CIS_LAMP /* Which flags are needed for this scanner? */
+    /* Tested by some people. This scanner is a bit strange as it uses a CIS
+       sensor but it also has a lamp. So the lamp needs to be heated but CIS
+       mode must be used for scanning. */
 };
 
 static GT68xx_Model lexmark_x73_model = {
@@ -815,23 +902,23 @@ static GT68xx_Model plustek_u16b_model = {
   "ccd68861.fw",		/* Name of the firmware file */
   SANE_FALSE,			/* Dynamic allocation flag */
 
-  &plustek_gt6801_command_set,	/* Command set used by this scanner */
+  &plustek_u16b_command_set,	/* Command set used by this scanner */
 
   600,				/* maximum optical sensor resolution */
   1200,				/* maximum motor resolution */
   600,				/* base x-res used to calculate geometry */
   600,				/* base y-res used to calculate geometry */
-  50,				/* if ydpi is equal or higher, use linemode */
+  2400,				/* if ydpi is equal or higher, use linemode */
   SANE_TRUE,			/* Use base_ydpi for all resolutions */
 
   {600, 300, 150, 75, 0},	/* possible x-resolutions */
-  {1200, 600, 300, 150, 75, 0},	/* possible y-resolutions */
-  {12, 8, 0},			/* possible depths in gray mode */
-  {12, 8, 0},			/* possible depths in color mode */
+  {600, 300, 150, 75, 0},	/* possible y-resolutions */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 12, 8, 0},			/* possible depths in color mode */
 
-  SANE_FIX (3.5),		/* Start of scan area in mm  (x) */
-  SANE_FIX (7.5),		/* Start of scan area in mm (y) */
-  SANE_FIX (218.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (5.5),		/* Start of scan area in mm  (x) */
+  SANE_FIX (8.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (216.0),		/* Size of scan area in mm (x) */
   SANE_FIX (299.0),		/* Size of scan area in mm (y) */
 
   SANE_FIX (0.0),		/* Start of white strip in mm (y) */
@@ -844,19 +931,70 @@ static GT68xx_Model plustek_u16b_model = {
 
   SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
 
-  0, 8, 16,			/* RGB CCD Line-distance correction in pixel */
+  0, 16, 32,			/* RGB CCD Line-distance correction in pixel */
   0,				/* CCD distcance for CCD with 6 lines) */
 
   COLOR_ORDER_BGR,		/* Order of the CCD/CIS colors */
-  {0x1c, 0x29, 0x1c, 0x2c, 0x1c, 0x2b},	/* Default offset/gain */
+  {0x18, 0x16, 0x16, 0x0f, 0x17, 0x11},	/* Default offset/gain */
   {0x157, 0x157, 0x157},	/* Default exposure parameters */
   SANE_FIX (2.0),		/* Default gamma value */
 
   SANE_FALSE,			/* Is this a CIS scanner? */
-  GT68XX_FLAG_UNTESTED | GT68XX_FLAG_OFFSET_INV	/* Which flags are needed for this scanner? */
-  /* completely untested */
+  GT68XX_FLAG_OFFSET_INV	/* Which flags are needed for this scanner? */
+  /* Tested with a U16B by Henning Meier-Geinitz */
 };
 
+static GT68xx_Model genius_vivid4_model = {
+  "genius-colorpage-vivid4",		/* Name */
+  "Genius",			/* Device vendor string */
+  "ColorPage Vivid 4",		/* Device model name */
+  "ccd68861.fw",		/* Name of the firmware file */
+  SANE_FALSE,			/* Dynamic allocation flag */
+
+  &plustek_u16b_command_set,	/* Command set used by this scanner */
+
+  600,				/* maximum optical sensor resolution */
+  1200,				/* maximum motor resolution */
+  600,				/* base x-res used to calculate geometry */
+  600,				/* base y-res used to calculate geometry */
+  2400,				/* if ydpi is equal or higher, use linemode */
+  SANE_TRUE,			/* Use base_ydpi for all resolutions */
+
+  {600, 300, 150, 75, 0},	/* possible x-resolutions */
+  {600, 300, 150, 75, 0},	/* possible y-resolutions */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 12, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (5.5),		/* Start of scan area in mm  (x) */
+  SANE_FIX (8.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (216.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (1.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  0, 16, 32,			/* RGB CCD Line-distance correction in pixel */
+  0,				/* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_BGR,		/* Order of the CCD/CIS colors */
+  {0x18, 0x16, 0x16, 0x0f, 0x17, 0x11},	/* Default offset/gain */
+  {0x157, 0x157, 0x157},	/* Default exposure parameters */
+  SANE_FIX (2.0),		/* Default gamma value */
+
+  SANE_FALSE,			/* Is this a CIS scanner? */
+  GT68XX_FLAG_OFFSET_INV	/* Which flags are needed for this scanner? */
+  /* Untested but as this scanner seems to be very similar to Plustelk U16B it
+     should work. */
+};
+
+
 static GT68xx_Model genius_vivid3x_model  = {
   "genius-colorpage-vivid3x",	/* Name */
   "Genius",			/* Device vendor string */
@@ -906,6 +1044,104 @@ static GT68xx_Model genius_vivid3x_model  = {
   /* Tested zo some degree, based on the Plustek OpticPro 1248U*/
 };
 
+static GT68xx_Model genius_vivid4x_model  = {
+  "genius-colorpage-vivid4x",	/* Name */
+  "Genius",			/* Device vendor string */
+  "Colorpage Vivid4x",		/* Device model name */
+  "ccd548.fw",			/* Name of the firmware file */
+  SANE_FALSE,			/* Dynamic allocation flag */
+
+  &mustek_gt6816_command_set,	/* Command set used by this scanner */
+
+  600,				/* maximum optical sensor resolution */
+  600,				/* maximum motor resolution */
+  600,				/* base x-res used to calculate geometry */
+  600,				/* base y-res used to calculate geometry */
+  50,				/* if ydpi is equal or higher, use linemode */
+  SANE_TRUE,			/* Use base_ydpi for all resolutions */
+
+  {600, 300, 150, 75, 0},	/* possible x-resolutions */
+  {600, 300, 150, 75, 0},	/* possible y-resolutions */
+  {12, 8, 0},			/* possible depths in gray mode */
+  {12, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (3.5),		/* Start of scan area in mm  (x) */
+  SANE_FIX (7.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (218.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (1.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  0, 8, 16,			/* RGB CCD Line-distance correction in pixel */
+  0,				/* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+  {0x1c, 0x29, 0x1c, 0x2c, 0x1c, 0x2b},	/* Default offset/gain */
+  {0x157, 0x157, 0x157},	/* Default exposure parameters */
+  SANE_FIX (2.0),		/* Default gamma value */
+
+  SANE_FALSE,			/* Is this a CIS scanner? */
+  GT68XX_FLAG_OFFSET_INV | GT68XX_FLAG_UNTESTED 	/* Which flags are needed for this scanner? */
+  /* Mostly untested, copied from 3x, some values from Claudio Filho <filhocf@openoffice.org> */
+};
+
+static GT68xx_Model genius_vivid4xe_model  = {
+  "genius-colorpage-vivid4xe",  /* Name */
+  "Genius",                     /* Device vendor string */
+  "Colorpage Vivid4xe",         /* Device model name */
+  "ccd548.fw",                  /* Name of the firmware file */
+  SANE_FALSE,                   /* Dynamic allocation flag */
+
+  &mustek_gt6816_command_set,  /* Command set used by this scanner */
+
+  600,                          /* maximum optical sensor resolution */
+  600,                          /* maximum motor resolution */
+  600,                          /* base x-res used to calculate geometry */
+  600,                          /* base y-res used to calculate geometry */
+  50,                           /* if ydpi is equal or higher, use linemode */
+  SANE_TRUE,                    /* Use base_ydpi for all resolutions */
+
+  {600, 300, 150, 75, 0},       /* possible x-resolutions */
+  {600, 300, 150, 75, 0},       /* possible y-resolutions */
+  {12, 8, 0},                   /* possible depths in gray mode */
+  {12, 8, 0},                   /* possible depths in color mode */
+
+  SANE_FIX (3.5),               /* Start of scan area in mm  (x) */
+  SANE_FIX (7.5),               /* Start of scan area in mm (y) */
+  SANE_FIX (218.0),             /* Size of scan area in mm (x) */
+  SANE_FIX (299.0),             /* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),               /* Start of white strip in mm (y) */
+  SANE_FIX (1.0),               /* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),               /* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),               /* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),             /* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),             /* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),               /* Start of white strip in TA mode in mm (y) */
+
+  0, 8, 16,                     /* RGB CCD Line-distance correction in pixel */
+  0,                            /* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_RGB,              /* Order of the CCD/CIS colors */
+  {0x1c, 0x29, 0x1c, 0x2c, 0x1c, 0x2b}, /* Default offset/gain */
+  {0x157, 0x157, 0x157},        /* Default exposure parameters */
+  SANE_FIX (2.0),               /* Default gamma value */
+
+  SANE_FALSE,                   /* Is this a CIS scanner? */
+  GT68XX_FLAG_OFFSET_INV /* Which flags are needed for this scanner? */
+  /* tested a bit */
+};
+
 static GT68xx_Model genius_vivid3xe_model  = {
   "genius-colorpage-vivid3xe",	/* Name */
   "Genius",			/* Device vendor string */
@@ -955,6 +1191,202 @@ static GT68xx_Model genius_vivid3xe_model  = {
   /* mostly untested, based on the Genius Vivid3x */
 };
 
+static GT68xx_Model genius_vivid1200ex_model  = {
+  "genius-colorpage-vivid-1200-ex",  /* Name */
+  "Genius",                     /* Device vendor string */
+  "Colorpage Vivid 1200 EX",         /* Device model name */
+  "ccd548.fw",                  /* Name of the firmware file */
+  SANE_FALSE,                   /* Dynamic allocation flag */
+
+  &mustek_gt6816_command_set,  /* Command set used by this scanner */
+
+  600,                          /* maximum optical sensor resolution */
+  600,                          /* maximum motor resolution */
+  600,                          /* base x-res used to calculate geometry */
+  600,                          /* base y-res used to calculate geometry */
+  50,                           /* if ydpi is equal or higher, use linemode */
+  SANE_TRUE,                    /* Use base_ydpi for all resolutions */
+
+  {600, 300, 150, 75, 0},       /* possible x-resolutions */
+  {600, 300, 150, 75, 0},       /* possible y-resolutions */
+  {12, 8, 0},                   /* possible depths in gray mode */
+  {12, 8, 0},                   /* possible depths in color mode */
+
+  SANE_FIX (3.5),               /* Start of scan area in mm  (x) */
+  SANE_FIX (7.5),               /* Start of scan area in mm (y) */
+  SANE_FIX (218.0),             /* Size of scan area in mm (x) */
+  SANE_FIX (299.0),             /* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),               /* Start of white strip in mm (y) */
+  SANE_FIX (1.0),               /* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),               /* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),               /* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),             /* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),             /* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),               /* Start of white strip in TA mode in mm (y) */
+
+  0, 8, 16,                     /* RGB CCD Line-distance correction in pixel */
+  0,                            /* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_BGR,              /* Order of the CCD/CIS colors */
+  {0x1c, 0x29, 0x1c, 0x2c, 0x1c, 0x2b}, /* Default offset/gain */
+  {0x157, 0x157, 0x157},        /* Default exposure parameters */
+  SANE_FIX (2.0),               /* Default gamma value */
+
+  SANE_FALSE,                   /* Is this a CIS scanner? */
+  GT68XX_FLAG_OFFSET_INV | GT68XX_FLAG_UNTESTED /* Which flags are needed for this scanner? */
+  /* untested */
+};
+
+static GT68xx_Model plustek_opticslim1200_model = {
+  "plustek-opticslim-1200",	/* Name */
+  "Plustek",				/* Device vendor string */
+  "OpticSlim 1200",			/* Device model name */
+  "cism216.fw",				/* Name of the firmware file */
+  SANE_FALSE,				/* Dynamic allocation flag */
+
+  &mustek_gt6816_command_set,	/* Command set used by this scanner */
+
+  600,					/* maximum optical sensor resolution */
+  1200,					/* maximum motor resolution */
+  600,					/* base x-res used to calculate geometry */
+  600,					/* base y-res used to calculate geometry */
+  1200,					/* if ydpi is equal or higher, use linemode */
+  SANE_FALSE,			/* Use base_ydpi for all resolutions */
+
+  {600, 300, 150, 75, 50, 0},	/* possible x-resolutions */
+  {1200, 600, 300, 150, 75, 50, 0},	/* possible y-resolutions */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (1.0),		/* Start of scan area in mm  (x) */
+  SANE_FIX (9.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (218.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (140.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  0, 0, 0,				/* RGB CCD Line-distance correction in pixel */
+  0,					/* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+  {0x15, 0x09, 0x18, 0x11, 0x16, 0x0c},	/* Default offset/gain */
+  {0x157, 0x157, 0x157},	/* Default exposure parameters */
+  SANE_FIX (2.0),			/* Default gamma value */
+
+  SANE_TRUE,				/* Is this a CIS scanner? */
+  0
+};
+
+static GT68xx_Model plustek_opticslim2400_model = {
+  "plustek-opticslim-2400",	/* Name */
+  "Plustek",				/* Device vendor string */
+  "OpticSlim 2400",			/* Device model name */
+  "cis3R5B1.fw",			/* Name of the firmware file */
+  SANE_FALSE,				/* Dynamic allocation flag */
+
+  &mustek_gt6816_command_set,	/* Command set used by this scanner */
+
+  1200,					/* maximum optical sensor resolution */
+  2400,					/* maximum motor resolution */
+  1200,					/* base x-res used to calculate geometry */
+  1200,					/* base y-res used to calculate geometry */
+  2400,					/* if ydpi is equal or higher, use linemode */
+  SANE_FALSE,			/* Use base_ydpi for all resolutions */
+
+  {1200, 600, 300, 150, 75, 50, 0},	/* possible x-resolutions */
+  {2400, 1200, 600, 300, 150, 75, 50, 0},	/* possible y-resolutions */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (1.0),		/* Start of scan area in mm  (x) */
+  SANE_FIX (9.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (218.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (140.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  0, 0, 0,				/* RGB CCD Line-distance correction in pixel */
+  0,					/* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+  {0x15, 0x09, 0x18, 0x11, 0x16, 0x0c},	/* Default offset/gain */
+  {0x157, 0x157, 0x157},	/* Default exposure parameters */
+  SANE_FIX (2.0),			/* Default gamma value */
+
+  SANE_TRUE,				/* Is this a CIS scanner? */
+  GT68XX_FLAG_UNTESTED
+};
+/* By Detlef Gausepohl <detlef at sunrise.psycho.rwth-aachen.de> */
+
+static GT68xx_Model visioneer_onetouch_7300_model = {
+  "visioneer-onetouch-7300",	/* Name */
+  "Visioneer",				/* Device vendor string */
+  "OneTouch 7300",			/* Device model name */
+  "Cis3r5b1.fw",			/* Name of the firmware file */
+  SANE_FALSE,				/* Dynamic allocation flag */
+
+  &mustek_gt6816_command_set,	/* Command set used by this scanner */
+
+  1200,					/* maximum optical sensor resolution */
+  1200,					/* maximum motor resolution */
+  1200,					/* base x-res used to calculate geometry */
+  1200,					/* base y-res used to calculate geometry */
+  1200,					/* if ydpi is equal or higher, use linemode */
+  SANE_FALSE,			/* Use base_ydpi for all resolutions */
+
+  {1200, 600, 300, 150, 75, 50, 0},	/* possible x-resolutions */
+  {1200, 1200, 600, 300, 150, 75, 50, 0},	/* possible y-resolutions */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (1.0),		/* Start of scan area in mm  (x) */
+  SANE_FIX (9.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (218.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (140.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  0, 0, 0,				/* RGB CCD Line-distance correction in pixel */
+  0,					/* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+  {0x15, 0x09, 0x18, 0x11, 0x16, 0x0c},	/* Default offset/gain */
+  {0x157, 0x157, 0x157},	/* Default exposure parameters */
+  SANE_FIX (2.0),			/* Default gamma value */
+
+  SANE_TRUE,				/* Is this a CIS scanner? */
+  GT68XX_FLAG_UNTESTED
+};
+/* Untested. Based on Plustek OpticPro 2400. */
+
+
 static GT68xx_USB_Device_Entry gt68xx_usb_device_list[] = {
   {0x055f, 0x0218, &mustek_2400ta_model},
   {0x055f, 0x0219, &mustek_2400taplus_model},
@@ -967,11 +1399,20 @@ static GT68xx_USB_Device_Entry gt68xx_usb_device_list[] = {
   {0x05d8, 0x4002, &mustek_2400cu_model},	/* manual override */
   {0x05d8, 0x4002, &mustek_scanexpress2400usb_model}, /* manual override */
   {0x055f, 0x0210, &mustek_a3usb_model},
+  {0x055f, 0x021a, &mustek_2448taplus_model},
   {0x043d, 0x002d, &lexmark_x73_model},
   {0x07b3, 0x0400, &plustek_op1248u_model},
   {0x07b3, 0x0401, &plustek_op1248u_model}, /* Same scanner, different id? */
-  {0x07b3, 0x0403, &plustek_u16b_model},
+  {0x07b3, 0x0402, &plustek_u16b_model},
+  {0x07b3, 0x0403, &plustek_u16b_model}, /* two ids? 403 seems to be more common */
+  {0x07b3, 0x0413, &plustek_opticslim1200_model},
+  {0x07b3, 0x0422, &plustek_opticslim2400_model},
   {0x0458, 0x2011, &genius_vivid3x_model},
+  {0x0458, 0x2014, &genius_vivid4_model}, 
   {0x0458, 0x2017, &genius_vivid3xe_model},
+  {0x0458, 0x201a, &genius_vivid4xe_model},
+  {0x0458, 0x201b, &genius_vivid4x_model},
+  {0x0458, 0x201f, &genius_vivid1200ex_model},
+  {0x04a7, 0x0444, &visioneer_onetouch_7300_model},
   {0, 0, NULL}
 };
