@@ -172,6 +172,44 @@ gt6801_download_firmware (GT68xx_Device * dev,
   return SANE_STATUS_GOOD;
 }
 
+SANE_Status
+gt6801_u16b_download_firmware (GT68xx_Device * dev,
+			       SANE_Byte * data, SANE_Word size)
+{
+  DECLARE_FUNCTION_NAME ("gt6801_download_firmware") SANE_Status status;
+  SANE_Byte download_buf[MAX_DOWNLOAD_BLOCK_SIZE];
+  SANE_Byte *block;
+  SANE_Word addr, bytes_left;
+  GT68xx_Packet boot_req;
+  SANE_Word block_size = MAX_DOWNLOAD_BLOCK_SIZE;
+
+  CHECK_DEV_ACTIVE (dev, function_name);
+
+  for (addr = 0; addr < size; addr += block_size)
+    {
+      bytes_left = size - addr;
+      if (bytes_left > block_size)
+	block = data + addr;
+      else
+	{
+	  memset (download_buf, 0, block_size);
+	  memcpy (download_buf, data + addr, bytes_left);
+	  block = download_buf;
+	}
+      RIE (gt68xx_device_memory_write (dev, addr, block_size, block));
+    }
+
+  /* not tested: */
+  memset (boot_req, 0, sizeof (boot_req));
+  boot_req[0] = 0x69;
+  boot_req[1] = 0x01;
+  boot_req[2] = 0xc0;
+  boot_req[3] = 0x1c;
+  RIE (gt68xx_device_req (dev, boot_req, boot_req));
+
+  return SANE_STATUS_GOOD;
+}
+
 
 SANE_Status
 gt6801_get_power_status (GT68xx_Device * dev, SANE_Bool * power_ok)
@@ -277,6 +315,41 @@ gt6801_stop_scan (GT68xx_Device * dev)
   return SANE_STATUS_GOOD;
 }
 
+SANE_Status
+gt6801_u16b_stop_scan (GT68xx_Device * dev)
+{
+  GT68xx_Packet req;
+  SANE_Status status;
+  SANE_Int count = 0;
+
+  /* For some reason, the Plustek U16B needs both stop commands. */
+  memset (req, 0, sizeof (req));
+  req[0] = 0x42;
+  req[1] = 0x01;
+
+  DBG (5, "gt6801_u16b_stop_scan: stopping scan\n");
+  for (count = 0; count < 20; count++)
+    {  
+      memset (req, 0, sizeof (req));
+      req[0] = 0x42;
+      req[1] = 0x01;
+      RIE (gt68xx_device_req (dev, req, req));
+      if (gt68xx_device_check_result (req, 0x42) == SANE_STATUS_GOOD)
+	break;
+      DBG (7, "gt6801_u16b_stop_scan: failed, count = %d\n", count);
+    }
+  if (count <= 20)
+    {
+      DBG (7, "gt6801_u16b_stop_scan: succeeded\n");
+      memset (req, 0, sizeof (req));
+      req[0] = 0x41;
+      req[1] = 0x01;
+      return gt68xx_device_small_req (dev, req, req);
+    }
+
+  return SANE_STATUS_IO_ERROR;
+}
+
 SANE_Status
 gt6801_setup_scan (GT68xx_Device * dev,
 		   GT68xx_Scan_Request * request,
@@ -298,7 +371,7 @@ gt6801_setup_scan (GT68xx_Device * dev,
   SANE_Fixed x0, y0, xs, ys;
   SANE_Bool backtrack = SANE_FALSE;
 
-  XDBG ((5, "%s: enter (action=%s)\n", function_name,
+  XDBG ((6, "%s: enter (action=%s)\n", function_name,
 	 action == SA_CALIBRATE ? "calibrate" :
 	 action == SA_CALIBRATE_ONE_LINE ? "calibrate one line" :
 	 action == SA_SCAN ? "scan" : "calculate only"));
@@ -324,7 +397,7 @@ gt6801_setup_scan (GT68xx_Device * dev,
 	base_ydpi = model->optical_ydpi;
     }
 
-  XDBG ((5, "%s: base_xdpi=%d, base_ydpi=%d\n", function_name,
+  XDBG ((6, "%s: base_xdpi=%d, base_ydpi=%d\n", function_name,
 	 base_xdpi, base_ydpi));
 
   switch (action)
@@ -379,14 +452,14 @@ gt6801_setup_scan (GT68xx_Device * dev,
   pixel_ys = SANE_UNFIX (ys) * ydpi / MM_PER_INCH + 0.5;
   pixel_xs = SANE_UNFIX (xs) * xdpi / MM_PER_INCH + 0.5;
 
-  XDBG ((5, "%s: xdpi=%d, ydpi=%d\n", function_name, xdpi, ydpi));
-  XDBG ((5, "%s: color=%s, depth=%d\n", function_name,
+  XDBG ((6, "%s: xdpi=%d, ydpi=%d\n", function_name, xdpi, ydpi));
+  XDBG ((6, "%s: color=%s, depth=%d\n", function_name,
 	 color ? "TRUE" : "FALSE", depth));
-  XDBG ((5, "%s: pixel_x0=%d, pixel_y0=%d\n", function_name,
+  XDBG ((6, "%s: pixel_x0=%d, pixel_y0=%d\n", function_name,
 	 pixel_x0, pixel_y0));
-  XDBG ((5, "%s: pixel_xs=%d, pixel_ys=%d\n", function_name,
+  XDBG ((6, "%s: pixel_xs=%d, pixel_ys=%d\n", function_name,
 	 pixel_xs, pixel_ys));
-  XDBG ((5, "%s: backtrack=%d\n", function_name, backtrack));
+  XDBG ((6, "%s: backtrack=%d\n", function_name, backtrack));
 
   color_mode_code = 0x80;	/* What does this mean ? */
   if (color)
@@ -401,7 +474,7 @@ gt6801_setup_scan (GT68xx_Device * dev,
       color_mode_code &= 0x7f;
       color_mode_code |= (1 << 4);
     }
-  XDBG ((5, "%s: color_mode_code = 0x%02X\n", function_name,
+  XDBG ((6, "%s: color_mode_code = 0x%02X\n", function_name,
 	 color_mode_code));
 
   overscan_lines = 0;
@@ -422,7 +495,7 @@ gt6801_setup_scan (GT68xx_Device * dev,
       params->ld_shift_g = ld_shift_g * ydpi / optical_ydpi;
       params->ld_shift_b = ld_shift_b * ydpi / optical_ydpi;
 
-      XDBG ((5, "%s: overscan=%d, ld=%d/%d/%d\n", function_name,
+      XDBG ((6, "%s: overscan=%d, ld=%d/%d/%d\n", function_name,
 	     overscan_lines, params->ld_shift_r, params->ld_shift_g,
 	     params->ld_shift_b));
     }
@@ -436,17 +509,17 @@ gt6801_setup_scan (GT68xx_Device * dev,
 	overscan_lines += (params->ld_shift_double * 3);
       else
 	overscan_lines += params->ld_shift_double;
-      XDBG ((5, "%s: overscan=%d, ld double=%d\n", function_name,
+      XDBG ((6, "%s: overscan=%d, ld double=%d\n", function_name,
 	     overscan_lines, params->ld_shift_double));
     }
 
 
-  XDBG ((5, "%s: base_xdpi=%d, base_ydpi=%d\n", function_name,
+  XDBG ((6, "%s: base_xdpi=%d, base_ydpi=%d\n", function_name,
 	 base_xdpi, base_ydpi));
 
   abs_x0 = pixel_x0 * base_xdpi / xdpi;
   abs_y0 = pixel_y0 * base_ydpi / ydpi;
-  XDBG ((5, "%s: abs_x0=%d, abs_y0=%d\n", function_name, abs_x0, abs_y0));
+  XDBG ((6, "%s: abs_x0=%d, abs_y0=%d\n", function_name, abs_x0, abs_y0));
 
   params->double_column = abs_x0 & 1;
 
@@ -455,7 +528,7 @@ gt6801_setup_scan (GT68xx_Device * dev,
   pixel_align = 32;		/* best case for depth = 16 */
   while ((depth * pixel_align) % (64 * 8) != 0)
     pixel_align *= 2;
-  XDBG ((5, "%s: pixel_align=%d\n", function_name, pixel_align));
+  XDBG ((6, "%s: pixel_align=%d\n", function_name, pixel_align));
 
   if (pixel_xs % pixel_align == 0)
     scan_xs = pixel_xs;
@@ -479,31 +552,31 @@ gt6801_setup_scan (GT68xx_Device * dev,
     abs_ys = 2;
   else
     abs_ys = scan_ys * base_ydpi / ydpi;
-  XDBG ((5, "%s: abs_xs=%d, abs_ys=%d\n", function_name, abs_xs, abs_ys));
+  XDBG ((6, "%s: abs_xs=%d, abs_ys=%d\n", function_name, abs_xs, abs_ys));
 
   if (model->is_cis)
     {
       line_mode = SANE_TRUE;
-      XDBG ((5, "%s: using line mode (CIS)\n", function_name));
+      XDBG ((6, "%s: using line mode (CIS)\n", function_name));
     }
   else
     {
       line_mode = SANE_FALSE;
       if (!color)
 	{
-	  XDBG ((5, "%s: using line mode for monochrome scan\n",
+	  XDBG ((6, "%s: using line mode for monochrome scan\n",
 		 function_name));
 	  line_mode = SANE_TRUE;
 	}
       else if (ydpi >= model->ydpi_force_line_mode)
 	{
-	  XDBG ((5, "%s: forcing line mode for ydpi=%d\n", function_name,
+	  XDBG ((6, "%s: forcing line mode for ydpi=%d\n", function_name,
 		 ydpi));
 	  line_mode = SANE_TRUE;
 	}
       else if (ydpi == 600 && depth == 16)	/* XXX */
 	{
-	  XDBG ((5, "%s: forcing line mode for ydpi=%d, depth=%d\n",
+	  XDBG ((6, "%s: forcing line mode for ydpi=%d, depth=%d\n",
 		 function_name, ydpi, depth));
 	  line_mode = SANE_TRUE;
 	}
@@ -514,7 +587,7 @@ gt6801_setup_scan (GT68xx_Device * dev,
       scan_ys *= 3;
 
 
-  XDBG ((5, "%s: scan_xs=%d, scan_ys=%d\n", function_name, scan_xs, scan_ys));
+  XDBG ((6, "%s: scan_xs=%d, scan_ys=%d\n", function_name, scan_xs, scan_ys));
 
   if (color && !line_mode)
     bits_per_line *= 3;
@@ -528,7 +601,7 @@ gt6801_setup_scan (GT68xx_Device * dev,
 
   if (scan_bpl > 15600 && !line_mode)
     {
-      XDBG ((5, "%s: scan_bpl=%d, trying line mode\n", function_name,
+      XDBG ((6, "%s: scan_bpl=%d, trying line mode\n", function_name,
 	     scan_bpl));
       line_mode = SANE_TRUE;
       if (scan_bpl % 3)
@@ -548,11 +621,11 @@ gt6801_setup_scan (GT68xx_Device * dev,
 
   if (scan_bpl > 15600)
     {
-      XDBG ((3, "%s: scan_bpl=%d, too large\n", function_name, scan_bpl));
+      XDBG ((1, "%s: scan_bpl=%d, too large\n", function_name, scan_bpl));
       return SANE_STATUS_INVAL;
     }
 
-  XDBG ((5, "%s: scan_bpl=%d\n", function_name, scan_bpl));
+  XDBG ((6, "%s: scan_bpl=%d\n", function_name, scan_bpl));
 
 
   if (!request->calculate)
@@ -567,7 +640,7 @@ gt6801_setup_scan (GT68xx_Device * dev,
 
       motor_mode_2 = (request->lamp ? 0 : 1) << 0;
       motor_mode_2 |= (line_mode ? 0 : 1) << 2;
-      XDBG ((5, "%s: motor_mode_1 = 0x%02X, motor_mode_2 = 0x%02X\n",
+      XDBG ((6, "%s: motor_mode_1 = 0x%02X, motor_mode_2 = 0x%02X\n",
 	     function_name, motor_mode_1, motor_mode_2));
 
       /* Fill in the setup command */
@@ -610,7 +683,7 @@ gt6801_setup_scan (GT68xx_Device * dev,
       status = gt68xx_device_req (dev, req, req);
       if (status != SANE_STATUS_GOOD)
 	{
-	  XDBG ((3, "%s: setup request failed: %s\n", function_name,
+	  XDBG ((1, "%s: setup request failed: %s\n", function_name,
 		 sane_strstatus (status)));
 	  return status;
 	}
