@@ -2,7 +2,7 @@
 
    Copyright (C) 2002 Sergey Vlasov <vsu@altlinux.ru>
    AFE offset/gain setting by David Stevenson <david.stevenson@zoom.co.uk>
-   Copyright (C) 2002 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2002 - 2004 Henning Meier-Geinitz <henning@meier-geinitz.de>
 
    This file is part of the SANE package.
    
@@ -52,7 +52,8 @@
 static SANE_Status
 gt68xx_afe_ccd_auto (GT68xx_Scanner * scanner, GT68xx_Scan_Request * request);
 static SANE_Status gt68xx_afe_cis_auto (GT68xx_Scanner * scanner);
-
+static void
+gt68xx_afe_correction (SANE_Byte * value, SANE_Int correction);
 
 SANE_Status
 gt68xx_calibrator_new (SANE_Int width,
@@ -476,7 +477,7 @@ gt68xx_scanner_start_scan_extended (GT68xx_Scanner * scanner,
 {
   DECLARE_FUNCTION_NAME ("gt68xx_scanner_start_scan_extended")
     SANE_Status status;
-  GT68xx_AFE_Parameters *afe = scanner->dev->afe;
+  GT68xx_AFE_Parameters afe = *scanner->dev->afe;
 
   status = gt68xx_scanner_wait_for_positioning (scanner);
   if (status != SANE_STATUS_GOOD)
@@ -518,10 +519,25 @@ gt68xx_scanner_start_scan_extended (GT68xx_Scanner * scanner,
     }
   DBG (5,
        "gt68xx_start_scan_extended: afe: %02X %02X  %02X %02X  %02X %02X\n",
-       afe->r_offset, afe->r_pga, afe->g_offset, afe->g_pga, afe->b_offset,
-       afe->b_pga);
+       afe.r_offset, afe.r_pga, afe.g_offset, afe.g_pga, afe.b_offset,
+       afe.b_pga);
 
-  status = gt68xx_device_set_afe (scanner->dev, afe);
+  if (action == SA_SCAN)
+    {
+
+      gt68xx_afe_correction (&afe.r_offset, scanner->val[OPT_OFFSET].w);
+      gt68xx_afe_correction (&afe.g_offset, scanner->val[OPT_OFFSET].w);
+      gt68xx_afe_correction (&afe.b_offset, scanner->val[OPT_OFFSET].w);
+      gt68xx_afe_correction (&afe.r_pga, scanner->val[OPT_GAIN].w);
+      gt68xx_afe_correction (&afe.b_pga, scanner->val[OPT_GAIN].w);
+      gt68xx_afe_correction (&afe.g_pga, scanner->val[OPT_GAIN].w);
+      DBG (5,
+	   "gt68xx_start_scan_extended: corrected afe: %02X %02X  %02X %02X  %02X %02X\n",
+	   afe.r_offset, afe.r_pga, afe.g_offset, afe.g_pga, afe.b_offset,
+	   afe.b_pga);
+    }
+
+  status = gt68xx_device_set_afe (scanner->dev, &afe);
   if (status != SANE_STATUS_GOOD)
     {
       XDBG ((5, "%s: gt68xx_device_set_afe failed: %s\n", function_name,
@@ -678,8 +694,12 @@ gt68xx_scanner_calibrate (GT68xx_Scanner * scanner,
   req.mds = SANE_TRUE;
   req.mas = SANE_FALSE;
 
+#if 0
   if (scanner->dev->model->is_cis == SANE_TRUE)
     req.color = SANE_TRUE;
+#else
+  req.color = SANE_TRUE;
+#endif
 
   if (req.use_ta)
     {
@@ -709,7 +729,7 @@ gt68xx_scanner_calibrate (GT68xx_Scanner * scanner,
     status = gt68xx_scanner_create_gray_calibrators (scanner, &params);
 
 #if defined(SAVE_WHITE_CALIBRATION) || defined(SAVE_BLACK_CALIBRATION)
-  printf ("P5\n%d %d\n255\n", params.pixel_xs, params.pixel_ys);
+  printf ("P5\n%d %d\n255\n", params.pixel_xs, params.pixel_ys * 3);
 #endif
   for (i = 0; i < params.pixel_ys; ++i)
     {
@@ -761,7 +781,7 @@ gt68xx_scanner_calibrate (GT68xx_Scanner * scanner,
       return status;
     }
 
-  if (!scanner->dev->model->is_cis)
+  if (!scanner->dev->model->is_cis || (scanner->dev->model->flags & GT68XX_FLAG_CIS_LAMP))
     usleep (500000);
   status = gt68xx_scanner_start_scan_extended (scanner, &req, SA_CALIBRATE,
 					       &params);
@@ -881,12 +901,18 @@ gt68xx_scanner_read_line (GT68xx_Scanner * scanner,
 	}
       else
 	{
+#if 0
 	  if (scanner->dev->model->is_cis == SANE_TRUE)
 	    gt68xx_calibrator_process_line (scanner->cal_g,
 					    buffer_pointers[0]);
 	  else
 	    gt68xx_calibrator_process_line (scanner->cal_gray,
 					    buffer_pointers[0]);
+#else
+	  gt68xx_calibrator_process_line (scanner->cal_g,
+					  buffer_pointers[0]);
+#endif
+	
 	}
     }
 
@@ -921,13 +947,15 @@ struct GT68xx_Afe_Values
   SANE_Int scan_dpi;
   SANE_Fixed start_black;
   SANE_Int offset_direction;
+  SANE_Int coarse_black;
+  SANE_Int coarse_white;
 };
 
 #ifndef NDEBUG
 static void
 gt68xx_afe_dump (SANE_String_Const phase, int i, GT68xx_AFE_Parameters * afe)
 {
-  XDBG ((5, "set afe %s %2d: RGB offset/pga: %02x %02x  %02x %02x  "
+  XDBG ((3, "set afe %s %2d: RGB offset/pga: %02x %02x  %02x %02x  "
 	 "%02x %02x\n",
 	 phase, i, afe->r_offset, afe->r_pga, afe->g_offset, afe->g_pga,
 	 afe->b_offset, afe->b_pga));
@@ -939,11 +967,22 @@ static void
 gt68xx_afe_exposure_dump (SANE_String_Const phase, int i,
 			  GT68xx_Exposure_Parameters * exposure)
 {
-  XDBG ((5, "set exposure %s %2d: RGB exposure time: %02x %02x %02x\n",
+  XDBG ((3, "set exposure %s %2d: RGB exposure time: %02x %02x %02x\n",
 	 phase, i, exposure->r_time, exposure->g_time, exposure->b_time));
 }
 #endif /* not NDEBUG */
 
+static void
+gt68xx_afe_correction (SANE_Byte * value, SANE_Int correction)
+{
+  SANE_Int corrected_value = *value + correction;
+
+  if (corrected_value < 0)
+    corrected_value = 0;
+  if (corrected_value > 63)
+    corrected_value = 63;
+  *value = corrected_value;
+}
 /************************************************************************/
 /* CCD scanners                                                         */
 /************************************************************************/
@@ -1033,22 +1072,31 @@ gt68xx_afe_ccd_adjust_offset_gain (GT68xx_Afe_Values * values,
 				   unsigned int *buffer, SANE_Byte * offset,
 				   SANE_Byte * pga)
 {
-  SANE_Int black_low = 3, black_high = 18;
-  SANE_Int white_low = 234, white_high = 252;
+  SANE_Int black_low = values->coarse_black, black_high = black_low + 15;
+  SANE_Int white_high = values->coarse_white, white_low = white_high - 15; 
   SANE_Bool done = SANE_TRUE;
+  SANE_Byte local_pga = *pga;
+  SANE_Byte local_offset = *offset;
 
   gt68xx_afe_ccd_calc (values, buffer);
 
+#if 0
+  /* test all offset values */
+  local_offset++;
+  done = SANE_FALSE;
+  goto finish;
+#endif
+
   if (values->white > white_high)
     {
       if (values->black > black_high)
-	*offset += values->offset_direction;
+	  local_offset += values->offset_direction;
       else if (values->black < black_low)
-	(*pga)--;
+	  local_pga--;
       else
 	{
-	  *offset += values->offset_direction;
-	  (*pga)--;
+	  local_offset += values->offset_direction;
+	  local_pga--;
 	}
       done = SANE_FALSE;
       goto finish;
@@ -1056,13 +1104,13 @@ gt68xx_afe_ccd_adjust_offset_gain (GT68xx_Afe_Values * values,
   else if (values->white < white_low)
     {
       if (values->black < black_low)
-	*offset -= values->offset_direction;
+	local_offset -= values->offset_direction;
       else if (values->black > black_high)
-	(*pga)++;
+	local_pga++;
       else
 	{
-	  *offset -= values->offset_direction;
-	  (*pga)++;
+	  local_offset -= values->offset_direction;
+	  local_pga++;
 	}
       done = SANE_FALSE;
       goto finish;
@@ -1070,13 +1118,13 @@ gt68xx_afe_ccd_adjust_offset_gain (GT68xx_Afe_Values * values,
   if (values->black > black_high)
     {
       if (values->white > white_high)
-	*offset += values->offset_direction;
+	local_offset += values->offset_direction;
       else if (values->white < white_low)
-	(*pga)++;
+	local_pga++;
       else
 	{
-	  *offset += values->offset_direction;
-	  (*pga)++;
+	  local_offset += values->offset_direction;
+	  local_pga++;
 	}
       done = SANE_FALSE;
       goto finish;
@@ -1084,20 +1132,28 @@ gt68xx_afe_ccd_adjust_offset_gain (GT68xx_Afe_Values * values,
   else if (values->black < black_low)
     {
       if (values->white < white_low)
-	*offset -= values->offset_direction;
+	local_offset -= values->offset_direction;
       else if (values->white > white_high)
-	(*pga)--;
+	local_pga--;
       else
 	{
-	  *offset -= values->offset_direction;
-	  (*pga)--;
+	  local_offset -= values->offset_direction;
+	  local_pga--;
 	}
       done = SANE_FALSE;
       goto finish;
     }
  finish:
-  DBG (5, "%swhite=%d, black=%d, offset=%d, gain=%d\n",
-       done ? "DONE: " : "", values->white, values->black, *offset, *pga);
+  if ((local_pga == *pga) && (local_offset == *offset))
+    done = SANE_TRUE;
+
+  DBG (4, "white=%3d, black=%3d, offset=%2d, gain=%2d, old offs=%2d, "
+       "old gain=%2d, total_white=%5d %s\n", values->white, values->black,
+       local_offset, local_pga, *offset, *pga, values->total_white,
+       done ? "DONE " : "");
+
+  *pga = local_pga;
+  *offset = local_offset;
   return done;
 
 }
@@ -1138,7 +1194,8 @@ gt68xx_afe_ccd_auto (GT68xx_Scanner * scanner,
   request.xdpi = 300;
   request.ydpi = 300;
   request.depth = 8;
-  request.color = orig_request->color;
+  /*  request.color = orig_request->color;*/
+  request.color = SANE_TRUE;
   request.mas = SANE_FALSE;
   request.mbs = SANE_FALSE;
   request.mds = SANE_TRUE;
@@ -1174,6 +1231,9 @@ gt68xx_afe_ccd_auto (GT68xx_Scanner * scanner,
     values.start_black = SANE_FIX (20.0);
   else
     values.start_black = scanner->dev->model->x_offset_mark;
+  values.coarse_black = 5;
+  values.coarse_white = 250;
+
   request.mds = SANE_FALSE;
   XDBG ((5, "%s: scan_dpi=%d, calwidth=%d, max_width=%d, "
 	 "start_black=%.1f mm\n", function_name, values.scan_dpi,
@@ -1216,7 +1276,7 @@ gt68xx_afe_ccd_auto (GT68xx_Scanner * scanner,
 
       gt68xx_afe_ccd_calc (&values, buffer_pointers[0]);
 
-      XDBG ((5, "%s: check lamp stable: this white = %d, last white = %d\n",
+      XDBG ((4, "%s: check lamp stable: this white = %d, last white = %d\n",
 	     function_name, values.total_white, last_white));
 
       if (scanner->val[OPT_AUTO_WARMUP].w == SANE_TRUE)
@@ -1263,21 +1323,29 @@ gt68xx_afe_ccd_auto (GT68xx_Scanner * scanner,
 
       if (params.color)
 	{
+	  /* red */
 	  done =
 	    gt68xx_afe_ccd_adjust_offset_gain (&values, buffer_pointers[0],
-						 &afe->r_offset, &afe->r_pga);
+					       &afe->r_offset, &afe->r_pga);
+	  /* green */
 	  done &=
 	    gt68xx_afe_ccd_adjust_offset_gain (&values, buffer_pointers[1],
-						 &afe->g_offset, &afe->g_pga);
+					       &afe->g_offset, &afe->g_pga);
+
+	  /* blue */
 	  done &=
 	    gt68xx_afe_ccd_adjust_offset_gain (&values, buffer_pointers[2],
-						 &afe->b_offset, &afe->b_pga);
+					       &afe->b_offset, &afe->b_pga);
 	}
       else
 	{
+	  if (scanner->dev->model->flags & GT68XX_FLAG_OFFSET_INV)
+	    values.offset_direction = -1;
+	  else
+	    values.offset_direction = 1;
 	  done =
 	    gt68xx_afe_ccd_adjust_offset_gain (&values, buffer_pointers[0],
-						 &afe->g_offset, &afe->g_pga);
+					       &afe->g_offset, &afe->g_pga);
 	}
 
       gt68xx_scanner_stop_scan (scanner);
@@ -1300,6 +1368,7 @@ gt68xx_afe_cis_calc_black (GT68xx_Afe_Values * values,
   SANE_Int end_black;
   SANE_Int i, j;
   SANE_Int min_black = 255;
+  SANE_Int average = 0;
 
   start_black = 0;
   end_black = values->calwidth;
@@ -1311,11 +1380,14 @@ gt68xx_afe_cis_calc_black (GT68xx_Afe_Values * values,
       for (j = 0; j < values->callines; j++)
 	avg_black += (*(black_buffer + i + j * values->calwidth) >> 8);
       avg_black /= values->callines;
+      average += avg_black;
       if (avg_black < min_black)
 	min_black = avg_black;
     }
   values->black = min_black;
-  XDBG ((5, "%s: min_black=%02x\n", function_name, values->black));
+  average /= (end_black - start_black);
+  XDBG ((4, "%s: min_black=0x%02x, average_black=0x%02x\n", function_name, values->black,
+	 average));
 }
 
 static void
@@ -1329,19 +1401,25 @@ gt68xx_afe_cis_calc_white (GT68xx_Afe_Values * values,
 
   start_white = 0;
   end_white = values->calwidth;
+  values->total_white = 0;
 
   /* find max average white value */
   for (i = start_white; i < end_white; ++i)
     {
       SANE_Int avg_white = 0;
       for (j = 0; j < values->callines; j++)
-	avg_white += (*(white_buffer + i + j * values->calwidth) >> 8);
+	{
+	  avg_white += (*(white_buffer + i + j * values->calwidth) >> 8);
+	  values->total_white += (*(white_buffer + i + j * values->calwidth));
+	}
       avg_white /= values->callines;
       if (avg_white > max_white)
 	max_white = avg_white;
     }
   values->white = max_white;
-  XDBG ((5, "%s: max_white=%02x\n", function_name, values->white));
+  values->total_white /= (values->callines * (end_white - start_white));
+  XDBG ((4, "%s: max_white=0x%02x, average_white=0x%02x\n", function_name, values->white,
+	 values->total_white >> 8));
 }
 
 static SANE_Bool
@@ -1350,7 +1428,7 @@ gt68xx_afe_cis_adjust_offset (GT68xx_Afe_Values * values,
 			      SANE_Byte * offset)
 {
   SANE_Int offs = 0, tmp_offset = *offset;
-  SANE_Int low = 8, high = 22;
+  SANE_Int low = values->coarse_black, high = low + 15;
 
   gt68xx_afe_cis_calc_black (values, black_buffer);
   if (values->black < low)
@@ -1358,18 +1436,18 @@ gt68xx_afe_cis_adjust_offset (GT68xx_Afe_Values * values,
       offs = (values->offset_direction * (low - values->black) / 4);
       if (offs == 0)
 	offs = values->offset_direction;
-      DBG (5, "black = %d (too low) --> offs = %d\n", values->black, offs);
+      DBG (4, "black = %d (too low) --> offs = %d\n", values->black, offs);
     }
   else if (values->black > high)
     {
       offs = -(values->offset_direction * (values->black - high) / 7);
       if (offs == 0)
 	offs = -values->offset_direction;
-      DBG (5, "black = %d (too high) --> offs = %d\n", values->black, offs);
+      DBG (4, "black = %d (too high) --> offs = %d\n", values->black, offs);
     }
   else
     {
-      DBG (5, "black = %d (ok)\n", values->black);
+      DBG (4, "black = %d (ok)\n", values->black);
     }
 
   if (offs == 0)
@@ -1389,22 +1467,23 @@ gt68xx_afe_cis_adjust_gain (GT68xx_Afe_Values * values,
 			    unsigned int *white_buffer, SANE_Byte * gain)
 {
   SANE_Int g = *gain;
+  SANE_Int white_high = values->coarse_white, white_low = white_high - 15; 
 
   gt68xx_afe_cis_calc_white (values, white_buffer);
 
-  if (values->white < 235)
+  if (values->white < white_low)
     {
       g += 1;
-      DBG (5, "white = %d (too low) --> gain += 1\n", values->white);
+      DBG (4, "white = %d (too low) --> gain += 1\n", values->white);
     }
-  else if (values->white > 250)
+  else if (values->white > white_high)
     {
       g -= 1;
-      DBG (5, "white = %d (too high) --> gain -= 1\n", values->white);
+      DBG (4, "white = %d (too high) --> gain -= 1\n", values->white);
     }
   else
     {
-      DBG (5, "white = %d (ok)\n", values->white);
+      DBG (4, "white = %d (ok)\n", values->white);
     }
   if (g < 0)
     g = 0;
@@ -1426,20 +1505,20 @@ gt68xx_afe_cis_adjust_exposure (GT68xx_Afe_Values * values,
   if (values->white < border)
     {
       *exposure_time += ((border - values->white) * 2);
-      DBG (5, "white = %d (too low) --> += %d\n",
+      DBG (4, "white = %d (too low) --> += %d\n",
 	   values->white, ((border - values->white) * 2));
       return SANE_FALSE;
     }
   else if (values->white > border + 10)
     {
       *exposure_time -= ((values->white - (border + 10)) * 2);
-      DBG (5, "white = %d (too high) --> -= %d\n",
+      DBG (4, "white = %d (too high) --> -= %d\n",
 	   values->white, ((values->white - (border + 10)) * 2));
       return SANE_FALSE;
     }
   else
     {
-      DBG (5, "white = %d (ok)\n", values->white);
+      DBG (4, "white = %d (ok)\n", values->white);
     }
   return SANE_TRUE;
 }
@@ -1501,6 +1580,8 @@ gt68xx_afe_cis_read_lines (GT68xx_Afe_Values * values,
   values->scan_dpi = params.xdpi;
   values->calwidth = params.pixel_xs;
   values->callines = params.pixel_ys;
+  values->coarse_black = 5;
+  values->coarse_white = 250;
 
   if (r_buffer && g_buffer && b_buffer)
     for (line = 0; line < values->callines; line++)
@@ -1557,6 +1638,49 @@ gt68xx_afe_cis_auto (GT68xx_Scanner * scanner)
   if (!r_buffer || !g_buffer || !b_buffer)
     return SANE_STATUS_NO_MEM;
 
+  if (scanner->dev->model->flags & GT68XX_FLAG_CIS_LAMP)
+    {
+      GT68xx_Afe_Values lamp_values = values;
+      SANE_Int last_white = -21;
+      SANE_Word i;
+      
+      lamp_values.callines = 1;
+      
+      /* loop waiting for lamp to give stable brightness */
+      for (i = 0; i < 80; i++)
+	{
+	  usleep (200000);
+	  if (i == 10)
+	    DBG (0, "Please wait for lamp warm-up\n");
+	  
+	  /* read line */
+	  RIE (gt68xx_afe_cis_read_lines (&lamp_values, scanner, SANE_TRUE, SANE_FALSE,
+					  r_buffer, g_buffer, b_buffer));
+	  
+	  gt68xx_afe_cis_calc_white (&lamp_values, r_buffer);
+	  
+	  XDBG ((4, "%s: check lamp stable: this white = 0x%04X, last white =0x%04X\n",
+		 function_name, lamp_values.total_white, last_white));
+	  
+	  if (scanner->val[OPT_AUTO_WARMUP].w == SANE_TRUE)
+	    {
+	      if (lamp_values.total_white <= (last_white + 20))
+		break;		/* lamp is warmed up */
+	    }
+	  else
+	    {			/* insist on 30 seconds */
+	      struct timeval now;
+	      int secs;
+	      
+	      gettimeofday (&now, 0);
+	      secs = now.tv_sec - scanner->lamp_on_time.tv_sec;
+	      if (secs >= WARMUP_TIME)
+		break;
+	    }
+	  last_white = lamp_values.total_white;
+	}
+    }
+
   total_count = 0;
   do
     {
@@ -1602,6 +1726,9 @@ gt68xx_afe_cis_auto (GT68xx_Scanner * scanner)
     }
   while (total_count < 100 && !done);
 
+  if (total_count >= 100)
+    XDBG ((0, "%s: setting AFE reached limit\n", function_name));
+
   IF_DBG (gt68xx_afe_dump ("final", total_count, afe));
 
   /* Exposure time */
@@ -1624,6 +1751,9 @@ gt68xx_afe_cis_auto (GT68xx_Scanner * scanner)
     }
   while (!done && exposure_count < 10);
 
+  if (exposure_count >= 10)
+    XDBG ((0, "%s: setting exposure reached limit\n", function_name));
+
   free (r_buffer);
   free (g_buffer);
   free (b_buffer);
