@@ -43,9 +43,38 @@
    HP Scanner Control Language (SCL).
 */
 
-static char *hp_backend_version = "1.01";
+static char *hp_backend_version = "1.06";
+static char *hp_backend_revision = "$Revision: 1.20 $";
 /* Changes:
 
+   V 1.06:
+   $Log: hp.c,v $
+   Revision 1.20  2004/03/27 13:52:39  kig-guest
+   Keep USB-connection open (was problem with Linux 2.6.x)
+
+
+   V 1.05:
+   Revision 1.19  2003/10/24 17:26:07  kig-guest
+   Use new sanei-thread-interface
+
+   Revision 1.18  2003/10/09 19:37:29  kig-guest
+   Redo when TEST UNIT READY failed
+   Redo when read returns with 0 bytes (non-SCSI only)
+   Bug #300241: fix invers image on 3c/4c/6100C at 10 bit depth
+
+   Revision 1.17  2003/10/06 19:54:07  kig-guest
+   Bug #300248: correct "Negatives" to "Negative" in option description
+ 
+
+   V 1.04, 24-Jul-2003, PK (peter@kirchgessner.net)
+      - Add internationalization
+
+   V 1.03, 14-Apr-2003, PK (peter@kirchgessner.net)
+      - check valp in call of sane_control_option()
+
+   V 1.02, 02-Feb-2003, PK (peter@kirchgessner.net)
+      - add OS/2-support by Franz Bakan
+
    V 1.01, 06-Dec-2002, PK (peter@kirchgessner.net)
       - add option dumb-read to work around problems
         with BusLogic SCSI driver (error during device I/O)
@@ -203,6 +232,7 @@ static char *hp_backend_version = "1.01";
 #include "sane/sanei_config.h"
 #include "sane/sanei_backend.h"
 #include "sane/sanei_usb.h"
+#include "sane/sanei_thread.h"
 /* #include <sane/sanei_debug.h> */
 #include "hp-device.h"
 #include "hp-handle.h"
@@ -617,8 +647,8 @@ hp_read_config (void)
   is_df_config = 1;
   cu_device[0] = '\0';
 
-  DBG(1, "hp_read_config: hp backend v%s starts reading config file\n",
-      hp_backend_version);
+  DBG(1, "hp_read_config: hp backend v%s/%s starts reading config file\n",
+      hp_backend_version, hp_backend_revision);
 
   if ((fp = sanei_config_open(HP_CONFIG_FILE)) != 0)
     {
@@ -777,7 +807,9 @@ sane_init (SANE_Int *version_code, SANE_Auth_Callback UNUSEDARG authorize)
 
   DBG_INIT();
   DBG(3, "sane_init called\n");
+  sanei_thread_init ();
 
+  hp_init_openfd ();
   hp_destroy();
 
   if (version_code)
