@@ -42,7 +42,7 @@
 */
 
 /* 
-	$Id: leo.h,v 1.2 2002/05/29 04:51:21 fzago Exp $
+	$Id: leo.h,v 1.3 2003/09/12 16:10:33 hmg-guest Exp $
 */
 
 /* Commands supported by the scanner. */
@@ -240,14 +240,6 @@ enum Leo_Option
   OPT_NUM_OPTIONS
 };
 
-typedef union
-{
-  SANE_Word w;			/* word */
-  SANE_Word *wa;		/* word array */
-  SANE_String s;		/* string */
-}
-Option_Value;
-
 /*--------------------------------------------------------------------------*/
 
 /* 
