@@ -75,14 +75,16 @@
 #include "../include/sane/sanei.h"
 #include "../include/sane/saneopts.h"
 
-#include "mustek_pp.h"
-#include "mustek_pp_drivers.h"
-
 #include "../include/sane/sanei_backend.h"
 
 #include "../include/sane/sanei_config.h"
 #define MUSTEK_PP_CONFIG_FILE "mustek_pp.conf"
 
+#include "../include/sane/sanei_pa4s2.h"
+
+#include "mustek_pp.h"
+#include "mustek_pp_drivers.h"
+
 #define MIN(a,b)	((a) < (b) ? (a) : (b))
    
 /* converts millimeter to pixels at a given resolution */
@@ -92,8 +94,8 @@
 
 /* if you change the source, please set MUSTEK_PP_STATE to "devel". Do *not*
  * change the MUSTEK_PP_BUILD. */
-#define MUSTEK_PP_BUILD	12
-#define MUSTEK_PP_STATE	"alpha"
+#define MUSTEK_PP_BUILD	13
+#define MUSTEK_PP_STATE	"beta"
 
 
 /* auth callback... since basic user authentication is done by saned, this
@@ -220,11 +222,25 @@ do_stop(Mustek_pp_Handle *hndl)
  *
  * Description:
  *	just exit... reader_process takes care that nothing bad will happen
+ *
+ * EDG - Jan 14, 2004:
+ *      Make sure that the parport is released again by the child process
+ *      under all circumstances, because otherwise the parent process may no 
+ *      longer be able to claim it (they share the same file descriptor, and
+ *      the kernel doesn't release the child's claim because the file
+ *      descriptor isn't cleaned up). If that would happen, the lamp may stay
+ *      on and may not return to its home position, unless the scanner
+ *      frontend is restarted.
+ *      (This happens only when sanei_pa4s2 uses libieee1284 AND
+ *      libieee1284 goes via /dev/parportX).
+ *
  */
+static int fd_to_release = 0;
 /*ARGSUSED*/
 static RETSIGTYPE
 sigterm_handler (int signal __UNUSED__)
 {
+	sanei_pa4s2_enable(fd_to_release, SANE_FALSE);
 	_exit (SANE_STATUS_GOOD);
 }
 
@@ -257,14 +273,15 @@ reader_process (Mustek_pp_Handle * hndl, int pipe)
 	
 	if (!(fp = fdopen(pipe, "w")))
 		return SANE_STATUS_IO_ERROR;
-
-	if ((status = hndl->dev->func->start (hndl)) != SANE_STATUS_GOOD)
-		return status;
-
+        
+	fd_to_release = hndl->fd;
 	memset (&act, 0, sizeof(act));
 	act.sa_handler = sigterm_handler;
 	sigaction (SIGTERM, &act, NULL);
 
+	if ((status = hndl->dev->func->start (hndl)) != SANE_STATUS_GOOD)
+		return status;
+
         size = hndl->params.bytes_per_line;
   	elem = 1;
 
@@ -620,36 +637,54 @@ static void
 attach_device(SANE_String *driver, SANE_String *name, 
               SANE_String *port, SANE_String *option_ta)
 {
-   int found = 0, driver_no;
+  int found = 0, driver_no, port_no;
+  const char **ports;
 
-   for (driver_no=0 ; driver_no<MUSTEK_PP_NUM_DRIVERS ; driver_no++)
-     {
-       if (strcasecmp (Mustek_pp_Drivers[driver_no].driver, *driver) == 0)
-	 {
-	   Mustek_pp_Drivers[driver_no].init (
-	     (*option_ta == 0 ? CAP_NOTHING : CAP_TA),
-	     *port, *name, sane_attach);
-	   found = 1;
-	   break;
-	 }
-     }
+  if (!strcmp (*port, "*"))
+    {
+      ports = sanei_pa4s2_devices();
+      DBG (3, "sanei_init: auto probing port\n");
+    }
+  else
+    {
+      ports = malloc (sizeof(char *) * 2);
+      ports[0] = *port;
+      ports[1] = NULL;
+    }
 
-   if (found == 0)
-     {
-       DBG (1, "sane_init: invalid driver name %s\n", *driver);
-     }
+  for (port_no=0; ports[port_no] != NULL; port_no++)
+    {
+      for (driver_no=0 ; driver_no<MUSTEK_PP_NUM_DRIVERS ; driver_no++)
+        {
+          if (strcasecmp (Mustek_pp_Drivers[driver_no].driver, *driver) == 0)
+   	     {
+   	       Mustek_pp_Drivers[driver_no].init (
+   	         (*option_ta == 0 ? CAP_NOTHING : CAP_TA),
+   	         ports[port_no], *name, sane_attach);
+   	       found = 1;
+   	       break;
+   	     }
+        }
+    }
 
-   free (*name);
-   free (*port);
-   free (*driver);
-   if (*option_ta)
-     free (*option_ta);
-   *name = *port = *driver = *option_ta = 0;
-   
-   /* In case of a successful initialization, the configuration options
-      should have been transfered to the device, but this function can
-      deal with that. */
-   free_cfg_options(&numcfgoptions, &cfgoptions);
+  free (ports);
+
+  if (found == 0)
+    {
+      DBG (1, "sane_init: invalid driver name ``%s''\n", *driver);
+    }
+
+  free (*name);
+  free (*port);
+  free (*driver);
+  if (*option_ta)
+    free (*option_ta);
+  *name = *port = *driver = *option_ta = 0;
+  
+  /* In case of a successful initialization, the configuration options
+     should have been transfered to the device, but this function can
+     deal with that. */
+  free_cfg_options(&numcfgoptions, &cfgoptions);
 }
    
 /* sane_init:
@@ -702,21 +737,28 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   if (fp == NULL)
     {
       char driver_name[64];
+      const char **devices = sanei_pa4s2_devices();
+      int device_no;
 	
       DBG (2, "sane_init: could not open configuration file\n");
       
-      for (driver_no=0 ; driver_no<MUSTEK_PP_NUM_DRIVERS ; driver_no++)
-	{
-	  Mustek_pp_Drivers[driver_no].init(CAP_NOTHING, "0x378",
-		    Mustek_pp_Drivers[driver_no].driver, sane_attach);
+      for (device_no = 0; devices[device_no] != NULL; device_no++)
+        {
+	  DBG (3, "sane_init: trying ``%s''\n", devices[device_no]);
+          for (driver_no=0 ; driver_no<MUSTEK_PP_NUM_DRIVERS ; driver_no++)
+	    {
+	      Mustek_pp_Drivers[driver_no].init(CAP_NOTHING, devices[device_no],
+	  	        Mustek_pp_Drivers[driver_no].driver, sane_attach);
 
-	  snprintf (driver_name, 64, "%s-ta",
-		Mustek_pp_Drivers[driver_no].driver);
+	      snprintf (driver_name, 64, "%s-ta",
+		    Mustek_pp_Drivers[driver_no].driver);
 
-	  Mustek_pp_Drivers[driver_no].init(CAP_TA, "0x378",
-		    driver_name, sane_attach);
+	      Mustek_pp_Drivers[driver_no].init(CAP_TA, devices[device_no],
+		        driver_name, sane_attach);
+	    }
 	}
 
+      free (devices);
       return SANE_STATUS_GOOD;
     }
 
