@@ -89,13 +89,13 @@
 #if defined (HAVE_GETADDRINFO) && defined (HAVE_GETNAMEINFO)
 # define NET_USES_AF_INDEP
 # ifdef ENABLE_IPV6
-#  define NET_VERSION "1.0.11 (AF-indep+IPv6)"
+#  define NET_VERSION "1.0.12 (AF-indep+IPv6)"
 # else
-#  define NET_VERSION "1.0.11 (AF-indep)"
+#  define NET_VERSION "1.0.12 (AF-indep)"
 # endif /* ENABLE_IPV6 */
 #else
 # undef ENABLE_IPV6
-# define NET_VERSION "1.0.11"
+# define NET_VERSION "1.0.12"
 #endif /* HAVE_GETADDRINFO && HAVE_GETNAMEINFO */
 
 static SANE_Auth_Callback auth_callback;
@@ -305,7 +305,7 @@ connect_dev (Net_Device * dev)
 	{
 	  DBG (1, "connect_dev: [%d] don't know how to deal with addr family %d\n",
 	       i, addrp->ai_family);
-	  break;
+	  continue;
 	}
       
       dev->ctl = socket (addrp->ai_family, SOCK_STREAM, 0);
@@ -314,14 +314,14 @@ connect_dev (Net_Device * dev)
 	  DBG (1, "connect_dev: [%d] failed to obtain socket (%s)\n",
 	       i, strerror (errno));
 	  dev->ctl = -1;
-	  break;
+	  continue;
 	}
 
       if (connect (dev->ctl, addrp->ai_addr, addrp->ai_addrlen) < 0)
 	{
 	  DBG (1, "connect_dev: [%d] failed to connect (%s)\n", i, strerror (errno));
 	  dev->ctl = -1;
-	  break;
+	  continue;
 	}
       DBG (3, "connect_dev: [%d] connection succeeded (%s)\n", i, (addrp->ai_family == AF_INET6) ? "IPv6" : "IPv4");
       dev->addr_used = addrp;
@@ -468,7 +468,7 @@ static SANE_Status
 fetch_options (Net_Scanner * s)
 {
   int option_number;
-  DBG (3, "fetch_options: %p\n", s);
+  DBG (3, "fetch_options: %p\n", (void *) s);
 
   if (s->opt.num_options)
     {
@@ -544,7 +544,7 @@ fetch_options (Net_Scanner * s)
 static SANE_Status
 do_cancel (Net_Scanner * s)
 {
-  DBG (2, "do_cancel: %p\n", s);
+  DBG (2, "do_cancel: %p\n", (void *) s);
   s->hw->auth_active = 0;
   if (s->data >= 0)
     {
@@ -563,7 +563,7 @@ do_authorization (Net_Device * dev, SANE_String resource)
   SANE_Char password[SANE_MAX_PASSWORD_LEN];
   char *net_resource;
 
-  DBG (2, "do_authorization: dev=%p resource=%s\n", dev, resource);
+  DBG (2, "do_authorization: dev=%p resource=%s\n", (void *) dev, resource);
 
   dev->auth_active = 1;
 
@@ -632,8 +632,8 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 
   DBG_INIT ();
 
-  DBG (2, "sane_init: authorize = %p, version_code = %p\n", authorize,
-       version_code);
+  DBG (2, "sane_init: authorize = %p, version_code = %p\n", (void *) authorize,
+       (void *) version_code);
 
   devlist = 0;
   first_device = 0;
@@ -773,7 +773,7 @@ sane_exit (void)
     {
       next_device = dev->next;
 
-      DBG (2, "sane_exit: closing dev %p, ctl=%d\n", dev, dev->ctl);
+      DBG (2, "sane_exit: closing dev %p, ctl=%d\n", (void *) dev, dev->ctl);
 
       if (dev->ctl >= 0)
 	{
@@ -1718,7 +1718,7 @@ sane_read (SANE_Handle handle, SANE_Byte * data, SANE_Int max_length,
   int is_even;
 
   DBG (3, "sane_read: handle=%p, data=%p, max_length=%d, length=%p\n",
-       handle, data, max_length, length);
+       handle, data, max_length, (void *) length);
   if (!length)
     {
       DBG (1, "sane_read: length == NULL\n");
