@@ -7,7 +7,7 @@
  *  @brief Scanning...
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2003 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2004 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -21,6 +21,8 @@
  *        - fixed cancel hang problem
  *        - fixed CIS PhyBytes adjustment
  *        - removed CanoScan specific setting stuff
+ * - 0.46 - fixed problem in usb_SetScanParameters()
+ * - 0.47 - no changes
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -63,6 +65,10 @@
  * <hr>
  */
 
+/** array used to get motor-settings and mclk-settings
+ */
+static int dpi_ranges[] = {	75,100,150,200,300,400,600,800,1200,2400 };
+
 static u_char     bMaxITA;
 
 static SANE_Bool  m_fAutoPark;
@@ -1155,15 +1161,17 @@ static SANE_Bool usb_SetScanParameters( pPlustek_Device dev, pScanParam pParam )
 		return SANE_FALSE;
 
 	a_bRegs[0x07] = 0;
+	a_bRegs[0x28] = 0;
 
 	/* set unused registers to 0 */
 	memset( &a_bRegs[0x03], 0, 3 );
 	memset( &a_bRegs[0x5f], 0, 0x7f-0x5f+1 );
 
 	/* set the merlin registers */
+	_UIO(sanei_lm983x_write( dev->fd, 0x03, &a_bRegs[0x03], 3, SANE_TRUE));
 	_UIO(sanei_lm983x_write( dev->fd, 0x08, &a_bRegs[0x08], 0x7f - 0x08+1, SANE_TRUE));
 
-    usleep( 50 );
+    usleep( 100 );
 	
 	if( !usbio_WriteReg( dev->fd, 0x07, 0 ))
 		return SANE_FALSE;
