@@ -7,7 +7,7 @@
  *  @brief Calibration routines.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2003 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2004 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -20,6 +20,10 @@
  * - 0.45 - added coarse calibration for CIS devices
  *        - added _WAF_SKIP_FINE to skip the results of fine calibration
  *        - CanoScan fixes and fine-tuning
+ * - 0.46 - CanoScan will now be calibrated by code in plustek-usbcal.c
+ *        - added functions to save and restore calibration data from a file
+ *        - fixed some TPA issues
+ * - 0.47 - made calibration work on big-endian machines
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -69,8 +73,8 @@
 #define _MAX_SHAD       0x4000
 #define _SHADING_BUF	(_MAX_SHAD*3)	 /**< max size of the shading buffer */
 
-#define _CIS_GAIN	0
-#define _CIS_OFFS	6
+#define SWAP_COARSE
+#define SWAP_FINE
 
 static u_short a_wWhiteShading[_SHADING_BUF] = {0};
 static u_short a_wDarkShading[_SHADING_BUF]  = {0};
@@ -129,7 +133,11 @@ static void usb_line_statistics( char *cmt, u_short* buf,
 
 		for( dw = 0; dw < dim_x; pvd++, dw++ ) {
 
+#ifdef SWAP_FINE
+			tmp = pvd->Mono;
+#else
 			tmp = _LOBYTE(pvd->Mono) * 256 + _HIBYTE(pvd->Mono);
+#endif
 
 			if( tmp > mad ) {
 				mad  = tmp;
@@ -150,7 +158,11 @@ static void usb_line_statistics( char *cmt, u_short* buf,
 
 		for( dw = 0; dw < dim_x; pvd2++, dw++ ) {
 
+#ifdef SWAP_FINE
+			tmp = pvd2->Mono;
+#else
 			tmp = _LOBYTE(pvd2->Mono) * 256 + _HIBYTE(pvd2->Mono);
+#endif
 
 			if( tmp > ubd ) {
 				cud++;
@@ -301,6 +313,15 @@ static void usb_GetSoftwareOffsetGain( pPlustek_Device dev )
 
 	switch( sCaps->bCCD ) {
 
+	case kEPSON:
+		DBG( _DBG_INFO2, "kEPSON adjustments\n" );
+#if 0
+		pParam->swGain[0] = 800;
+		pParam->swGain[1] = 800;
+		pParam->swGain[2] = 800;
+#endif
+		break;
+	
 	case kNECSLIM:
 		DBG( _DBG_INFO2, "kNECSLIM adjustments\n" );
 		if( pParam->PhyDpi.x <= 150 ) {
@@ -330,56 +351,38 @@ static void usb_GetSoftwareOffsetGain( pPlustek_Device dev )
 	case kNEC8861:
 		DBG( _DBG_INFO2, "kNEC8861 adjustments\n" );
 		break;
-
+#if 0
 	case kCIS650:
-		DBG( _DBG_INFO2, "kCIS650 adjustments\n" );
-		pParam->swGain[0] = 1160;
-		pParam->swGain[1] = 1160;
-		pParam->swGain[2] = 1160;
-		break;
-
 	case kCIS670:
-		DBG( _DBG_INFO2, "kCIS670 adjustments\n" );
-		if(pParam->bDataType == SCANDATATYPE_Color) {
-			pParam->swOffset[0] = -2650;
-			pParam->swOffset[1] = -2800;
-			pParam->swOffset[2] = -2850;
+	case kCIS1220:
 
-			pParam->swGain[0] = 1150;
-			pParam->swGain[1] = 1150;
-			pParam->swGain[2] = 1150;
-		} else {
-			pParam->swOffset[0] = -1500;
-			pParam->swOffset[1] = -1500;
-			pParam->swOffset[2] = -1500;
-			
-			pParam->swGain[0] = 900;
-			pParam->swGain[1] = 900;
-			pParam->swGain[2] = 900;
+		DBG( _DBG_INFO2, "kCIS adjustments\n" );
+		if(pParam->bDataType == SCANDATATYPE_Color) {
+
+			pParam->swGain[0] =
+			pParam->swGain[1] =
+			pParam->swGain[2] = 952;
+
+			pParam->swOffset[0] =
+			pParam->swOffset[1] =
+			pParam->swOffset[2] = 1000;
 		}
 		break;
-
 	case kCIS1240:
 		DBG( _DBG_INFO2, "kCIS1240 adjustments\n" );
 		if(pParam->bDataType == SCANDATATYPE_Color) {
-			pParam->swOffset[0] = -1650;
-			pParam->swOffset[1] = -1500;
-			pParam->swOffset[2] = -1500;
 
-			pParam->swGain[0] = 1010;
-			pParam->swGain[1] = 1050;
-			pParam->swGain[2] = 1030;
-		} else {
-			pParam->swOffset[0] = -1000;
-			pParam->swOffset[1] = -1000;
-			pParam->swOffset[2] = -1000;
+			pParam->swGain[0] = 950;
+			pParam->swGain[1] = 950;
+			pParam->swGain[2] = 900;
 
-			pParam->swGain[0] = 1100;
-			pParam->swGain[1] = 1100;
-			pParam->swGain[2] = 1100;
-		}	
+			pParam->swOffset[0] =
+			pParam->swOffset[1] =
+			pParam->swOffset[2] = 0; /*1000;*/
+		}
 		break;
-		
+#endif
+
 	case kNEC3799:
 		DBG( _DBG_INFO2, "kNEC3799 adjustments\n" );
 		if( sCaps->bPCB == 2 ) {
@@ -554,19 +557,9 @@ static void usb_GetSoftwareOffsetGain( pPlustek_Device dev )
 	}
 }
 
-/** as the name says..
- *
- */
-static void usb_Swap( u_short *pw, u_long dwBytes )
-{
-	for( dwBytes /= 2; dwBytes--; pw++ )
-		_SWAP(((u_char*) pw)[0], ((u_char*)pw)[1]);
-}
-
 /** according to the pixel values,
- *
  */
-static u_char usb_GetNewGain( u_short wMax )
+static u_char usb_GetNewGain( u_short wMax, int channel )
 {
 	double dRatio, dAmp;
 	u_long dwInc, dwDec;
@@ -575,7 +568,7 @@ static u_char usb_GetNewGain( u_short wMax )
 	if( !wMax )
 		wMax = 1;
 
-	dAmp = 0.93 + 0.067 * a_bRegs[0x3d];
+	dAmp = 0.93 + 0.067 * a_bRegs[0x3b+channel];
 
 	if((m_dwIdealGain / (wMax / dAmp)) < 3) {
 
@@ -587,9 +580,9 @@ static u_char usb_GetNewGain( u_short wMax )
 		dwDec = (u_long)((0.93 + floor (dRatio) * 0.067) * wMax / dAmp);
 		if((dwInc >= 0xff00) ||
 		   (labs (dwInc - m_dwIdealGain) > labs(dwDec - m_dwIdealGain))) {
-			bGain = (u_char)floor (dRatio);
+			bGain = (u_char)floor(dRatio);
 		} else {
-			bGain = ceil(dRatio);
+			bGain = (u_char)ceil(dRatio);
         }
 
 		if( bGain > 0x3f )
@@ -693,14 +686,6 @@ static SANE_Bool usb_AdjustGain( pPlustek_Device dev, int fNegative )
 	bMaxITA = 0xff;
 
 	DBG( _DBG_INFO2, "usb_AdjustGain()\n" );
-	if( scaps->workaroundFlag & _WAF_FIX_GAIN ) {
-		a_bRegs[0x3b] =
-		a_bRegs[0x3c] =
-		a_bRegs[0x3d] = _CIS_GAIN;
-
-		/* don't blame on me - I know it's shitty... */
-		goto show_sets;
-	}
 	
 	/*
      * define the strip to scan for coarse calibration
@@ -709,7 +694,11 @@ static SANE_Bool usb_AdjustGain( pPlustek_Device dev, int fNegative )
 	m_ScanParam.Size.dwLines  = 1;				/* for gain */
 	m_ScanParam.Size.dwPixels = scaps->Normal.Size.x *
 								scaps->OpticDpi.x / 300UL;
-
+/*for TPA tests*/
+#if 0
+	m_ScanParam.Size.dwPixels = dev->usbDev.pSource->Size.x *
+								scaps->OpticDpi.x / 300UL;
+#endif
 	m_ScanParam.Size.dwBytes  = m_ScanParam.Size.dwPixels *
 								2 * m_ScanParam.bChannels;
 
@@ -720,6 +709,11 @@ static SANE_Bool usb_AdjustGain( pPlustek_Device dev, int fNegative )
 
 	m_ScanParam.Origin.x = (u_short)((u_long) hw->wActivePixelsStart *
 													300UL / scaps->OpticDpi.x);
+/*for TPA tests*/
+#if 0
+	m_ScanParam.Origin.x = (u_short)((u_long)dev->usbDev.pSource->DataOrigin.x *
+													300UL / scaps->OpticDpi.x);
+#endif
 	m_ScanParam.bCalibration = PARAM_Gain;
 
 	DBG( _DBG_INFO2, "Coarse Calibration Strip:\n" );
@@ -755,7 +749,10 @@ TOGAIN:
 	dumpPicInit( &m_ScanParam, tmp );
 	dumpPic( tmp, pScanBuffer, m_ScanParam.Size.dwPhyBytes );
 		
-	usb_Swap((u_short *)pScanBuffer, m_ScanParam.Size.dwPhyBytes );
+#ifdef SWAP_COARSE
+	if(usb_HostSwap())
+#endif
+		usb_Swap((u_short *)pScanBuffer, m_ScanParam.Size.dwPhyBytes );
 
 	if( fNegative ) {
 
@@ -767,7 +764,7 @@ TOGAIN:
 
 			rgb.Red = rgb.Green = rgb.Blue = dwGrayMax = 0;
 
-			for( dw = 0; dw < dwLoop;) {
+			for( dw = 0; dw < dwLoop; ) {
 
 				rgbSum.Red = rgbSum.Green = rgbSum.Blue = 0;
 				for( dw10 = 20; dw10--; dw++ ) {
@@ -791,9 +788,9 @@ TOGAIN:
 			Gain_Hilight.Red   = (u_short)(rgb.Red / 20UL);
 			Gain_Hilight.Green = (u_short)(rgb.Green / 20UL);
 			Gain_Hilight.Blue  = (u_short)(rgb.Blue / 20UL);
-			a_bRegs[0x3b] = usb_GetNewGain(Gain_Hilight.Red);
-			a_bRegs[0x3c] = usb_GetNewGain(Gain_Hilight.Green);
-			a_bRegs[0x3d] = usb_GetNewGain(Gain_Hilight.Blue);
+			a_bRegs[0x3b] = usb_GetNewGain(Gain_Hilight.Red,   0 );
+			a_bRegs[0x3c] = usb_GetNewGain(Gain_Hilight.Green, 1 );
+			a_bRegs[0x3d] = usb_GetNewGain(Gain_Hilight.Blue,  2 );
 
 		} else {
 
@@ -815,8 +812,9 @@ TOGAIN:
 			Gain_Hilight.Red  = Gain_Hilight.Green =
 			Gain_Hilight.Blue = (u_short)(dwMax / 20UL);
 
-			Gain_Reg.Red  = Gain_Reg.Green = Gain_Reg.Blue =
-			a_bRegs[0x3b] = a_bRegs[0x3c]  = a_bRegs[0x3d] = usb_GetNewGain(Gain_Hilight.Green);
+			Gain_Reg.Red  = Gain_Reg.Green =
+			Gain_Reg.Blue = a_bRegs[0x3b] =
+			a_bRegs[0x3c] = a_bRegs[0x3d] = usb_GetNewGain(Gain_Hilight.Green,1);
 		}
 	} else {
 
@@ -834,8 +832,8 @@ TOGAIN:
 			for( dw = 0; dwLoop1; dwLoop1-- ) {
 
 				/* do some averaging... */
-				for (dwLoop2 = dwDiv, dwR = dwG = dwB = 0; dwLoop2; dwLoop2--, dw++)
-				{
+				for (dwLoop2 = dwDiv, dwR = dwG = dwB = 0;
+													dwLoop2; dwLoop2--, dw++) {
 					if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
 						dwR += ((u_short*)pScanBuffer)[dw];
 						dwG += ((u_short*)pScanBuffer)
@@ -885,12 +883,12 @@ TOGAIN:
 				  	tmp_rgb.Red, tmp_rgb.Red, tmp_rgb.Green,
 				    tmp_rgb.Green, tmp_rgb.Blue, tmp_rgb.Blue);
 
-			m_dwIdealGain = IDEAL_GainNormal;
-						 /* min(min(rgb.wRed, rgb.wGreen), rgb.wBlue) */
+/*			m_dwIdealGain = IDEAL_GainNormal;
+*/						 /* min(min(rgb.wRed, rgb.wGreen), rgb.wBlue) */
 
-			a_bRegs[0x3b] = usb_GetNewGain( tmp_rgb.Red   );
-			a_bRegs[0x3c] = usb_GetNewGain( tmp_rgb.Green );
-			a_bRegs[0x3d] = usb_GetNewGain( tmp_rgb.Blue  );
+			a_bRegs[0x3b] = usb_GetNewGain( tmp_rgb.Red,   0 );
+			a_bRegs[0x3c] = usb_GetNewGain( tmp_rgb.Green, 1 );
+			a_bRegs[0x3d] = usb_GetNewGain( tmp_rgb.Blue,  2 );
 
 			if( !_IS_PLUSTEKMOTOR(hw->motorModel)) {
 
@@ -899,8 +897,8 @@ TOGAIN:
 				/* on CIS devices, we can control the lamp off settings */
 				if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
 
-					m_dwIdealGain = IDEAL_GainNormal;
-
+/*					m_dwIdealGain = IDEAL_GainNormal;
+ */
 					if( adjLampSetting( CHANNEL_red, tmp_rgb.Red, m_dwIdealGain,
 									    hw->red_lamp_on, &hw->red_lamp_off )) {
    						adj = SANE_TRUE;
@@ -1011,14 +1009,14 @@ TOGAIN:
 
 			a_bRegs[0x3b] =
 			a_bRegs[0x3c] =
-			a_bRegs[0x3d] = usb_GetNewGain(w_tmp);
+			a_bRegs[0x3d] = usb_GetNewGain(w_tmp, 0);
 
 			DBG(_DBG_INFO2, "MAX(G)= 0x%04x(%u)\n", w_max, w_max );
 			DBG(_DBG_INFO2, "MIN(G)= 0x%04x(%u)\n", w_min, w_min );
 			DBG(_DBG_INFO2, "CUR(G)= 0x%04x(%u)\n", w_tmp, w_tmp );
 
-			m_dwIdealGain = IDEAL_GainNormal;
-
+/*			m_dwIdealGain = IDEAL_GainNormal;
+ */
 			if( !_IS_PLUSTEKMOTOR(hw->motorModel)) {
 
 				SANE_Bool adj = SANE_FALSE;
@@ -1065,7 +1063,6 @@ TOGAIN:
 		}
 	}
 
-show_sets:		
 	DBG( _DBG_INFO2, "REG[0x3b] = %u\n", a_bRegs[0x3b] );
 	DBG( _DBG_INFO2, "REG[0x3c] = %u\n", a_bRegs[0x3c] );
 	DBG( _DBG_INFO2, "REG[0x3d] = %u\n", a_bRegs[0x3d] );
@@ -1145,6 +1142,9 @@ static void usb_GetNewOffset( u_long *pdwSum, u_long *pdwDiff,
  * black pixels.
  * Affects register 0x38, 0x39 and 0x3a
  */
+static SANE_Bool cano_AdjustOffset( pPlustek_Device dev );
+
+ 
 static SANE_Bool usb_AdjustOffset( pPlustek_Device dev )
 {
 	char          tmp[40];
@@ -1155,19 +1155,12 @@ static SANE_Bool usb_AdjustOffset( pPlustek_Device dev )
 	u_long        dw, dwPixels;
     u_long        dwDiff[3], dwSum[3];
 
-	pDCapsDef scaps = &dev->usbDev.Caps;
-	pHWDef    hw    = &dev->usbDev.HwSetting;
+	pHWDef hw = &dev->usbDev.HwSetting;
 
 	if( usb_IsEscPressed())
 		return SANE_FALSE;
 
 	DBG( _DBG_INFO2, "usb_AdjustOffset()\n" );
-	if( scaps->workaroundFlag & _WAF_FIX_OFS ) {
-		a_bRegs[0x38] =
-		a_bRegs[0x39] =
-		a_bRegs[0x3a] = _CIS_OFFS;
-		return SANE_TRUE;
-	}
 
 	m_ScanParam.Size.dwLines  = 1;				/* for gain */
 	m_ScanParam.Size.dwPixels = 2550;
@@ -1177,6 +1170,7 @@ static SANE_Bool usb_AdjustOffset( pPlustek_Device dev )
 	else
 		dwPixels = (u_long)(hw->bOpticBlackEnd - hw->bOpticBlackStart );
 
+	m_ScanParam.Size.dwPixels = 2550;
 	m_ScanParam.Size.dwBytes  = m_ScanParam.Size.dwPixels * 2 *
 														m_ScanParam.bChannels;
 	if( hw->bReg_0x26 & _ONE_CH_COLOR &&
@@ -1246,7 +1240,11 @@ static SANE_Bool usb_AdjustOffset( pPlustek_Device dev )
 		}
 
 		sprintf( tmp, "coarse-off-%u.raw", i++ );
-
+		
+#ifdef SWAP_COARSE
+		if(usb_HostSwap())
+			usb_Swap((u_short *)pScanBuffer, m_ScanParam.Size.dwPhyBytes );
+#endif
 		dumpPicInit( &m_ScanParam, tmp );
 		dumpPic( tmp, pScanBuffer, m_ScanParam.Size.dwPhyBytes );
 
@@ -1255,10 +1253,15 @@ static SANE_Bool usb_AdjustOffset( pPlustek_Device dev )
 			dwSum[0] = dwSum[1] = dwSum[2] = 0;
 
 			for (dw = 0; dw < dwPixels; dw++) {
-
+#ifndef SWAP_COARSE
 				dwSum[0] += (u_long)_HILO2WORD(((pColorWordDef)pScanBuffer)[dw].HiLo[0]);
 				dwSum[1] += (u_long)_HILO2WORD(((pColorWordDef)pScanBuffer)[dw].HiLo[1]);
 				dwSum[2] += (u_long)_HILO2WORD(((pColorWordDef)pScanBuffer)[dw].HiLo[2]);
+#else
+				dwSum[0] += ((pRGBUShortDef)pScanBuffer)[dw].Red;
+				dwSum[1] += ((pRGBUShortDef)pScanBuffer)[dw].Green;
+				dwSum[2] += ((pRGBUShortDef)pScanBuffer)[dw].Blue;
+#endif
 			}
 
             DBG( _DBG_INFO2, "RedSum   = %lu, ave = %lu\n",
@@ -1284,8 +1287,13 @@ static SANE_Bool usb_AdjustOffset( pPlustek_Device dev )
 		} else {
 			dwSum[0] = 0;
 
-			for( dw = 0; dw < dwPixels; dw++ )
+			for( dw = 0; dw < dwPixels; dw++ ) {
+#ifndef SWAP_COARSE
 				dwSum[0] += (u_long)_HILO2WORD(((pHiLoDef)pScanBuffer)[dw]);
+#else
+				dwSum[0] += ((u_short*)pScanBuffer)[dw];
+#endif
+			}
 			dwSum [0] /= dwPixels;
 			usb_GetNewOffset( dwSum, dwDiff, cOffset, bExpect, 0, cAdjust );
 			a_bRegs[0x3a] = a_bRegs[0x39] = a_bRegs[0x38];
@@ -1350,7 +1358,11 @@ static void usb_GetDarkShading( pPlustek_Device dev, u_short *pwDest,
 		/* here we use the source  buffer + a static offset */
 		for (dw = 0; dw < dwPixels; dw++, pSrce += dwAdd)
 		{
+#ifndef SWAP_FINE
 			wtmp = ((int)_PHILO2WORD(pSrce) + iOffset);
+#else
+			wtmp = (*((int*)pSrce) + iOffset);
+#endif
             if( wtmp < 0 )
             	wtmp = 0;
 
@@ -1359,19 +1371,28 @@ static void usb_GetDarkShading( pPlustek_Device dev, u_short *pwDest,
 				             
             w = (u_short)wtmp;
 			
+#ifndef SWAP_FINE
 			pwDest[dw] = _LOBYTE(w) * 256 + _HIBYTE(w);
+#else
+			pwDest[dw] = w;
+#endif
 		}
 	}
 	else
 	{
-		dwSum [0] = dwSum [1] = 0;
+		dwSum[0] = dwSum[1] = 0;
 		if( hw->bSensorConfiguration & 0x04 ) {
 
 			/* Even/Odd CCD */
-			for( dw = 0; dw < dwPixels; dw++, pSrce += dwAdd )
-				dwSum [dw & 1] += (u_long)_PHILO2WORD(pSrce);
-			dwSum [0] /= ((dwPixels + 1UL) >> 1);
-			dwSum [1] /= (dwPixels >> 1);
+			for( dw = 0; dw < dwPixels; dw++, pSrce += dwAdd ) {
+#ifndef SWAP_FINE
+				dwSum[dw & 1] += (u_long)_PHILO2WORD(pSrce);
+#else
+				dwSum[dw & 1] += (u_long)(*(u_short*)pSrce);
+#endif
+			}
+			dwSum[0] /= ((dwPixels + 1UL) >> 1);
+			dwSum[1] /= (dwPixels >> 1);
 
 			if( /*Registry.GetEvenOdd() == 1 ||*/ scaps->bPCB == 2)
 			{
@@ -1386,39 +1407,53 @@ static void usb_GetDarkShading( pPlustek_Device dev, u_short *pwDest,
 
 			if((int)dwSum[1] < 0)
 				dwSum[1] = 0;
-
+#ifndef SWAP_FINE
 			dwSum[0] = (u_long)_LOBYTE(_LOWORD(dwSum[0])) * 256UL +
 													_HIBYTE(_LOWORD(dwSum[0]));
-			dwSum[1] = (u_long)_LOBYTE(_LOWORD (dwSum[1])) * 256UL +
+			dwSum[1] = (u_long)_LOBYTE(_LOWORD(dwSum[1])) * 256UL +
 													_HIBYTE(_LOWORD(dwSum[1]));
+#else
+			dwSum[0] = (u_long)_LOWORD(dwSum[0]);
+			dwSum[1] = (u_long)_LOWORD(dwSum[1]);
+#endif
 
 			for( dw = 0; dw < dwPixels; dw++ )
-				pwDest [dw] = (u_short)dwSum[dw & 1];
+				pwDest[dw] = (u_short)dwSum[dw & 1];
 		} else {
 			
-			u_long dwEnd = 0;
-
 			/* Standard CCD */
 
 			/* do some averaging on the line */
-			for( dw = dwEnd; dw < dwPixels; dw++, pSrce += dwAdd )
+			for( dw = 0; dw < dwPixels; dw++, pSrce += dwAdd ) {
+#ifndef SWAP_FINE
 				dwSum[0] += (u_long)_PHILO2WORD(pSrce);
+#else
+				dwSum[0] += (u_long)(*(u_short*)pSrce);
+#endif
+			}
 
-			dwSum[0] /= (dwPixels - dwEnd);
+			dwSum[0] /= dwPixels;
 
 			/* add our offset... */
 			dwSum[0] = (int)dwSum[0] + iOffset;
 			if((int)dwSum[0] < 0)
-				dwSum [0] = 0;
-
+				dwSum[0] = 0;
+#ifndef SWAP_FINE
 			dwSum[0] = (u_long)_LOBYTE(_LOWORD(dwSum[0])) * 256UL +
 													_HIBYTE(_LOWORD(dwSum[0]));
+#else
+			dwSum[0] = (u_long)_LOWORD(dwSum[0]);
+#endif
 
 			/* fill the shading data */
-			for( dw = dwEnd; dw < dwPixels; dw++ )
+			for( dw = 0; dw < dwPixels; dw++ )
 				pwDest[dw] = (u_short)dwSum[0];
 		}
 	}
+#ifdef SWAP_FINE
+	if(usb_HostSwap())
+		usb_Swap( pwDest, dwPixels *2 );
+#endif
 }
 
 /** usb_AdjustDarkShading
@@ -1520,6 +1555,11 @@ static SANE_Bool usb_AdjustDarkShading( pPlustek_Device dev )
 		return SANE_FALSE;
 	}	
 
+#ifdef SWAP_FINE
+	if(usb_HostSwap())
+		usb_Swap((u_short *)pScanBuffer, m_ScanParam.Size.dwPhyBytes );
+#endif
+
 	sprintf( tmp, "fine-black.raw" );
 
 	dumpPicInit( &m_ScanParam, tmp );
@@ -1592,7 +1632,8 @@ static SANE_Bool usb_AdjustWhiteShading( pPlustek_Device dev )
 	pMonoWordDef pValue;
 	u_long*      pdw;
 	int          i;
-
+	SANE_Bool    swap = usb_HostSwap();
+	
 	if( scaps->workaroundFlag & _WAF_SKIP_FINE )
 		return SANE_TRUE;
 	
@@ -1760,7 +1801,7 @@ static SANE_Bool usb_AdjustWhiteShading( pPlustek_Device dev )
 	}
 
 	if( _LM9831 == hw->chip ) {
-		
+
 		u_short *pwDest = (u_short*)pBuf;
 		pHiLoDef pwSrce = (pHiLoDef)pBuf;
 
@@ -1770,9 +1811,17 @@ static SANE_Bool usb_AdjustWhiteShading( pPlustek_Device dev )
 
 		for( dwLines = dwShadingLines /*SHADING_Lines*/; dwLines; dwLines--) {
 
-			for( dw = 0; dw < m_dwPixels * m_ScanParam.bChannels; dw++ )
-				pwDest[dw] = _HILO2WORD( pwSrce[dw] );
-
+#ifdef SWAP_FINE
+			if(usb_HostSwap()) {
+#endif
+				for( dw = 0; dw < m_dwPixels * m_ScanParam.bChannels; dw++ ) 
+					pwDest[dw] = _HILO2WORD( pwSrce[dw] );
+#ifdef SWAP_FINE
+			} else {
+				for( dw = 0; dw < m_dwPixels * m_ScanParam.bChannels; dw++ )
+					pwDest[dw] = ((u_short*)pwSrce)[dw];
+			}
+#endif
 			pwDest += (u_long)m_dwPixels * m_ScanParam.bChannels;
 			pwSrce  = (pHiLoDef)((u_char*)pwSrce + m_ScanParam.Size.dwPhyBytes);
 		}
@@ -1785,9 +1834,21 @@ static SANE_Bool usb_AdjustWhiteShading( pPlustek_Device dev )
 
 		for( dwLines = dwShadingLines/*SHADING_Lines*/; dwLines; dwLines-- ) {
 
-			for( dw = 0; dw < m_ScanParam.Size.dwPhyPixels * m_ScanParam.bChannels; dw++) 
-				pwDest[dw] = _HILO2WORD( pwSrce[dw] );
-
+#ifdef SWAP_FINE
+			if(usb_HostSwap()) {
+#endif
+				for( dw = 0; dw < m_ScanParam.Size.dwPhyPixels *
+									m_ScanParam.bChannels; dw++) {
+					pwDest[dw] = _HILO2WORD( pwSrce[dw] );
+				}
+#ifdef SWAP_FINE
+			} else {
+				for( dw = 0; dw < m_ScanParam.Size.dwPhyPixels *
+									m_ScanParam.bChannels; dw++) {
+					pwDest[dw] = ((u_short*)pwSrce)[dw];
+				}
+			}
+#endif
 			pwDest += m_ScanParam.Size.dwPhyPixels * m_ScanParam.bChannels;
 			pwSrce = (pHiLoDef)((u_char*)pwSrce + m_ScanParam.Size.dwPhyBytes);
 		}
@@ -1902,7 +1963,11 @@ static SANE_Bool usb_AdjustWhiteShading( pPlustek_Device dev )
 						pValue->Mono = (u_short)(GAIN_Target * 16384U / pValue->Mono);
 					else
 						pValue->Mono = GAIN_Target;
-					_SWAP(pValue->HiLo.bHi, pValue->HiLo.bLo);
+
+#ifdef SWAP_FINE
+					if( swap ) 
+#endif
+						_SWAP(pValue->HiLo.bHi, pValue->HiLo.bLo);
 				}
 			}
 		} else {
@@ -1947,7 +2012,7 @@ static SANE_Bool usb_AdjustWhiteShading( pPlustek_Device dev )
 					wV = pwAv [dw];
 					for (w = 0; w < m_wShadow; w++, pw += m_ScanParam.Size.dwPhyPixels)
 						if (wV < pw [dw])
-							_SWAP (wV, pw [dw]);
+							_SWAP (wV, pw[dw]);
 					pwAv [dw] = wV;
 				}
 		}
@@ -1955,18 +2020,23 @@ static SANE_Bool usb_AdjustWhiteShading( pPlustek_Device dev )
 		/* Sum */
 		pdw = (u_long*)m_pSum; /* HEINER: check this */
 
-		for (dwLines = m_wHilight, pwAv = m_pAvMono + m_ScanParam.Size.dwPhyPixels * dwLines;
-			 dwLines < (dwShadingLines/*SHADING_Lines*/ - m_wShadow); dwLines++, pwAv += m_ScanParam.Size.dwPhyPixels)
+		for (dwLines = m_wHilight,
+			 pwAv = m_pAvMono + m_ScanParam.Size.dwPhyPixels * dwLines;
+			 dwLines < (dwShadingLines/*SHADING_Lines*/ - m_wShadow);
+			 dwLines++, pwAv += m_ScanParam.Size.dwPhyPixels) {
 			for (dw = 0; dw < m_ScanParam.Size.dwPhyPixels; dw++)
-				pdw[dw] += pwAv [dw];
+				pdw[dw] += pwAv[dw];
+		}
 
 		/* Software gain */
 		pValue = (pMonoWordDef)a_wWhiteShading;
 		if( scanning->sParam.bSource != SOURCE_Negative ) {
 
 			for( dw = 0; dw < m_ScanParam.Size.dwPhyPixels; dw++) {
+				
 				pdw[dw] = pdw [dw] * 1000 / ((dwShadingLines/*SHADING_Lines*/ -
 						 m_wHilight - m_wShadow) * scanning->sParam.swGain[1]);
+						 
 				if( pdw[dw] > 65535U )
 					pValue[dw].Mono = 65535;
 				else
@@ -1974,14 +2044,24 @@ static SANE_Bool usb_AdjustWhiteShading( pPlustek_Device dev )
 
 				if( pValue[dw].Mono > 16384U ) {
 					pValue[dw].Mono = (u_short)(GAIN_Target * 16384U / pValue[dw].Mono);
-					_SWAP(pValue[dw].HiLo.bHi, pValue[dw].HiLo.bLo);
-				} else
+				} else {
 					pValue[dw].Mono = GAIN_Target;
+				}
+
+#ifdef SWAP_FINE
+				if( swap )
+#endif
+					_SWAP(pValue[dw].HiLo.bHi, pValue[dw].HiLo.bLo);
 			}
+			
 		} else{
-			for( dw = 0; dw < m_ScanParam.Size.dwPhyPixels; dw++ )
+			
+			for( dw = 0; dw < m_ScanParam.Size.dwPhyPixels; dw++ ) {
 				pValue[dw].Mono = (u_short)(pdw[dw] /
-  				   (dwShadingLines - m_wHilight - m_wShadow));
+				   (dwShadingLines - m_wHilight - m_wShadow));
+				   
+			/* swapping will be done later... */
+			}
 		}
 	}
 
@@ -1999,16 +2079,26 @@ static void usb_ResizeWhiteShading( double dAmp, u_short *pwShading, int iGain )
 	u_short	w;
 
 	for( dw = 0; dw < m_ScanParam.Size.dwPhyPixels; dw++ ) {
+		
 		dwAmp = (u_long)(GAIN_Target * 0x4000 /
 									(pwShading[dw] + 1) * dAmp) * iGain / 1000;
 		if( dwAmp <= GAIN_Target)
 			w = (u_short)dwAmp;
 		else
 			w = GAIN_Target;
+#ifndef SWAP_FINE
 		pwShading[dw] = (u_short)_LOBYTE(w) * 256 + _HIBYTE(w);
+#else
+		pwShading[dw] = w;
+#endif
 	}
-}
 
+#ifdef SWAP_FINE
+	if( usb_HostSwap()) {
+		usb_Swap( pwShading, m_ScanParam.Size.dwPhyPixels );
+	}
+#endif
+}
 /**
  */
 static void usb_PrepareCalibration( pPlustek_Device dev )
@@ -2044,10 +2134,15 @@ static void usb_PrepareCalibration( pPlustek_Device dev )
 
 	memset( a_wWhiteShading, 0, _SHADING_BUF );
 	memset( a_wDarkShading,  0, _SHADING_BUF );
+
+	scanning->skipCoarseCalib = SANE_FALSE;
+
+	if( dev->adj.cacheCalData )
+		if( usb_ReadAndSetCalData( dev ))
+			scanning->skipCoarseCalib = SANE_TRUE;
 }
 
 /** usb_DoCalibration
- *
  */
 static int usb_DoCalibration( pPlustek_Device dev )
 {
@@ -2062,14 +2157,12 @@ static int usb_DoCalibration( pPlustek_Device dev )
 
 	/* Go to shading position
      */
-	if( !(scaps->workaroundFlag & (_WAF_FIX_GAIN & _WAF_FIX_OFS))) {
-     
-		DBG( _DBG_INFO, "goto shading position\n" );
+	DBG( _DBG_INFO, "goto shading position\n" );
 
-		/* HEINER: Currently not clear why Plustek didn't use the ShadingOriginY
-		 *         for all modes
-		 * It should be okay to remove this and reference to the ShadingOriginY
-		 */	
+	/* HEINER: Currently not clear why Plustek didn't use the ShadingOriginY
+	 *         for all modes
+	 * It should be okay to remove this and reference to the ShadingOriginY
+	 */	
 #if 0	
 	if( scanning->sParam.bSource == SOURCE_Negative ) {
 
@@ -2083,22 +2176,21 @@ static int usb_DoCalibration( pPlustek_Device dev )
 
 	} else {
 #endif
-			DBG( _DBG_INFO, "ShadingOriginY=%lu\n",
-					(u_long)dev->usbDev.pSource->ShadingOriginY );
+		DBG( _DBG_INFO, "ShadingOriginY=%lu\n",
+				(u_long)dev->usbDev.pSource->ShadingOriginY );
 
-			if((hw->motorModel == MODEL_HuaLien) && (scaps->OpticDpi.x==600)) {
-				if (!usb_ModuleMove(dev, MOVE_ToShading,
-								(u_long)dev->usbDev.pSource->ShadingOriginY)) {
-					return _E_LAMP_NOT_IN_POS;
-				}
-			} else {
-				if( !usb_ModuleMove(dev, MOVE_Forward,
-								(u_long)dev->usbDev.pSource->ShadingOriginY)) {
-					return _E_LAMP_NOT_IN_POS;
-				}
+		if((hw->motorModel == MODEL_HuaLien) && (scaps->OpticDpi.x==600)) {
+			if (!usb_ModuleMove(dev, MOVE_ToShading,
+							(u_long)dev->usbDev.pSource->ShadingOriginY)) {
+				return _E_LAMP_NOT_IN_POS;
 			}
-/*		}*/
-	}
+		} else {
+			if( !usb_ModuleMove(dev, MOVE_Forward,
+							(u_long)dev->usbDev.pSource->ShadingOriginY)) {
+				return _E_LAMP_NOT_IN_POS;
+			}
+		}
+/*	}*/
 
 	DBG( _DBG_INFO, "shading position reached\n" );
 
@@ -2465,7 +2557,8 @@ static int usb_DoCalibration( pPlustek_Device dev )
 	}
 
 	scanning->fCalibrated = SANE_TRUE;
-	DBG( _DBG_INFO, "Calibration done\n-----------------------\n" );
+	DBG( _DBG_INFO, "Calibration done\n" );
+	DBG( _DBG_INFO, "-----------------------\n" );
 	DBG( _DBG_INFO, "Static Gain:\n" );
 	DBG( _DBG_INFO, "REG[0x3b] = %u\n", a_bRegs[0x3b] );
 	DBG( _DBG_INFO, "REG[0x3c] = %u\n", a_bRegs[0x3c] );
@@ -2485,11 +2578,16 @@ static int usb_DoCalibration( pPlustek_Device dev )
  */
 static SANE_Bool usb_DownloadShadingData( pPlustek_Device dev, u_char bJobID )
 {
+	u_char    channel;
 	pDCapsDef scaps = &dev->usbDev.Caps;
 	pHWDef    hw    = &dev->usbDev.HwSetting;
 
 	DBG( _DBG_INFO2, "usb_DownloadShadingData(%u)\n", bJobID );
 
+	channel = CHANNEL_green;
+	if( hw->bReg_0x26 & _ONE_CH_COLOR )
+		channel = CHANNEL_blue;
+
 	switch( bJobID ) {
 
 		case PARAM_WhiteShading:
@@ -2504,7 +2602,7 @@ static SANE_Bool usb_DownloadShadingData( pPlustek_Device dev, u_char bJobID )
 									m_ScanParam.Size.dwPhyPixels * 2,
 								    (u_short)m_ScanParam.Size.dwPhyPixels * 2);
 			} else {
-				usb_SetDarkShading( dev->fd, CHANNEL_green, a_wDarkShading +
+				usb_SetDarkShading( dev->fd, channel, a_wDarkShading +
 									m_ScanParam.Size.dwPhyPixels,
 								    (u_short)m_ScanParam.Size.dwPhyPixels * 2);
 			}
@@ -2575,7 +2673,7 @@ static SANE_Bool usb_DownloadShadingData( pPlustek_Device dev, u_char bJobID )
 										a_wDarkShading + m_dwPixels * 2,
 									(u_short)m_ScanParam.Size.dwPhyPixels * 2);
 				} else {
-					usb_SetDarkShading( dev->fd, CHANNEL_green,
+					usb_SetDarkShading( dev->fd, channel,
 										a_wDarkShading + m_dwPixels,
 									(u_short)m_ScanParam.Size.dwPhyPixels * 2);
 				}
@@ -2586,21 +2684,21 @@ static SANE_Bool usb_DownloadShadingData( pPlustek_Device dev, u_char bJobID )
 					usb_SetWhiteShading( dev->fd, CHANNEL_green,
 										 a_wWhiteShading +
 										 m_ScanParam.Size.dwPhyPixels,
-									 (u_short)m_ScanParam.Size.dwPhyPixels * 2);
+									(u_short)m_ScanParam.Size.dwPhyPixels * 2);
 					usb_SetWhiteShading( dev->fd, CHANNEL_blue,
 									     a_wWhiteShading +
 										 m_ScanParam.Size.dwPhyPixels * 2,
-									 (u_short)m_ScanParam.Size.dwPhyPixels * 2);
+									(u_short)m_ScanParam.Size.dwPhyPixels * 2);
 				} else {
-					usb_SetWhiteShading( dev->fd,CHANNEL_green,a_wWhiteShading,
+					usb_SetWhiteShading( dev->fd, channel, a_wWhiteShading,
 									(u_short)m_ScanParam.Size.dwPhyPixels * 2);
 				}
 				
 				/* set RAM configuration AND
-	             * Gain = Multiplier Coefficient/16384
-                 * External DRAM for Multiplier Coefficient Source
-                 * External DRAM for Offset Coefficient Source
-                 */
+				 * Gain = Multiplier Coefficient/16384
+				 * External DRAM for Multiplier Coefficient Source
+				 * External DRAM for Offset Coefficient Source
+				 */
 				a_bRegs[0x42] = (u_char)((hw->wDRAMSize > 512)? 0x66: 0x26);
 
 				if( scaps->workaroundFlag & _WAF_SKIP_WHITEFINE ) {
@@ -2621,10 +2719,10 @@ static SANE_Bool usb_DownloadShadingData( pPlustek_Device dev, u_char bJobID )
 			a_bRegs[0x41] = 0x00;
 
 			/* set RAM configuration AND
-             * GAIN = Multiplier Coefficient/16384
-             * CFG Register 0x40/0x41 for Multiplier Coefficient Source
-             * CFG Register 0x3e/0x3f for Offset Coefficient Source
-             */
+			 * GAIN = Multiplier Coefficient/16384
+			 * CFG Register 0x40/0x41 for Multiplier Coefficient Source
+			 * CFG Register 0x3e/0x3f for Offset Coefficient Source
+			 */
 			a_bRegs[0x42] = (u_char)((hw->wDRAMSize > 512)? 0x60: 0x20);
 
 			_UIO(sanei_lm983x_write( dev->fd, 0x3e, &a_bRegs[0x3e],
