@@ -46,9 +46,62 @@
    This file is a component of the implementation of a backend for many
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
-/* $Id: snapscan-options.c,v 1.7 2003/04/30 20:49:39 oliverschwartz Exp $
+/* $Id: snapscan-options.c,v 1.16 2004/04/08 21:53:10 oliver-guest Exp $
    SANE SnapScan backend */
 
+/* default option values */
+
+#define DEFAULT_RES             300
+#define DEFAULT_PREVIEW         SANE_FALSE
+#define DEFAULT_HIGHQUALITY     SANE_FALSE
+#define DEFAULT_BRIGHTNESS      0
+#define DEFAULT_CONTRAST        0
+#define DEFAULT_GAMMA           SANE_FIX(1.8)
+#define DEFAULT_HALFTONE        SANE_FALSE
+#define DEFAULT_NEGATIVE        SANE_FALSE
+#define DEFAULT_THRESHOLD       50
+#define DEFAULT_QUALITY         SANE_TRUE
+#define DEFAULT_CUSTOM_GAMMA    SANE_FALSE
+#define DEFAULT_GAMMA_BIND      SANE_FALSE
+
+static SANE_Int def_rgb_lpr = 4;
+static SANE_Int def_gs_lpr = 12;
+
+
+/* predefined preview mode name */
+static char md_auto[] = "Auto";
+
+/* predefined scan mode names */
+static char md_colour[] = SANE_I18N("Color");
+static char md_bilevelcolour[] = SANE_I18N("Halftone");
+static char md_greyscale[] = SANE_I18N("Gray");
+static char md_lineart[] = SANE_I18N("Lineart");
+
+/* predefined scan source names */
+static char src_flatbed[] = SANE_I18N("Flatbed");
+static char src_tpo[] = SANE_I18N("Transparency Adapter");
+static char src_adf[] = SANE_I18N("Document Feeder");
+
+/* predefined scan window setting names */
+static char pdw_none[] = SANE_I18N("None");
+static char pdw_6X4[] = SANE_I18N("6x4 (inch)");
+static char pdw_8X10[] = SANE_I18N("8x10 (inch)");
+static char pdw_85X11[] = SANE_I18N("8.5x11 (inch)");
+
+/* predefined dither matrix names */
+static char dm_none[] = SANE_I18N("Halftoning Unsupported");
+static char dm_dd8x8[] = SANE_I18N("DispersedDot8x8");
+static char dm_dd16x16[] = SANE_I18N("DispersedDot16x16");
+
+/* strings */
+static char lpr_desc[] = SANE_I18N(
+    "Number of scan lines to request in a SCSI read. "
+    "Changing this parameter allows you to tune the speed at which "
+    "data is read from the scanner during scans. If this is set too "
+    "low, the scanner will have to stop periodically in the middle of "
+    "a scan; if it's set too high, X-based frontends may stop responding "
+    "to X events and your system could bog down.");
+
 /* ranges */
 static const SANE_Range x_range_fb =
 {
@@ -59,7 +112,7 @@ static const SANE_Range y_range_fb =
     SANE_FIX (0.0), SANE_FIX (297.0), 0
 };        /* mm */
 
-/* default TPO range (shortest y_range 
+/* default TPO range (shortest y_range
    to avoid tray collision.
 */
 static const SANE_Range x_range_tpo_default =
@@ -91,6 +144,16 @@ static const SANE_Range y_range_tpo_e50 =
     SANE_FIX (0.0), SANE_FIX (240.0), 0
 };        /* mm */
 
+/* TPO range for the Epson 1670 */
+static const SANE_Range x_range_tpo_1670 =
+{
+    SANE_FIX (0.0), SANE_FIX (101.0), 0
+};        /* mm */
+static const SANE_Range y_range_tpo_1670 =
+{
+    SANE_FIX (0.0), SANE_FIX (228.0), 0
+};        /* mm */
+
 static SANE_Range x_range_tpo;
 static SANE_Range y_range_tpo;
 static const SANE_Range gamma_range =
@@ -127,6 +190,8 @@ static const SANE_Range positive_percent_range =
     1 << SANE_FIXED_SCALE_SHIFT
 };
 
+static void control_options(SnapScan_Scanner *pss);
+
 /* init_options -- initialize the option set for a scanner; expects the
    scanner structure's hardware configuration byte (hconfig) to be valid.
 
@@ -142,6 +207,8 @@ static void init_options (SnapScan_Scanner * ps)
         {8, 50, 75, 100, 150, 200, 300, 450, 600};
     static SANE_Word resolutions_1200[] =
         {10, 50, 75, 100, 150, 200, 300, 450, 600, 900, 1200};
+    static SANE_Word resolutions_1600[] =
+        {10, 50, 75, 100, 150, 200, 300, 400, 600, 800, 1600};
     static SANE_String_Const names_all[] =
         {md_colour, md_bilevelcolour, md_greyscale, md_lineart, NULL};
     static SANE_String_Const names_basic[] =
@@ -165,6 +232,10 @@ static void init_options (SnapScan_Scanner * ps)
         x_range_tpo = x_range_tpo_e50;
         y_range_tpo = y_range_tpo_e50;
         break;
+    case PERFECTION1670:
+        x_range_tpo = x_range_tpo_1670;
+        y_range_tpo = y_range_tpo_1670;
+        break;
     default:
         x_range_tpo = x_range_tpo_default;
         y_range_tpo = y_range_tpo_default;
@@ -204,7 +275,7 @@ static void init_options (SnapScan_Scanner * ps)
     switch (ps->pdev->model)
     {
     case SNAPSCAN310:
-    case VUEGO310S:                /* WG changed */
+    case PRISA310:                /* WG changed */
         po[OPT_SCANRES].constraint.word_list = resolutions_300;
         break;
     case SNAPSCANE50:
@@ -215,6 +286,9 @@ static void init_options (SnapScan_Scanner * ps)
     case ARCUS1200:
         po[OPT_SCANRES].constraint.word_list = resolutions_1200;
         break;
+    case PERFECTION1670:
+        po[OPT_SCANRES].constraint.word_list = resolutions_1600;
+        break;
     default:
         po[OPT_SCANRES].constraint.word_list = resolutions_600;
         break;
@@ -232,6 +306,17 @@ static void init_options (SnapScan_Scanner * ps)
     po[OPT_PREVIEW].constraint_type = SANE_CONSTRAINT_NONE;
     ps->preview = DEFAULT_PREVIEW;
 
+    po[OPT_HIGHQUALITY].name = "high-quality";
+    po[OPT_HIGHQUALITY].title = SANE_I18N("Quality scan");
+    po[OPT_HIGHQUALITY].desc = SANE_I18N("Highest quality but lower speed");
+    po[OPT_HIGHQUALITY].type = SANE_TYPE_BOOL;
+    po[OPT_HIGHQUALITY].unit = SANE_UNIT_NONE;
+    po[OPT_HIGHQUALITY].size = sizeof (SANE_Word);
+    po[OPT_HIGHQUALITY].cap =
+        SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT | SANE_CAP_AUTOMATIC;
+    po[OPT_HIGHQUALITY].constraint_type = SANE_CONSTRAINT_NONE;
+    ps->highquality = DEFAULT_HIGHQUALITY;
+
     po[OPT_BRIGHTNESS].name = SANE_NAME_BRIGHTNESS;
     po[OPT_BRIGHTNESS].title = SANE_TITLE_BRIGHTNESS;
     po[OPT_BRIGHTNESS].desc = SANE_DESC_BRIGHTNESS;
@@ -265,7 +350,7 @@ static void init_options (SnapScan_Scanner * ps)
     switch (ps->pdev->model)
     {
     case SNAPSCAN310:
-    case VUEGO310S:
+    case PRISA310:
         po[OPT_MODE].constraint.string_list = names_basic;
         break;
     default:
@@ -283,15 +368,15 @@ static void init_options (SnapScan_Scanner * ps)
     po[OPT_PREVIEW_MODE].type = SANE_TYPE_STRING;
     po[OPT_PREVIEW_MODE].unit = SANE_UNIT_NONE;
     po[OPT_PREVIEW_MODE].size = 32;
-    po[OPT_PREVIEW_MODE].cap = SANE_CAP_SOFT_SELECT 
+    po[OPT_PREVIEW_MODE].cap = SANE_CAP_SOFT_SELECT
                                | SANE_CAP_SOFT_DETECT
-                               | SANE_CAP_ADVANCED 
+                               | SANE_CAP_ADVANCED
                                | SANE_CAP_AUTOMATIC;
     po[OPT_PREVIEW_MODE].constraint_type = SANE_CONSTRAINT_STRING_LIST;
     switch (ps->pdev->model)
     {
     case SNAPSCAN310:
-    case VUEGO310S:
+    case PRISA310:
         po[OPT_PREVIEW_MODE].constraint.string_list = preview_names_basic;
         break;
     default:
@@ -420,6 +505,7 @@ static void init_options (SnapScan_Scanner * ps)
        although HCFG_CAL_ALLOWED is set. */
     if ((!(ps->hconfig & HCFG_CAL_ALLOWED))
         || (ps->pdev->model == SNAPSCANE52)
+        || (ps->pdev->model == PERFECTION1670)
         || (ps->pdev->model == PRISA5300)) {
         po[OPT_QUALITY_CAL].cap |= SANE_CAP_INACTIVE;
         ps->val[OPT_QUALITY_CAL].b = SANE_FALSE;
@@ -654,6 +740,7 @@ static void init_options (SnapScan_Scanner * ps)
     po[OPT_GS_LPR].constraint_type = SANE_CONSTRAINT_RANGE;
     po[OPT_GS_LPR].constraint.range = &lpr_range;
     ps->gs_lpr = def_gs_lpr;
+    control_options(ps);
 }
 
 const SANE_Option_Descriptor *sane_get_option_descriptor (SANE_Handle h,
@@ -778,6 +865,9 @@ SANE_Status sane_control_option (SANE_Handle h,
         case OPT_PREVIEW:
             *(SANE_Bool *) v = pss->preview;
             break;
+        case OPT_HIGHQUALITY:
+            *(SANE_Bool *) v = pss->highquality;
+            break;
         case OPT_MODE:
             DBG (DL_VERBOSE,
                  "%s: writing \"%s\" to location %p\n",
@@ -913,6 +1003,11 @@ SANE_Status sane_control_option (SANE_Handle h,
             if (i)
                 *i |= SANE_INFO_RELOAD_PARAMS;
             break;
+        case OPT_HIGHQUALITY:
+            pss->highquality = *(SANE_Bool *) v;
+            if (i)
+                *i |= SANE_INFO_RELOAD_PARAMS;
+            break;
         case OPT_MODE:
             {
                 char *s = (SANE_String) v;
@@ -1314,6 +1409,11 @@ SANE_Status sane_control_option (SANE_Handle h,
             if (i)
                 *i |= SANE_INFO_RELOAD_PARAMS;
             break;
+        case OPT_HIGHQUALITY:
+            pss->highquality = SANE_FALSE;
+            if (i)
+                *i |= SANE_INFO_RELOAD_PARAMS;
+            break;
         case OPT_MODE:
             pss->mode_s = md_colour;
             pss->mode = MD_COLOUR;
@@ -1388,6 +1488,33 @@ SANE_Status sane_control_option (SANE_Handle h,
 
 /*
  * $Log: snapscan-options.c,v $
+ * Revision 1.16  2004/04/08 21:53:10  oliver-guest
+ * Use sanei_thread in snapscan backend
+ *
+ * Revision 1.15  2004/04/02 20:19:23  oliver-guest
+ * Various bugfixes for gamma corretion (thanks to Robert Tsien)
+ *
+ * Revision 1.14  2004/02/01 13:32:26  oliver-guest
+ * Fixed resolutions for Epson 1670
+ *
+ * Revision 1.13  2003/11/28 23:23:18  oliver-guest
+ * Correct length of wordlist for resolutions_1600
+ *
+ * Revision 1.12  2003/11/09 21:43:45  oliver-guest
+ * Disabled quality calibration for Epson Perfection 1670
+ *
+ * Revision 1.11  2003/11/08 09:50:27  oliver-guest
+ * Fix TPO scanning range for Epson 1670
+ *
+ * Revision 1.10  2003/10/21 20:43:25  oliver-guest
+ * Bugfixes for SnapScan backend
+ *
+ * Revision 1.9  2003/10/07 18:29:20  oliver-guest
+ * Initial support for Epson 1670, minor bugfix
+ *
+ * Revision 1.8  2003/08/19 21:05:08  oliverschwartz
+ * Scanner ID cleanup
+ *
  * Revision 1.7  2003/04/30 20:49:39  oliverschwartz
  * SnapScan backend 1.4.26
  *
