@@ -47,7 +47,7 @@
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
 
-/* $Id: snapscan-scsi.c,v 1.22 2003/04/30 20:49:39 oliverschwartz Exp $
+/* $Id: snapscan-scsi.c,v 1.27 2004/04/02 20:19:24 oliver-guest Exp $
    SnapScan backend scsi command functions */
 
 /* scanner scsi commands */
@@ -433,44 +433,7 @@ static SANE_Status inquiry (SnapScan_Scanner *pss)
                pss->buf,
                &pss->read_bytes);
     CHECK_STATUS (status, me, "snapscan_cmd");
-    /* Download Firmware for USB scanners */
-    if ( (pss->pdev->bus == USB) && (*(pss->buf + INQUIRY_HWST) & 0x02) )
-    {
-        char model[17];
 
-        status = download_firmware(pss);
-        CHECK_STATUS (status, me, "download_firmware");
-        /* send inquiry command again, wait for scanner to initialize */
-        do {
-            status = snapscan_cmd (pss->pdev->bus,
-                pss->fd,
-                pss->cmd,
-                INQUIRY_LEN,
-                pss->buf,
-                &pss->read_bytes);
-            if (status == SANE_STATUS_DEVICE_BUSY) {
-                DBG (DL_INFO, "%s: Waiting for scanner after firmware upload\n",me);
-                sleep(1);
-            }
-        } while (status == SANE_STATUS_DEVICE_BUSY);
-        CHECK_STATUS (status, me, "snapscan_cmd");
-        /* The model identifier will change after firmware upload */
-        memcpy (model, &pss->buf[INQUIRY_PRODUCT], 16);
-        model[16] = 0;
-        remove_trailing_space(model);
-        DBG (DL_INFO,
-            "%s (after firmware upload): Checking if \"%s\" is a supported scanner\n",
-            me,
-            model);
-        /* Check if it is one of our supported models */
-        pss->pdev->model = snapscani_get_model_id(model, pss->fd, pss->pdev->bus);
-        if (pss->pdev->model == UNKNOWN) {
-            DBG (DL_MINOR_ERROR,
-                "%s (after firmware upload): \"%s\" is not a supported scanner\n",
-                me,
-                model);
-        }
-    }
     /* record current parameters */
     {
         char exptime[4] = {' ', '.', ' ', 0};
@@ -534,7 +497,27 @@ static SANE_Status inquiry (SnapScan_Scanner *pss)
     pss->bytes_remaining = pss->bytes_per_line * (pss->lines + pss->chroma);
     pss->expected_read_bytes = 0;
     pss->read_bytes = 0;
+    pss->hwst = pss->buf[INQUIRY_HWST];
     pss->hconfig = pss->buf[INQUIRY_HCFG];
+    pss->bpp = 8;
+    switch (pss->pdev->model)
+    {
+    case PERFECTION1670:
+        pss->bpp = 14;
+        break;
+    default:
+        if (pss->hconfig & HCFG_ADC)
+            pss->bpp = 10;
+        break;
+    }
+    DBG (DL_DATA_TRACE,
+        "%s: hardware config = 0x%02x\n",
+        me,
+        pss->hconfig);
+    DBG (DL_DATA_TRACE,
+        "%s: bits per pixel = %lu\n",
+        me,
+        (u_long) pss->bpp);
     DBG (DL_DATA_TRACE,
         "%s: pixels per scan line = %lu\n",
         me,
@@ -622,6 +605,10 @@ static void release_unit (SnapScan_Scanner *pss)
 #define DTCQ_GAMMA_RED10 0x81
 #define DTCQ_GAMMA_GREEN10 0x82
 #define DTCQ_GAMMA_BLUE10 0x83
+#define DTCQ_GAMMA_GRAY14 0x95 /* ? */
+#define DTCQ_GAMMA_RED14 0x96
+#define DTCQ_GAMMA_GREEN14 0x97
+#define DTCQ_GAMMA_BLUE14 0x98
 
 static SANE_Status send (SnapScan_Scanner *pss, u_char dtc, u_char dtcq)
 {
@@ -671,6 +658,12 @@ static SANE_Status send (SnapScan_Scanner *pss, u_char dtc, u_char dtcq)
         case DTCQ_GAMMA_BLUE10:
             tl = 1024;
             break;
+        case DTCQ_GAMMA_GRAY14:    /* 14-bit tables */
+        case DTCQ_GAMMA_RED14:
+        case DTCQ_GAMMA_GREEN14:
+        case DTCQ_GAMMA_BLUE14:
+            tl = 16384;
+            break;
         default:
             DBG (DL_MAJOR_ERROR, "%s: bad gamma data type qualifier 0x%x\n",
                  me, dtcq);
@@ -767,18 +760,17 @@ static SANE_Status set_window (SnapScan_Scanner *pss)
     DBG (DL_CALL_TRACE, "%s Resolution: %d\n", me, pss->res);
 
     pos_factor = pss->actual_res;
-    if (pss->pdev->model == PRISA5000) 
+    switch (pss->pdev->model)
     {
-        if (pss->res > 600) 
-        {
-            pos_factor = 1200;
-        }
-        else
-        {
-            pos_factor = 600;
-        }    
+        case PRISA5000:
+            pos_factor = (pss->res > 600) ?  1200 : 600;
+            break;
+        case PERFECTION1670:
+            pos_factor = (pss->res > 800) ?  1600 : 800;
+            break;
+        default:
+            break;
     }
-
     /* it's an ugly sound if the scanner drives against the rear
        wall, and with changing max values we better be sure */
     check_range(&(pss->brx), pss->pdev->x_range);
@@ -816,25 +808,23 @@ static SANE_Status set_window (SnapScan_Scanner *pss)
     pc[SET_WINDOW_P_CONTRAST] = 128;
     {
         SnapScan_Mode mode = pss->mode;
-        u_char bpp;
+        u_char bpp = 8;
 
         if (pss->preview)
             mode = pss->preview_mode;
 
-        bpp = pss->pdev->depths[mode];
         DBG (DL_MINOR_INFO, "%s Mode: %d\n", me, mode);
         switch (mode)
         {
         case MD_COLOUR:
             pc[SET_WINDOW_P_COMPOSITION] = 0x05;    /* multi-level RGB */
-            bpp *= 3;
             break;
         case MD_BILEVELCOLOUR:
             if (pss->halftone)
                 pc[SET_WINDOW_P_COMPOSITION] = 0x04;    /* halftone RGB */
             else
                 pc[SET_WINDOW_P_COMPOSITION] = 0x03;    /* bi-level RGB */
-            bpp *= 3;
+            bpp = 1;
             break;
         case MD_GREYSCALE:
             pc[SET_WINDOW_P_COMPOSITION] = 0x02;    /* grayscale */
@@ -866,19 +856,26 @@ static SANE_Status set_window (SnapScan_Scanner *pss)
        &&
        pss->pdev->model != SNAPSCAN310
        &&
-       pss->pdev->model != VUEGO310S
+       pss->pdev->model != PRISA310
        &&
-       pss->pdev->model != VUEGO610S
+       pss->pdev->model != PRISA610
     ) {
         pc[SET_WINDOW_P_DEBUG_MODE] = 2;        /* use full 128k buffer */
-        pc[SET_WINDOW_P_GAMMA_NO] = 0x01;        /* downloaded table */
+        if (pss->mode != MD_LINEART)
+        {
+            pc[SET_WINDOW_P_GAMMA_NO] = 0x01;   /* downloaded gamma table */
+        }
     }
-    source = 0x20;
+    source = 0x0;
     if (pss->preview) {
-        source |= 0x80; /* no high quality */
-    } else {
+        source |= 0x80; /* no high quality in preview */
+    }
+    else {
         source |= 0x40; /* no preview */
     }
+    if (!pss->highquality) {
+        source |= 0x80; /* no high quality */
+    }
 
     if (pss->source == SRC_TPO) {
         source |= 0x08;
@@ -887,7 +884,7 @@ static SANE_Status set_window (SnapScan_Scanner *pss)
         source |= 0x10;
     }
     pc[SET_WINDOW_P_OPERATION_MODE] = source;
-    DBG (DL_MINOR_INFO, "%s: operation mode set to %d\n", me, (int) source);
+    DBG (DL_MINOR_INFO, "%s: operation mode set to 0x%02x\n", me, (int) source);
     pc[SET_WINDOW_P_RED_UNDER_COLOR] = 0xff;    /* defaults */
     pc[SET_WINDOW_P_BLUE_UNDER_COLOR] = 0xff;
     pc[SET_WINDOW_P_GREEN_UNDER_COLOR] = 0xff;
@@ -976,13 +973,13 @@ static SANE_Status send_diagnostic (SnapScan_Scanner *pss)
     u_char cmd[] = {SEND_DIAGNOSTIC, 0x04, 0, 0, 0, 0};    /* self-test */
     SANE_Status status;
 
-    if (pss->pdev->model == PRISA620S
+    if (pss->pdev->model == PRISA620
         ||
-	pss->pdev->model == VUEGO610S
-	||
-	pss->pdev->model == SNAPSCAN1236
+     pss->pdev->model == PRISA610
         ||
-        pss->pdev->model == ARCUS1200) 
+     pss->pdev->model == SNAPSCAN1236
+        ||
+        pss->pdev->model == ARCUS1200)
     {
         return SANE_STATUS_GOOD;
     }
@@ -1015,9 +1012,9 @@ static SANE_Status wait_scanner_ready (SnapScan_Scanner *pss)
             /* first additional sense byte contains time to wait */
             {
                 int delay = pss->asi1 + 1;
-                DBG (DL_INFO,
-                    "%s: scanner warming up. Waiting %ld seconds.\n",
-                    me, (long) delay);
+                DBG (0,
+                    "Scanner warming up - waiting %ld seconds.\n",
+                    (long) delay);
                 sleep (delay);
             }
             break;
@@ -1157,9 +1154,9 @@ static SANE_Status download_firmware(SnapScan_Scanner * pss)
         {
             switch (pss->pdev->model)
             {
-            case VUEGO610S:
-            case VUEGO310S:
-            case PRISA620S:
+            case PRISA610:
+            case PRISA310:
+            case PRISA620:
             case PRISA1240:
             case PRISA640:
             case PRISA4300:
@@ -1171,6 +1168,20 @@ static SANE_Status download_firmware(SnapScan_Scanner * pss)
                 bufLength = ftell(fd);
                 fseek(fd, 0, SEEK_SET);
                 break;
+            case PERFECTION1670:
+                /* Epson firmware files contain an info block which
+                   specifies the length of the firmware data. The
+                   length information is stored at offset 0x64 from
+                   end of file */
+                {
+                    unsigned char size_l, size_h;
+                    fseek(fd, -0x64, SEEK_END);
+                    fread(&size_l, 1, 1, fd);
+                    fread(&size_h, 1, 1, fd);
+                    fseek(fd, 0, SEEK_SET);
+                    bufLength = (size_h << 8) + size_l;
+                }
+                break;
             default:
                 /* AGFA firmware files contain an info block which
                    specifies the length of the firmware data. The
@@ -1211,6 +1222,21 @@ static SANE_Status download_firmware(SnapScan_Scanner * pss)
 
 /*
  * $Log: snapscan-scsi.c,v $
+ * Revision 1.27  2004/04/02 20:19:24  oliver-guest
+ * Various bugfixes for gamma corretion (thanks to Robert Tsien)
+ *
+ * Revision 1.26  2003/11/07 23:26:49  oliver-guest
+ * Final bugfixes for bascic support of Epson 1670
+ *
+ * Revision 1.25  2003/10/21 20:43:25  oliver-guest
+ * Bugfixes for SnapScan backend
+ *
+ * Revision 1.24  2003/10/07 19:41:34  oliver-guest
+ * Updates for Epson Perfection 1670
+ *
+ * Revision 1.23  2003/08/19 21:05:08  oliverschwartz
+ * Scanner ID cleanup
+ *
  * Revision 1.22  2003/04/30 20:49:39  oliverschwartz
  * SnapScan backend 1.4.26
  *
