@@ -24,7 +24,7 @@
 
    As a special exception, the authors of SANE give permission for
    additional uses of the libraries contained in this release of SANE.
- 
+
    The exception is that, if you link a SANE library with other files
    to produce an executable, this does not by itself cause the
    resulting executable to be covered by the GNU General Public
@@ -47,7 +47,7 @@
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
 
-/* $Id: snapscan.c,v 1.29 2003/04/30 20:49:40 oliverschwartz Exp $
+/* $Id: snapscan.c,v 1.40 2004/04/09 11:59:02 oliver-guest Exp $
    SANE SnapScan backend */
 
 #include "../include/sane/config.h"
@@ -71,6 +71,7 @@
 #include "../include/sane/sanei.h"
 #include "../include/sane/sanei_scsi.h"
 #include "../include/sane/sanei_usb.h"
+#include "../include/sane/sanei_thread.h"
 
 #ifndef PATH_MAX
 #define PATH_MAX        1024
@@ -78,15 +79,17 @@
 
 #define EXPECTED_MAJOR       1
 #define MINOR_VERSION        4
-#define BUILD               26
-
-#include "snapscan.h"
+#define BUILD               37
 
 #define BACKEND_NAME snapscan
 
+#define UNUSEDARG __attribute__ ((unused))
+
 #include "../include/sane/sanei_backend.h"
 #include "../include/sane/saneopts.h"
 
+#include "snapscan.h"
+
 #define MIN(x,y) ((x)<(y) ? (x) : (y))
 #define MAX(x,y) ((x)>(y) ? (x) : (y))
 #define LIMIT(x,min,max) MIN(MAX(x, min), max)
@@ -127,63 +130,10 @@ if ((s) != SANE_STATUS_GOOD) { DBG(DL_MAJOR_ERROR, "%s: %s command failed: %s\n"
 #define MM_PER_IN 25.4                /* # millimetres per inch */
 #define IN_PER_MM 0.03937        /* # inches per millimetre  */
 
-/* default option values */
-
-#define DEFAULT_RES                  300
-#define DEFAULT_PREVIEW         SANE_FALSE
-
-#define DEFAULT_BRIGHTNESS        0
-#define DEFAULT_CONTRAST        0
-#define DEFAULT_GAMMA                SANE_FIX(1.8)
-#define DEFAULT_HALFTONE        SANE_FALSE
-#define DEFAULT_NEGATIVE        SANE_FALSE
-#define DEFAULT_THRESHOLD        50
-#define DEFAULT_QUALITY         SANE_TRUE
-#define DEFAULT_CUSTOM_GAMMA        SANE_FALSE
-#define DEFAULT_GAMMA_BIND        SANE_FALSE
-
 #ifndef SANE_I18N
 #define SANE_I18N(text) text
 #endif
 
-static SANE_Int def_rgb_lpr = 4;
-static SANE_Int def_gs_lpr = 12;
-
-
-/* predefined preview mode name */
-static char md_auto[] = "Auto";
-
-/* predefined scan mode names */
-static char md_colour[] = SANE_I18N("Color");
-static char md_bilevelcolour[] = SANE_I18N("Halftone");
-static char md_greyscale[] = SANE_I18N("Gray");
-static char md_lineart[] = SANE_I18N("Lineart");
-
-/* predefined scan source names */
-static char src_flatbed[] = SANE_I18N("Flatbed");
-static char src_tpo[] = SANE_I18N("Transparency Adapter");
-static char src_adf[] = SANE_I18N("Document Feeder");
-
-/* predefined scan window setting names */
-static char pdw_none[] = SANE_I18N("none");
-static char pdw_6X4[] = SANE_I18N("6x4 (inch)");
-static char pdw_8X10[] = SANE_I18N("8x10 (inch)");
-static char pdw_85X11[] = SANE_I18N("8.5x11 (inch)");
-
-/* predefined dither matrix names */
-static char dm_none[] = SANE_I18N("Halftoning Unsupported");
-static char dm_dd8x8[] = SANE_I18N("DispersedDot8x8");
-static char dm_dd16x16[] = SANE_I18N("DispersedDot16x16");
-
-/* strings */
-static char lpr_desc[] = SANE_I18N(
-    "Number of scan lines to request in a SCSI read. "
-    "Changing this parameter allows you to tune the speed at which "
-    "data is read from the scanner during scans. If this is set too "
-    "low, the scanner will have to stop periodically in the middle of "
-    "a scan; if it's set too high, X-based frontends may stop responding "
-    "to X events and your system could bog down.");
-
 /* authorization stuff */
 static SANE_Auth_Callback auth = NULL;
 #if UNUSED
@@ -191,12 +141,6 @@ static SANE_Char username[SANE_MAX_USERNAME_LEN];
 static SANE_Char password[SANE_MAX_PASSWORD_LEN];
 #endif
 
-/* bit depth tables */
-static u_char depths8[MD_NUM_MODES] =        {8, 1, 8, 1};
-static u_char depths10[MD_NUM_MODES] =        {10, 1, 10, 1};
-static u_char depths12[MD_NUM_MODES] =        {12, 1, 12, 1};
-static u_char depths14[MD_NUM_MODES] =        {14, 1, 14, 1};
-
 /* function prototypes */
 
 static void gamma_n (double gamma, int brightness, int contrast,
@@ -221,13 +165,19 @@ static inline int is_colour_mode (SnapScan_Mode m)
 static inline int calibration_line_length(SnapScan_Scanner *pss)
 {
     int pos_factor = pss->actual_res;
-    int pixel_length; 
+    int pixel_length;
 
-    if (pss->pdev->model == PRISA5000) 
+    switch (pss->pdev->model)
     {
+    case PRISA5000:
         pos_factor = 600;
+        break;
+    case PERFECTION1670:
+        pos_factor = 800;
+        break;
+    default:
+        break;
     }
-
     pixel_length = pos_factor * 8.5;
 
     if(is_colour_mode(actual_mode(pss))) {
@@ -244,6 +194,7 @@ static inline int calibration_line_length(SnapScan_Scanner *pss)
 static SnapScan_Device *first_device = NULL;        /* device list head */
 static SANE_Int n_devices = 0;                           /* the device count */
 static SANE_Char *default_firmware_filename;
+static SANE_Bool cancelRead;
 
 /* list returned from sane_get_devices() */
 static const SANE_Device **get_devices_list = NULL;
@@ -257,10 +208,13 @@ static const SANE_Device **get_devices_list = NULL;
 /* Initialize gamma tables */
 static SANE_Status init_gamma(SnapScan_Scanner * ps)
 {
+    static const char me[] = "init_gamma";
     u_char *gamma;
-    int bpp = (ps->hconfig & HCFG_ADC) ? 10 : 8;
 
-    ps->gamma_length = 1 << bpp;
+    ps->gamma_length = 1 << ps->bpp;
+    DBG (DL_MINOR_INFO, "%s: using 4*%d bytes for gamma table\n",
+         me,
+         ps->gamma_length);
 
     ps->gamma_tables =
         (SANE_Int *) malloc(4 * ps->gamma_length * sizeof(SANE_Int));
@@ -284,16 +238,16 @@ static SANE_Status init_gamma(SnapScan_Scanner * ps)
     ps->gamma_table_b = &ps->gamma_tables[3 * ps->gamma_length];
 
     /* Default tables */
-    gamma_n (ps->gamma_gs, ps->bright, ps->contrast, gamma, bpp);
+    gamma_n (SANE_UNFIX(ps->gamma_gs), ps->bright, ps->contrast, gamma, ps->bpp);
     gamma_to_sane (ps->gamma_length, gamma, ps->gamma_table_gs);
 
-    gamma_n (ps->gamma_r, ps->bright, ps->contrast, gamma, bpp);
+    gamma_n (SANE_UNFIX(ps->gamma_r), ps->bright, ps->contrast, gamma, ps->bpp);
     gamma_to_sane (ps->gamma_length, gamma, ps->gamma_table_r);
 
-    gamma_n (ps->gamma_g, ps->bright, ps->contrast, gamma, bpp);
+    gamma_n (SANE_UNFIX(ps->gamma_g), ps->bright, ps->contrast, gamma, ps->bpp);
     gamma_to_sane (ps->gamma_length, gamma, ps->gamma_table_g);
 
-    gamma_n (ps->gamma_b, ps->bright, ps->contrast, gamma, bpp);
+    gamma_n (SANE_UNFIX(ps->gamma_b), ps->bright, ps->contrast, gamma, ps->bpp);
     gamma_to_sane (ps->gamma_length, gamma, ps->gamma_table_b);
 
     free (gamma);
@@ -491,23 +445,6 @@ static SANE_Status snapscani_init_device_structure(
     (*pd)->dev.type = strdup (SNAPSCAN_TYPE);
     (*pd)->bus = bus_type;
     (*pd)->model = model_num;
-    switch (model_num)
-    {
-    case SNAPSCAN:
-    case SNAPSCAN300:
-        (*pd)->depths = depths8;
-        break;
-    case PRISA620S:
-        (*pd)->depths = depths12;
-        break;
-    case PRISA4300_2:
-    case PRISA5300:
-        (*pd)->depths = depths14;
-        break;
-    default:
-        (*pd)->depths = depths10;
-        break;
-    }
 
     if (!(*pd)->dev.name  ||  !(*pd)->dev.vendor  ||  !(*pd)->dev.model  ||  !(*pd)->dev.type)
     {
@@ -546,6 +483,10 @@ static SANE_Status add_scsi_device (SANE_String_Const full_name)
     DBG (DL_CALL_TRACE, "%s(%s)\n", me, full_name);
 
     sanei_config_get_string(full_name, &name);
+    if (!name)
+    {
+    	return SANE_STATUS_INVAL;
+    }
     /* Avoid adding the same device more then once */
     if (device_already_in_list (first_device, name)) {
         free(name);
@@ -600,6 +541,10 @@ static SANE_Status add_usb_device (SANE_String_Const full_name) {
 
     DBG (DL_CALL_TRACE, "%s(%s)\n", me, full_name);
     sanei_config_get_string(full_name, &name);
+    if (!name)
+    {
+    	return SANE_STATUS_INVAL;
+    }
     /* Avoid adding the same device more then once */
     if (device_already_in_list (first_device, name)) {
         free(name);
@@ -728,6 +673,7 @@ SANE_Status sane_init (SANE_Int *version_code,
     n_devices = 0;
 
     sanei_usb_init();
+    sanei_thread_init();
 
     /* build a device structure */
     fp = sanei_config_open (SNAPSCAN_CONFIG_FILE);
@@ -975,8 +921,40 @@ SANE_Status sane_open (SANE_String_Const name, SANE_Handle * h)
             free (pss);
             return status;
         }
-        close_scanner (pss);
 
+        /* Download Firmware for USB scanners */
+        if ( (pss->pdev->bus == USB) && (pss->hwst & 0x02) )
+        {
+            char model[17];
+
+            status = download_firmware(pss);
+            CHECK_STATUS (status, me, "download_firmware");
+            /* send inquiry command again, wait for scanner to initialize */
+            status = wait_scanner_ready(pss);
+            CHECK_STATUS (status, me, "wait_scanner_ready after firmware upload");
+            status =  inquiry (pss);
+            CHECK_STATUS (status, me, "inquiry after firmware upload");
+            /* The model identifier may change after firmware upload */
+            memcpy (model, &pss->buf[INQUIRY_PRODUCT], 16);
+            model[16] = 0;
+            remove_trailing_space(model);
+            DBG (DL_INFO,
+                "%s (after firmware upload): Checking if \"%s\" is a supported scanner\n",
+                me,
+                model);
+            /* Check if it is one of our supported models */
+            pss->pdev->model = snapscani_get_model_id(model, pss->fd, pss->pdev->bus);
+
+            if (pss->pdev->model == UNKNOWN) {
+                DBG (DL_MINOR_ERROR,
+                    "%s (after firmware upload): \"%s\" is not a supported scanner\n",
+                    me,
+                    model);
+            }
+        }
+        close_scanner(pss);
+
+        init_options (pss);
         status = init_gamma (pss);
         if (status != SANE_STATUS_GOOD)
         {
@@ -988,7 +966,6 @@ SANE_Status sane_open (SANE_String_Const name, SANE_Handle * h)
             return status;
         }
 
-        init_options (pss);
         pss->state = ST_IDLE;
     }
 
@@ -1101,14 +1078,6 @@ SANE_Status sane_get_parameters (SANE_Handle h,
 
 /* scan data reader routine for child process */
 
-static void handler (int signo)
-{
-    signal (signo, handler);
-    DBG (DL_MINOR_INFO, "child process: received signal %ld\n", (long) signo);
-    close (STDOUT_FILENO);
-    _exit(0);
-}
-
 #define READER_WRITE_SIZE 4096
 
 static void reader (SnapScan_Scanner *pss)
@@ -1124,28 +1093,27 @@ static void reader (SnapScan_Scanner *pss)
     if (wbuf == NULL)
     {
         DBG (DL_MAJOR_ERROR, "%s: failed to allocate write buffer.\n", me);
-        _exit(1);
+        return;
     }
 
-    while (pss->psrc->remaining(pss->psrc) > 0)
+    while ((pss->preadersrc->remaining(pss->preadersrc) > 0) && !cancelRead)
     {
         SANE_Int ndata = READER_WRITE_SIZE;
-        status = pss->psrc->get(pss->psrc, wbuf, &ndata);
+        status = pss->preadersrc->get(pss->preadersrc, wbuf, &ndata);
         if (status != SANE_STATUS_GOOD)
         {
             DBG (DL_MAJOR_ERROR,
                  "%s: %s on read.\n",
                  me,
                  sane_strstatus (status));
-            close (STDOUT_FILENO);
-            _exit (1);
+            return;
         }
         {
             SANE_Byte *buf = wbuf;
-            DBG (DL_DATA_TRACE, "READ %d BYTES\n", ndata);
+            DBG (DL_DATA_TRACE, "READ %d BYTES (%d)\n", ndata, cancelRead);
             while (ndata > 0)
             {
-                int written = write (STDOUT_FILENO, buf, ndata);
+                int written = write (pss->rpipe[1], buf, ndata);
                 DBG (DL_DATA_TRACE, "WROTE %d BYTES\n", written);
                 if (written == -1)
                 {
@@ -1164,6 +1132,74 @@ static void reader (SnapScan_Scanner *pss)
     }
 }
 
+/** signal handler to kill the child process
+ */
+static RETSIGTYPE usb_reader_process_sigterm_handler( int signo )
+{
+    DBG( DL_INFO, "(SIG) reader_process: terminated by signal %d\n", signo );
+    cancelRead = SANE_TRUE;
+}
+
+static RETSIGTYPE sigalarm_handler( int signo UNUSEDARG)
+{
+    DBG( DL_INFO, "ALARM!!!\n" );
+}
+
+/** executed as a child process
+ * read the data from the driver and send them to the parent process
+ */
+static int reader_process( void *args )
+{
+    SANE_Status      status;
+    struct SIGACTION act;
+    sigset_t         ignore_set;
+    SnapScan_Scanner *pss = (SnapScan_Scanner *) args;
+
+    if( sanei_thread_is_forked()) {
+        DBG( DL_MINOR_INFO, "reader_process started (forked)\n" );
+        /* child process - close read side, make stdout the write side of the pipe */
+        close( pss->rpipe[0] );            
+        pss->rpipe[0] = -1;
+    } else {
+        DBG( DL_MINOR_INFO, "reader_process started (as thread)\n" );
+    }
+
+    sigfillset ( &ignore_set );
+    sigdelset  ( &ignore_set, SIGUSR1 );
+    sigprocmask( SIG_SETMASK, &ignore_set, 0 );
+
+    memset   ( &act, 0, sizeof (act));
+    sigaction( SIGTERM, &act, 0 );
+
+    cancelRead = SANE_FALSE;
+
+    /* install the signal handler */
+    sigemptyset(&(act.sa_mask));
+    act.sa_flags = 0;
+
+    act.sa_handler = usb_reader_process_sigterm_handler;
+    sigaction( SIGUSR1, &act, 0 );
+
+    status = create_base_source (pss, SCSI_SRC, &(pss->preadersrc));
+    if (status == SANE_STATUS_GOOD)
+    {
+        reader (pss);
+    }
+    else
+    {
+        DBG (DL_MAJOR_ERROR,
+                "Reader process: failed to create SCSISource.\n");
+    }
+    pss->preadersrc->done(pss->preadersrc);
+    free(pss->preadersrc);
+    pss->preadersrc = 0;
+    close( pss->rpipe[1] );            
+    pss->rpipe[1] = -1;        
+    DBG( DL_MINOR_INFO, "reader_process: finished reading data\n" );
+    return SANE_STATUS_GOOD;
+}
+
+
 static SANE_Status start_reader (SnapScan_Scanner *pss)
 {
     SANE_Status status = SANE_STATUS_GOOD;
@@ -1171,68 +1207,49 @@ static SANE_Status start_reader (SnapScan_Scanner *pss)
 
     DBG (DL_CALL_TRACE, "%s\n", me);
 
-    /* We can implement nonblocking mode by forking a separate reader
-       process. It doesn't seem that we can poll the scsi descriptor. */
-
     pss->nonblocking = SANE_FALSE;
     pss->rpipe[0] = pss->rpipe[1] = -1;
     pss->child = -1;
 
-    if (pss->pdev->model == VUEGO610S
-        ||
-        pss->pdev->model == ACER300F
-        ||
-        pss->pdev->model == SNAPSCAN310
-        ||
-        pss->pdev->model == VUEGO310S
-        ||
-        pss->pdev->model == SNAPSCANE20
-        ||
-        pss->pdev->model == SNAPSCANE50
-        ||
-        pss->pdev->model == SNAPSCAN1236)
-    {
-        status = SANE_STATUS_UNSUPPORTED;
-    }
-    else if (pipe (pss->rpipe) != -1)
+    if (pipe (pss->rpipe) != -1)
     {
         pss->orig_rpipe_flags = fcntl (pss->rpipe[0], F_GETFL, 0);
-        switch (pss->child = fork ())
+        pss->child =  sanei_thread_begin(reader_process, (void *) pss);
+        
+        cancelRead = SANE_FALSE;
+        
+        if (pss->child < 0)
         {
-        case -1:
             /* we'll have to read in blocking mode */
             DBG (DL_MAJOR_ERROR,
-                 "%s: can't fork; must read in blocking mode.\n",
+                 "%s: Error while calling sanei_thread_begin; must read in blocking mode.\n",
                  me);
             close (pss->rpipe[0]);
             close (pss->rpipe[1]);
             status = SANE_STATUS_UNSUPPORTED;
-            break;
-        case 0:
-            /* child; close read side, make stdout the write side of the pipe */
-            signal (SIGTERM, handler);
-            dup2 (pss->rpipe[1], STDOUT_FILENO);
-            close (pss->rpipe[0]);
-            status = create_base_source (pss, SCSI_SRC, &(pss->psrc));
-            if (status == SANE_STATUS_GOOD)
-            {
-                reader (pss);
-            }
-            else
-            {
-                DBG (DL_MAJOR_ERROR,
-                     "Reader process: failed to create SCSISource.\n");
-            }
-            /* regular exit may cause a SIGPIPE */
-            DBG (DL_MINOR_INFO, "Reader process terminating.\n");
-            _exit (0);
-            break;                /* not reached */
-        default:
+        }
+        if (sanei_thread_is_forked())
+        {
             /* parent; close write side */
             close (pss->rpipe[1]);
-            pss->nonblocking = SANE_TRUE;
-            break;
+            pss->rpipe[1] = -1;
         }
+        pss->nonblocking = SANE_TRUE;
+    }
+    return status;
+}
+
+static SANE_Status send_gamma_table (SnapScan_Scanner *pss, u_char dtc, u_char dtcq)
+{
+    static char me[] = "send_gamma_table";
+    SANE_Status status = SANE_STATUS_GOOD;
+    status = send (pss, dtc, dtcq);
+    CHECK_STATUS (status, me, "send");
+    if (pss->pdev->model == PERFECTION1670)
+    {
+        /* Epson Perfection 1670 needs an extra invitation */
+        status = send (pss, dtc, dtcq);
+        CHECK_STATUS (status, me, "send");
     }
     return status;
 }
@@ -1241,17 +1258,17 @@ static SANE_Status download_gamma_tables (SnapScan_Scanner *pss)
 {
     static char me[] = "download_gamma_tables";
     SANE_Status status = SANE_STATUS_GOOD;
-    float gamma_gs = SANE_UNFIX (pss->gamma_gs);
-    float gamma_r = SANE_UNFIX (pss->gamma_r);
-    float gamma_g = SANE_UNFIX (pss->gamma_g);
-    float gamma_b = SANE_UNFIX (pss->gamma_b);
+    double gamma_gs = SANE_UNFIX (pss->gamma_gs);
+    double gamma_r = SANE_UNFIX (pss->gamma_r);
+    double gamma_g = SANE_UNFIX (pss->gamma_g);
+    double gamma_b = SANE_UNFIX (pss->gamma_b);
     SnapScan_Mode mode = actual_mode (pss);
     int dtcq_gamma_gray;
     int dtcq_gamma_red;
     int dtcq_gamma_green;
     int dtcq_gamma_blue;
-    int bpp = (pss->hconfig & HCFG_ADC) ? 10 : 8;
 
+    DBG (DL_CALL_TRACE, "%s\n", me);
     switch (mode)
     {
     case MD_COLOUR:
@@ -1273,19 +1290,27 @@ static SANE_Status download_gamma_tables (SnapScan_Scanner *pss)
         break;
     }
 
-    if (bpp == 10)
+    DBG (DL_DATA_TRACE, "%s: Sending gamma table for %d bpp\n", me, pss->bpp);
+    switch (pss->bpp)
     {
+    case 10:
         dtcq_gamma_gray = DTCQ_GAMMA_GRAY10;
         dtcq_gamma_red = DTCQ_GAMMA_RED10;
         dtcq_gamma_green = DTCQ_GAMMA_GREEN10;
         dtcq_gamma_blue = DTCQ_GAMMA_BLUE10;
-    }
-    else
-    {
+        break;
+    case 14:
+        dtcq_gamma_gray = DTCQ_GAMMA_GRAY14;
+        dtcq_gamma_red = DTCQ_GAMMA_RED14;
+        dtcq_gamma_green = DTCQ_GAMMA_GREEN14;
+        dtcq_gamma_blue = DTCQ_GAMMA_BLUE14;
+        break;
+    default:
         dtcq_gamma_gray = DTCQ_GAMMA_GRAY8;
         dtcq_gamma_red = DTCQ_GAMMA_RED8;
         dtcq_gamma_green = DTCQ_GAMMA_GREEN8;
         dtcq_gamma_blue = DTCQ_GAMMA_BLUE8;
+        break;
     }
 
     if (is_colour_mode(mode))
@@ -1297,34 +1322,34 @@ static SANE_Status download_gamma_tables (SnapScan_Scanner *pss)
                 /* Use greyscale gamma for all rgb channels */
                 gamma_from_sane (pss->gamma_length, pss->gamma_table_gs,
                                  pss->buf + SEND_LENGTH);
-                status = send (pss, DTC_GAMMA, dtcq_gamma_red);
+                status = send_gamma_table(pss, DTC_GAMMA, dtcq_gamma_red);
                 CHECK_STATUS (status, me, "send");
 
                 gamma_from_sane (pss->gamma_length, pss->gamma_table_gs,
                                  pss->buf + SEND_LENGTH);
-                status = send (pss, DTC_GAMMA, dtcq_gamma_green);
+                status = send_gamma_table(pss, DTC_GAMMA, dtcq_gamma_green);
                 CHECK_STATUS (status, me, "send");
 
                 gamma_from_sane (pss->gamma_length, pss->gamma_table_gs,
                                  pss->buf + SEND_LENGTH);
-                status = send (pss, DTC_GAMMA, dtcq_gamma_blue);
+                status = send_gamma_table(pss, DTC_GAMMA, dtcq_gamma_blue);
                 CHECK_STATUS (status, me, "send");
             }
             else
             {
                 gamma_from_sane (pss->gamma_length, pss->gamma_table_r,
                                  pss->buf + SEND_LENGTH);
-                status = send (pss, DTC_GAMMA, dtcq_gamma_red);
+                status = send_gamma_table(pss, DTC_GAMMA, dtcq_gamma_red);
                 CHECK_STATUS (status, me, "send");
 
                 gamma_from_sane (pss->gamma_length, pss->gamma_table_g,
                                  pss->buf + SEND_LENGTH);
-                status = send (pss, DTC_GAMMA, dtcq_gamma_green);
+                status = send_gamma_table(pss, DTC_GAMMA, dtcq_gamma_green);
                 CHECK_STATUS (status, me, "send");
 
                 gamma_from_sane (pss->gamma_length, pss->gamma_table_b,
                                  pss->buf + SEND_LENGTH);
-                status = send (pss, DTC_GAMMA, dtcq_gamma_blue);
+                status = send_gamma_table(pss, DTC_GAMMA, dtcq_gamma_blue);
                 CHECK_STATUS (status, me, "send");
             }
         }
@@ -1334,35 +1359,35 @@ static SANE_Status download_gamma_tables (SnapScan_Scanner *pss)
             {
                 /* Use greyscale gamma for all rgb channels */
                 gamma_n (gamma_gs, pss->bright, pss->contrast,
-                         pss->buf + SEND_LENGTH, bpp);
-                status = send (pss, DTC_GAMMA, dtcq_gamma_red);
+                         pss->buf + SEND_LENGTH, pss->bpp);
+                status = send_gamma_table(pss, DTC_GAMMA, dtcq_gamma_red);
                 CHECK_STATUS (status, me, "send");
 
                 gamma_n (gamma_gs, pss->bright, pss->contrast,
-                         pss->buf + SEND_LENGTH, bpp);
-                status = send (pss, DTC_GAMMA, dtcq_gamma_green);
+                         pss->buf + SEND_LENGTH, pss->bpp);
+                status = send_gamma_table(pss, DTC_GAMMA, dtcq_gamma_green);
                 CHECK_STATUS (status, me, "send");
 
                 gamma_n (gamma_gs, pss->bright, pss->contrast,
-                         pss->buf + SEND_LENGTH, bpp);
-                status = send (pss, DTC_GAMMA, dtcq_gamma_blue);
+                         pss->buf + SEND_LENGTH, pss->bpp);
+                status = send_gamma_table(pss, DTC_GAMMA, dtcq_gamma_blue);
                 CHECK_STATUS (status, me, "send");
             }
             else
             {
                 gamma_n (gamma_r, pss->bright, pss->contrast,
-                         pss->buf + SEND_LENGTH, bpp);
-                status = send (pss, DTC_GAMMA, dtcq_gamma_red);
+                         pss->buf + SEND_LENGTH, pss->bpp);
+                status = send_gamma_table(pss, DTC_GAMMA, dtcq_gamma_red);
                 CHECK_STATUS (status, me, "send");
 
                 gamma_n (gamma_g, pss->bright, pss->contrast,
-                         pss->buf + SEND_LENGTH, bpp);
-                status = send (pss, DTC_GAMMA, dtcq_gamma_green);
+                         pss->buf + SEND_LENGTH, pss->bpp);
+                status = send_gamma_table(pss, DTC_GAMMA, dtcq_gamma_green);
                 CHECK_STATUS (status, me, "send");
 
                 gamma_n (gamma_b, pss->bright, pss->contrast,
-                         pss->buf + SEND_LENGTH, bpp);
-                status = send (pss, DTC_GAMMA, dtcq_gamma_blue);
+                         pss->buf + SEND_LENGTH, pss->bpp);
+                status = send_gamma_table(pss, DTC_GAMMA, dtcq_gamma_blue);
                 CHECK_STATUS (status, me, "send");
             }
         }
@@ -1373,14 +1398,14 @@ static SANE_Status download_gamma_tables (SnapScan_Scanner *pss)
         {
             gamma_from_sane (pss->gamma_length, pss->gamma_table_gs,
                              pss->buf + SEND_LENGTH);
-            status = send (pss, DTC_GAMMA, dtcq_gamma_gray);
+            status = send_gamma_table(pss, DTC_GAMMA, dtcq_gamma_gray);
             CHECK_STATUS (status, me, "send");
         }
         else
         {
             gamma_n (gamma_gs, pss->bright, pss->contrast,
-                     pss->buf + SEND_LENGTH, bpp);
-            status = send (pss, DTC_GAMMA, dtcq_gamma_gray);
+                     pss->buf + SEND_LENGTH, pss->bpp);
+            status = send_gamma_table(pss, DTC_GAMMA, dtcq_gamma_gray);
             CHECK_STATUS (status, me, "send");
         }
     }
@@ -1508,13 +1533,12 @@ SANE_Status sane_start (SANE_Handle h)
     status = wait_scanner_ready (pss);
     CHECK_STATUS (status, me, "wait_scanner_ready");
 
-    /* download the gamma and halftone tables */
+    /* start scanning; reserve the unit first, because a release_unit is
+       necessary to abort a scan in progress */
 
-    status = download_gamma_tables(pss);
-    CHECK_STATUS (status, me, "download_gamma_tables");
+    pss->state = ST_SCAN_INIT;
 
-    status = download_halftone_matrices(pss);
-    CHECK_STATUS (status, me, "download_halftone_matrices");
+    reserve_unit(pss);
 
     /* set up the window and fetch the resulting scanner parameters */
     status = set_window(pss);
@@ -1523,6 +1547,20 @@ SANE_Status sane_start (SANE_Handle h)
     status = inquiry(pss);
     CHECK_STATUS (status, me, "inquiry");
 
+    /* download the gamma and halftone tables */
+
+    status = download_gamma_tables(pss);
+    CHECK_STATUS (status, me, "download_gamma_tables");
+    if (pss->pdev->model == PERFECTION1670)
+    {
+        /* Epson Perfections needs the gamma table twice */
+        status = download_gamma_tables(pss);
+        CHECK_STATUS (status, me, "download_gamma_tables (2nd try)");
+    }
+
+    status = download_halftone_matrices(pss);
+    CHECK_STATUS (status, me, "download_halftone_matrices");
+
     /* we must measure the data transfer rate between the host and the
        scanner, and the method varies depending on whether there is a
        ring buffer or not. */
@@ -1542,12 +1580,6 @@ SANE_Status sane_start (SANE_Handle h)
          pss->ms_per_line,
          pss->bytes_per_line/pss->ms_per_line);
 
-    /* start scanning; reserve the unit first, because a release_unit is
-       necessary to abort a scan in progress */
-
-    pss->state = ST_SCAN_INIT;
-
-    reserve_unit(pss);
 
     if(pss->val[OPT_QUALITY_CAL].b)
     {
@@ -1567,13 +1599,18 @@ SANE_Status sane_start (SANE_Handle h)
         release_unit (pss);
         return status;
     }
-    /* Wait for scanner ready again (e.g. until paper is loaded from an ADF) */
-    status = wait_scanner_ready (pss);
-    if (status != SANE_STATUS_GOOD)
+
+    if (pss->source == SRC_ADF)
     {
-        DBG (DL_MAJOR_ERROR, "%s: scan command failed while waiting for scanner: %s.\n", me, sane_strstatus(status));
-        release_unit (pss);
-        return status;
+        /* Wait for scanner ready again (e.g. until paper is loaded from an ADF) */
+        /* Maybe replace with get_data_buffer_status()? */
+        status = wait_scanner_ready (pss);
+        if (status != SANE_STATUS_GOOD)
+        {
+            DBG (DL_MAJOR_ERROR, "%s: scan command failed while waiting for scanner: %s.\n", me, sane_strstatus(status));
+            release_unit (pss);
+            return status;
+        }
     }
 
     DBG (DL_MINOR_INFO, "%s: starting the reader process.\n", me);
@@ -1617,8 +1654,8 @@ SANE_Status sane_read (SANE_Handle h,
     {
         if (pss->child > 0)
         {
-            int status;
-            wait (&status);        /* ensure no zombies */
+            sanei_thread_waitpid (pss->child, 0);        /* ensure no zombies */
+            pss->child = -1;
         }
         release_unit (pss);
         close_scanner (pss);
@@ -1662,6 +1699,8 @@ void sane_cancel (SANE_Handle h)
 {
     char *me = "sane_snapscan_cancel";
     SnapScan_Scanner *pss = (SnapScan_Scanner *) h;
+    struct SIGACTION act;
+    pid_t            res;
 
     DBG (DL_CALL_TRACE, "%s\n", me);
     switch (pss->state)
@@ -1675,21 +1714,41 @@ void sane_cancel (SANE_Handle h)
         /* signal the reader, if any */
         if (pss->child > 0)
         {
-            int result;
-            if ((result = kill (pss->child, SIGTERM)) < 0)
+            DBG( DL_INFO, ">>>>>>>> killing reader_process <<<<<<<<\n" );
+
+            sigemptyset(&(act.sa_mask));
+            act.sa_flags = 0;
+
+            act.sa_handler = sigalarm_handler;
+            sigaction( SIGALRM, &act, 0 );
+
+            if (sanei_thread_is_forked())
             {
-                DBG (DL_VERBOSE,
-                    "%s: error: kill returns %ld\n",
-                    me,
-                    (long) result);
+                /* use SIGUSR1 to set cancelRead in child process */
+                sanei_thread_sendsig( pss->child, SIGUSR1 );
             }
             else
             {
-                int status;
-                DBG (DL_VERBOSE, "%s: waiting on child reader.\n", me);
-                wait (&status);
-                DBG (DL_VERBOSE, "%s: child has terminated.\n", me);
+                cancelRead = SANE_TRUE;
             }
+               
+            /* give'em 10 seconds 'til done...*/
+            alarm(10);
+            res = sanei_thread_waitpid( pss->child, 0 );
+            alarm(0);
+
+            if( res != pss->child ) {
+                DBG( DL_MINOR_ERROR,"sanei_thread_waitpid() failed !\n");
+
+                /* do it the hard way...*/
+#ifdef USE_PTHREAD
+                sanei_thread_kill( pss->child );
+#else
+                sanei_thread_sendsig( pss->child, SIGKILL );
+#endif
+            }
+            pss->child = -1;
+            DBG( DL_INFO,"reader_process killed\n");
         }
         release_unit (pss);
         close_scanner (pss);
@@ -1762,6 +1821,39 @@ SANE_Status sane_get_select_fd (SANE_Handle h, SANE_Int * fd)
 
 /*
  * $Log: snapscan.c,v $
+ * Revision 1.40  2004/04/09 11:59:02  oliver-guest
+ * Fixes for pthread implementation
+ *
+ * Revision 1.39  2004/04/08 21:53:10  oliver-guest
+ * Use sanei_thread in snapscan backend
+ *
+ * Revision 1.37  2003/11/27 23:11:32  oliver-guest
+ * Send gamma table twice for Epson Perfection 1670
+ *
+ * Revision 1.36  2003/11/09 21:43:45  oliver-guest
+ * Disabled quality calibration for Epson Perfection 1670
+ *
+ * Revision 1.35  2003/11/07 23:26:49  oliver-guest
+ * Final bugfixes for bascic support of Epson 1670
+ *
+ * Revision 1.34  2003/10/21 20:43:25  oliver-guest
+ * Bugfixes for SnapScan backend
+ *
+ * Revision 1.33  2003/10/07 18:29:20  oliver-guest
+ * Initial support for Epson 1670, minor bugfix
+ *
+ * Revision 1.32  2003/09/24 18:05:39  oliver-guest
+ * Bug #300198: Check second argument of sanei_config_get_string
+ *
+ * Revision 1.31  2003/09/12 16:10:33  hmg-guest
+ * Moved union Option_Value from backend header files to sanei_backend.h. No need
+ * to copy it over and over again. Changed header inclusion order in backend
+ * files to include backend.h after sanei_backend.h. Based on a patch from stef
+ * <stef-listes@wanadoo.fr>.
+ *
+ * Revision 1.30  2003/08/19 21:05:08  oliverschwartz
+ * Scanner ID cleanup
+ *
  * Revision 1.29  2003/04/30 20:49:40  oliverschwartz
  * SnapScan backend 1.4.26
  *
