@@ -1,5 +1,5 @@
 /* sane - Scanner Access Now Easy.
- 
+
    Copyright (C) 1997, 1998, 1999, 2001  Franck Schnefra, Michel Roelofs,
    Emmanuel Blot, Mikko Tyolajarvi, David Mosberger-Tang, Wolfgang Goeller,
    Petter Reinholdtsen, Gary Plewa, Sebastien Sable, Mikael Magnusson,
@@ -38,7 +38,7 @@
    If you submit changes to SANE to the maintainers to be included in
    a subsequent release, you agree by submitting the changes that
    those changes may be distributed with this exception intact.
- 
+
    If you write modifications of your own for SANE, it is your choice
    whether to permit this exception to apply to your modifications.
    If you do not wish that, delete this exception notice.
@@ -46,12 +46,14 @@
    This file is a component of the implementation of a backend for many
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
-/* $Id: snapscan.h,v 1.21 2003/04/30 20:49:40 oliverschwartz Exp $
+/* $Id: snapscan.h,v 1.28 2004/04/08 21:53:10 oliver-guest Exp $
    SANE SnapScan backend */
 
 #ifndef snapscan_h
 #define snapscan_h
 
+#include "../include/sane/sane.h"
+
 #define UNREFERENCED_PARAMETER(x)           ((void) x)
 
 /* snapscan device field values */
@@ -88,9 +90,9 @@ typedef enum
     SNAPSCANE50,        /* SnapScan e40/e50, 1200 DPI */
     SNAPSCANE52,        /* SnapScan e52, 1200 DPI, no quality calibration */
     ACER300F,
-    VUEGO310S,          /* Vuego-Version of SnapScan 310 WG changed */
-    VUEGO610S,          /* Vuego 610S and 610plus SJU changed */
-    PRISA620S,          /* Acer ScanPrisa 620 - 600 DPI */
+    PRISA310,           /* Acer ScanPrisa 310 - 300 DPI */
+    PRISA610,           /* Acer ScanPrisa 610 - 600 DPI */
+    PRISA620,           /* Acer ScanPrisa 620 - 600 DPI */
     PRISA640,           /* Acer ScanPrisa 640 - 600 DPI */
     PRISA1240,          /* Acer ScanPrisa 1240 - 1200 DPI */
     PRISA4300,          /* Acer ScanPrisa 3300/4300 - 600 DPI */
@@ -98,6 +100,7 @@ typedef enum
     PRISA5000,          /* Acer ScanPrisa 5000 - 1200 DPI */
     PRISA5300,          /* Acer ScanPrisa 5300 - 1200 DPI */
     PERFECTION660,      /* Epson Perfection 660 - 1200 DPI */
+    PERFECTION1670,     /* Epson Perfection 1670 - 1600 DPI */
     ARCUS1200		/* Agfa Arcus 1200 - 1200 DPI (rebadged Acer?) */
 } SnapScan_Model;
 
@@ -120,17 +123,18 @@ static struct SnapScan_Driver_desc drivers[] =
     {SNAPSCANE50,    "SnapScanE50"},
     {SNAPSCANE52,    "SnapScanE52"},
     {ACER300F,       "Acer300"},
-    {VUEGO310S,      "Acer310"},
-    {VUEGO610S,      "Acer610"},
-    {PRISA620S,      "Acer620"},
+    {PRISA310,       "Acer310"},
+    {PRISA610,       "Acer610"},
+    {PRISA620,       "Acer620"},
     {PRISA640,       "Acer640"},
     {PRISA4300,      "Acer4300"},
     {PRISA4300_2,    "Acer4300 (42 bit)"},
     {PRISA1240,      "Acer1240"},
     {PRISA5000,      "Acer5000"},
     {PRISA5300,      "Acer5300"},
+    {ARCUS1200,      "Arcus1200"},
     {PERFECTION660,  "Perfection 660"},
-    {ARCUS1200,      "Arcus1200"}
+    {PERFECTION1670, "Perfection 1670"}
 };
 
 #define known_drivers ((int) (sizeof(drivers)/sizeof(drivers[0])))
@@ -144,14 +148,15 @@ struct SnapScan_Model_desc
 static struct SnapScan_Model_desc scanners[] =
 {
     /* SCSI model name -> enum value */
-    {"FlatbedScanner_2",    VUEGO310S},
-    {"FlatbedScanner_4",    VUEGO610S},
-    {"FlatbedScanner_5",    PRISA620S},
-    {"FlatbedScanner_9",    PRISA620S},
-    {"FlatbedScanner13",    PRISA620S},
-    {"FlatbedScanner16",    PRISA620S},
-    {"FlatbedScanner17",    PRISA620S},
-    {"FlatbedScanner18",    PRISA620S},
+    {"FlatbedScanner_2",    PRISA310},
+    {"FlatbedScanner_4",    PRISA610},
+    {"FlatbedScanner_5",    PRISA620},
+    {"FlatbedScanner_7",    PRISA310},
+    {"FlatbedScanner_9",    PRISA620},
+    {"FlatbedScanner13",    PRISA620},
+    {"FlatbedScanner16",    PRISA620},
+    {"FlatbedScanner17",    PRISA620},
+    {"FlatbedScanner18",    PRISA620},
     {"FlatbedScanner19",    PRISA1240},
     {"FlatbedScanner20",    PRISA640},
     {"FlatbedScanner21",    PRISA4300},
@@ -177,6 +182,7 @@ static struct SnapScan_Model_desc scanners[] =
     {"SnapScan",            SNAPSCAN},
     {"ACERSCAN_A4____1",    ACER300F},
     {"Perfection 660",      PERFECTION660},
+    {"EPSON Scanner",       PERFECTION1670}, /* dummy entry to detect scanner */
     {"ARCUS 1200",          ARCUS1200}
 };
 #define known_scanners ((int) (sizeof(scanners)/sizeof(scanners[0])))
@@ -210,8 +216,9 @@ struct SnapScan_USB_Model_desc
 
 static struct SnapScan_USB_Model_desc usb_scanners[] =
 {
-    {0x04a5, 0x1a20, SNAPSCAN310},  /* Acer 310U */
-    {0x04a5, 0x2022, SNAPSCAN310}   /* Acer 320U */
+    {0x04a5, 0x1a20, PRISA310},  /* Acer 310U */
+    {0x04a5, 0x2022, PRISA310},  /* Acer 320U */
+    {0x04b8, 0x0114, PERFECTION660}    /* Epson Perfection 660 */
 };
 #define known_usb_scanners ((int) (sizeof(usb_scanners)/sizeof(usb_scanners[0])))
 
@@ -223,6 +230,7 @@ typedef enum
     OPT_PREVIEW,           /* preview mode toggle */
     OPT_MODE,              /* scan mode */
     OPT_PREVIEW_MODE,      /* preview mode */
+    OPT_HIGHQUALITY,       /* scan quality (fast / high) */
     OPT_SOURCE,            /* scan source (flatbed / TPO) */
     OPT_GEOMETRY_GROUP,    /* geometry group */
     OPT_TLX,               /* top left x */
@@ -254,15 +262,6 @@ typedef enum
     NUM_OPTS               /* dummy (gives number of options) */
 } SnapScan_Options;
 
-typedef union
-  {
-    SANE_Bool b;
-    SANE_Word w;
-    SANE_Word *wa;              /* word array */
-    SANE_String s;
-  }
-Option_Value;
-
 typedef enum
 {
     MD_COLOUR = 0,       /* full colour */
@@ -294,7 +293,6 @@ typedef struct snapscan_device
     SANE_Range y_range;           /* y dimension of scan area */
     SnapScan_Model model;         /* type of scanner */
     SnapScan_Bus bus;             /* bus of the device usb/scsi */
-    u_char *depths;               /* bit depth table */
     SANE_Char *firmware_filename; /* The name of the firmware file for USB scanners */
     struct snapscan_device *pnext;
 }
@@ -332,6 +330,7 @@ struct snapscan_scanner
     size_t bytes_per_line;        /* bytes per scan line */
     size_t pixels_per_line;       /* pixels per scan line */
     u_char hconfig;               /* hardware configuration byte */
+    u_char hwst;                  /* hardware status byte */
     float ms_per_line;            /* speed: milliseconds per scan line */
     SANE_Bool nonblocking;        /* wait on reads for data? */
     char *sense_str;              /* sense string */
@@ -341,10 +340,13 @@ struct snapscan_scanner
     SANE_Byte chroma_offset[3];   /* chroma offsets */
     SANE_Int chroma;
     Source *psrc;                 /* data source */
+    Source *preadersrc;                 /* data source for reader thread */
     SANE_Option_Descriptor options[NUM_OPTS];  /* the option descriptors */
     Option_Value val[NUM_OPTS];  /* the options themselves... */
     SANE_Int res;                /* resolution */
+    SANE_Int bpp;                /* bit depth */
     SANE_Bool preview;           /* preview mode toggle */
+    SANE_Bool highquality;       /* high quality mode toggle */
     SANE_String mode_s;          /* scanning mode */
     SANE_String source_s;        /* scanning source */
     SANE_String preview_mode_s;  /* scanning mode for preview */
@@ -377,6 +379,30 @@ struct snapscan_scanner
 
 /*
  * $Log: snapscan.h,v $
+ * Revision 1.28  2004/04/08 21:53:10  oliver-guest
+ * Use sanei_thread in snapscan backend
+ *
+ * Revision 1.27  2004/03/22 00:00:40  oliver-guest
+ * Added detection for Epson 660 by USB ID since new models use new ID strings
+ *
+ * Revision 1.26  2003/10/21 20:43:25  oliver-guest
+ * Bugfixes for SnapScan backend
+ *
+ * Revision 1.25  2003/10/07 19:41:34  oliver-guest
+ * Updates for Epson Perfection 1670
+ *
+ * Revision 1.24  2003/10/07 18:29:20  oliver-guest
+ * Initial support for Epson 1670, minor bugfix
+ *
+ * Revision 1.23  2003/09/12 16:10:33  hmg-guest
+ * Moved union Option_Value from backend header files to sanei_backend.h. No need
+ * to copy it over and over again. Changed header inclusion order in backend
+ * files to include backend.h after sanei_backend.h. Based on a patch from stef
+ * <stef-listes@wanadoo.fr>.
+ *
+ * Revision 1.22  2003/08/19 21:05:08  oliverschwartz
+ * Scanner ID cleanup
+ *
  * Revision 1.21  2003/04/30 20:49:40  oliverschwartz
  * SnapScan backend 1.4.26
  *
