@@ -1,7 +1,8 @@
 /* sane - Scanner Access Now Easy.
 
-   Copyright (C) 2002 Frank Zago (fzago at austin dot rr dot com)
-   
+   Copyright (C) 2002-2003 Frank Zago (fzago at austin dot rr dot com)
+   Copyright (C) 2003 Gerard Klaver (gerard at gkall dot hobby dot nl)
+
    This file is part of the SANE package.
    
    This program is free software; you can redistribute it and/or
@@ -42,13 +43,19 @@
 */
 
 /*
-   $Id: teco2.c,v 1.3 2002/09/20 03:02:18 fzago Exp $
-   TECO scanner VM3575, VM6565, VM6575, VM6586
+   $Id: teco2.c,v 1.10 2004/01/15 17:37:22 gekl-guest Exp $
+   TECO scanner VM3575, VM6565, VM6575, VM6586, VM356A, VM3564
+   update 2003/02/14, Patch for VM356A Gerard Klaver
+   update 2003/03/19, traces, tests VM356A Gerard Klaver, Michael Hoeller
+   update 2003/07/19, white level calibration, color modes VM3564, VM356A, VM3575
+                      Gerard Klaver, Michael Hoeller
+   update 2004/01/15, white level , red, green, and blue calibration for the 
+                      VM3564, VM356A and VM3575 Gerard Klaver
 */
 
 /*--------------------------------------------------------------------------*/
 
-#define BUILD 3			/* 2002/08/28 */
+#define BUILD 5			/* 2003/04/18 */
 #define BACKEND_NAME teco2
 #define TECO2_CONFIG_FILE "teco2.conf"
 
@@ -78,6 +85,7 @@
 
 #include "teco2.h"
 
+/* This is used to debug the backend without having the real hardware. */
 #undef sim
 #ifdef sim
 #define sanei_scsi_cmd2(a, b, c, d, e, f, g) SANE_STATUS_GOOD
@@ -86,6 +94,8 @@
 #define sanei_scsi_close(a)   SANE_STATUS_GOOD
 #endif
 
+/* For debugging purposes: output a stream straight out from the
+ * scanner without reordering the colors, 0=normal, 1 = raw. */
 static int raw_output = 0;
 
 /*--------------------------------------------------------------------------*/
@@ -150,28 +160,117 @@ static const SANE_Range threshold_range = {
 
 /*--------------------------------------------------------------------------*/
 
+static const SANE_Range red_level_range = {
+  0,				/* minimum */
+  64,				/* maximum */
+  1				/* quantization */
+};
+
+/*--------------------------------------------------------------------------*/
+
+static const SANE_Range green_level_range = {
+  0,				/* minimum */
+  64,				/* maximum */
+  1				/* quantization */
+};
+
+/*--------------------------------------------------------------------------*/
+
+static const SANE_Range blue_level_range = {
+  0,				/* minimum */
+  64,				/* maximum */
+  1				/* quantization */
+};
+
+/*--------------------------------------------------------------------------*/
+
 /* Gamma range */
 static const SANE_Range gamma_range = {
   0,				/* minimum */
-  255,				/* maximum */
+  255,				/* 255 maximum */
   0				/* quantization */
 };
 
 /*--------------------------------------------------------------------------*/
 
+static const struct dpi_color_adjust vm3564_dpi_color_adjust[] = {
+
+  /*dpi, color sequence R G or B, 0 (-) or 1 (+) color skewing, lines skewing */
+  {25, 1, 0, 2, 0, 0},		/* Not tested */
+  {50, 1, 2, 0, 0, 1},		/* Not tested */
+  {75, 1, 0, 2, 1, 1},		/* Not tested */
+  {150, 1, 0, 2, 1, 2},		/* Not tested */
+  {160, 1, 0, 2, 1, 2},		/* Not tested */
+  {225, 1, 0, 2, 1, 3},		/* Not tested */
+  {300, 1, 0, 2, 1, 4},		/* Not tested */
+  {375, 1, 0, 2, 1, 5},		/* Not tested */
+  {450, 1, 0, 2, 1, 6},		/* Not tested */
+  {525, 1, 0, 2, 1, 7},		/* Not tested */
+  {600, 1, 0, 2, 1, 8},		/* Not tested */
+  /* must be the last entry */
+  {0, 0, 0, 0, 0, 0}
+};
+
+/* Used for VM356A only */
+
+static const struct dpi_color_adjust vm356a_dpi_color_adjust[] = {
+
+  /* All values with ydpi > 300 result in a wrong proportion for       */
+  /* the scan. The proportion can be adjusted with the following       */
+  /* command: convert -geometry (dpi/max_xdpi * 100%)x100%             */
+  /* max_xdpi is for the vm356a constant with 300 dpi                  */
+  /* e.g. 600dpi adjust with: convert -geometry 200%x100%              */
+
+  /* All resolutions in increments of 25 are testede, only the shown   */
+  /* bring back good results                                           */
+
+  /*dpi, color sequence R G or B, 0 (-) or 1 (+) color skewing, lines skewing */
+  /* some values disabled because of choice of nearby values to keep usertable short */
+  {25, 1, 0, 2, 0, 0},
+  {50, 1, 2, 0, 0, 1},
+  {75, 1, 0, 2, 1, 1},
+  {150, 1, 0, 2, 1, 2},
+/* {160, 1, 0, 2, 1, 2},		ok */
+  {225, 1, 0, 2, 1, 3},
+  {300, 1, 0, 2, 1, 4},
+/* {305, 1, 0, 2, 1, 4},		ok, *
+ * {310, 1, 0, 2, 1, 4},		ok, *
+ * {315, 1, 0, 2, 1, 4},   	        ok, */
+  {375, 1, 0, 2, 1, 5},
+/* {380, 1, 0, 2, 1, 5},		ok, *
+ * {385, 1, 0, 2, 1, 5},		ok, */
+  {450, 1, 0, 2, 1, 6},
+/* {455, 1, 0, 2, 1, 6},		ok, */
+  {525, 1, 0, 2, 1, 7},
+  {600, 1, 0, 2, 1, 8},
+
+  /* must be the last entry */
+  {0, 0, 0, 0, 0, 0}
+};
+
 /* Used for the VM3575 only */
 
 static const struct dpi_color_adjust vm3575_dpi_color_adjust[] = {
 
+  /* All values with ydpi > 300 result in a wrong proportion for */
+  /* the scan. The proportion can be adjusted with the following */
+  /* command: convert -geometry (dpi/max_xdpi * 100%)x100%       */
+  /* max_xdpi is for the vm3575 constant with 300 dpi            */
+  /* e.g. 600dpi adjust with: convert -geometry 200%x100%        */
+
   {50, 2, 0, 1, 1, 1},
   {60, 2, 1, 0, 0, 2},
-  {75, 2, 0, 1, 1, 2},
+/* {75, 2, 0, 1, 1, 2},           NOK, effects in scan, also with twain driver*/
   {100, 2, 1, 0, 0, 3},
   {120, 2, 0, 1, 1, 3},
   {150, 2, 0, 1, 1, 4},
-  {180, 2, 1, 0, 0, 4},
-  {225, 2, 0, 1, 0, 6},
+/* {180, 2, 1, 0, 0, 4},          NOK, skewing problem*/
+  {225, 2, 0, 1, 1, 6},
   {300, 2, 0, 1, 1, 8},
+  {375, 2, 0, 1, 1, 10},
+  {450, 2, 0, 1, 1, 12},
+  {525, 2, 0, 1, 1, 14},
+  {600, 2, 0, 1, 1, 16},
 
   /* must be the last entry */
   {0, 0, 0, 0, 0, 0}
@@ -181,8 +280,11 @@ static const struct dpi_color_adjust vm656a_dpi_color_adjust[1] = {
   {0, 0, 1, 2, 0, 0}
 };
 
-static const struct dpi_color_adjust vm6575_dpi_color_adjust[1] = {
-  {0, 1, 0, 2, 0, 0}
+static const struct dpi_color_adjust vm6575_dpi_color_adjust[] = {
+  {75, 1, 0, 2, 1, 1},
+  {150, 1, 0, 2, 1, 2},
+  {300, 1, 0, 2, 1, 4},
+  {600, 1, 0, 2, 1, 8},
 };
 
 static const struct dpi_color_adjust vm6586_dpi_color_adjust[] = {
@@ -224,39 +326,105 @@ static const struct dpi_color_adjust default_dpi_color_adjust[1] = {
 
 /* Define the supported scanners and their characteristics. */
 static const struct scanners_supported scanners[] = {
+
+  {6, "TECO VM3564",
+   TECO_VM3564,
+   "Relisys", "AVEC II S3",
+   {1, 600, 1},			/* resolution */
+   300, 600,			/* max x and Y resolution */
+   2550, 12, 3, 0,		/* calibration */
+   {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
+   {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
+   vm3564_dpi_color_adjust},
+
+  {6, "TECO VM356A",
+   TECO_VM356A,
+   "Relisys", "APOLLO Express 3",
+   {1, 600, 1},			/* resolution */
+   300, 600,			/* max x and Y resolution */
+   2550, 12, 3, 1,		/* calibration */
+   /* dots/inch * x-length, calibration samples, tot.bytes for 3 colors, default SANE_TECO2_CAL_ALGO value */
+   {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
+   {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
+   vm356a_dpi_color_adjust},
+
+  {6, "TECO VM356A",
+   TECO_VM356A,
+   "Primax", "Jewel 4800",
+   {1, 600, 1},			/* resolution */
+   300, 600,			/* max x and Y resolution */
+   2550, 12, 3, 1,		/* calibration */
+   /* dots/inch * x-length, calibration samples, tot.bytes for 3 colors, default SANE_TECO2_CAL_ALGO value */
+   {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
+   {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
+   vm356a_dpi_color_adjust},
+
   {6, "TECO VM3575",
    TECO_VM3575,
    "Relisys", "SCORPIO Super 3",
    {1, 600, 1},			/* resolution */
    300, 600,			/* max x and Y resolution */
-   2550, 12,			/* calibration */
+   2550, 12, 6, 1,		/* calibration */
    {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
    {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
    vm3575_dpi_color_adjust},
+
+  {6, "TECO VM3575",
+   TECO_VM3575,
+   "Relisys", "AVEC Super 3",
+   {1, 600, 1},			/* resolution */
+   300, 600,			/* max x and Y resolution */
+   2550, 12, 6, 1,		/* calibration */
+   {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
+   {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
+   vm3575_dpi_color_adjust},
+
   {6, "TECO VM656A",
    TECO_VM656A,
    "Relisys", "APOLLO Express 6",
    {1, 600, 1},			/* resolution */
    600, 1200,			/* max x and Y resolution */
-   5100, 8,			/* calibration */
+   5100, 8, 6, 0,		/* calibration */
    {SANE_FIX (0), SANE_FIX (210), 0},
    {SANE_FIX (0), SANE_FIX (297), 0},
    vm656a_dpi_color_adjust},
+
   {6, "TECO VM6575",
    TECO_VM6575,
    "Relisys", "SCORPIO Pro",
    {1, 600, 1},			/* resolution */
    600, 1200,			/* max x and Y resolution */
-   5100, 8,			/* calibration */
+   5100, 8, 6, 0,		/* calibration */
    {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
    {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
    vm6575_dpi_color_adjust},
+
+  {6, "TECO VM6575",
+   TECO_VM6575,
+   "Primax", "Profi 9600",
+   {1, 600, 1},			/* resolution */
+   600, 1200,			/* max x and Y resolution */
+   5100, 8, 6, 0,		/* calibration */
+   {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
+   {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
+   vm6575_dpi_color_adjust},
+
   {6, "TECO VM6586",
    TECO_VM6586,
    "Relisys", "SCORPIO Pro-S",
    {1, 600, 1},			/* resolution */
    600, 1200,			/* max x and Y resolution */
-   5100, 8,			/* calibration */
+   5100, 8, 6, 0,		/* calibration */
+   {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
+   {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
+   vm6586_dpi_color_adjust},
+
+  {6, "TECO VM6586",
+   TECO_VM6586,
+   "Primax", "Profi 19200",
+   {1, 600, 1},			/* resolution */
+   600, 1200,			/* max x and Y resolution */
+   5100, 8, 6, 0,		/* calibration */
    {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
    {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
    vm6586_dpi_color_adjust}
@@ -494,6 +662,32 @@ teco_identify_scanner (Teco_Scanner * dev)
 #ifdef sim
   {
 #if 0
+    /* vm3564 */
+    unsigned char table[] = {
+      0x06, 0x00, 0x02, 0x02, 0x43, 0x00, 0x00, 0x10, 0x52, 0x45,
+      0x4c, 0x49, 0x53, 0x59, 0x53, 0x20, 0x41, 0x56, 0x45, 0x43,
+      0x20, 0x49, 0x49, 0x20, 0x53, 0x33, 0x20, 0x20, 0x20, 0x20,
+      0x20, 0x20, 0x31, 0x2e, 0x30, 0x37, 0x31, 0x2e, 0x30, 0x37,
+      0x00, 0x01, 0x54, 0x45, 0x43, 0x4f, 0x20, 0x56, 0x4d, 0x33,
+      0x35, 0x36, 0x34, 0x20, 0x00, 0x01, 0x01, 0x2c, 0x00, 0x01,
+      0x02, 0x58, 0x09, 0xf6, 0x0d, 0xaf, 0x01, 0x2c, 0x00, 0x08,
+      0x01, 0x00
+    };
+#endif
+#if 0
+    /* vm356A */
+    unsigned char table[] = {
+      0x06, 0x00, 0x02, 0x02, 0x43, 0x00, 0x00, 0x10, 0x50, 0x72,
+      0x69, 0x6d, 0x61, 0x78, 0x20, 0x20, 0x4a, 0x65, 0x77, 0x65,
+      0x6c, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
+      0x20, 0x20, 0x31, 0x2e, 0x30, 0x30, 0x31, 0x2e, 0x30, 0x30,
+      0x00, 0x01, 0x54, 0x45, 0x43, 0x4f, 0x20, 0x56, 0x4d, 0x33,
+      0x35, 0x36, 0x41, 0x20, 0x00, 0x01, 0x01, 0x2c, 0x00, 0x01,
+      0x02, 0x58, 0x09, 0xf6, 0x0d, 0xaf, 0x01, 0x2c, 0x00, 0x08,
+      0x10, 0x00
+    };
+#endif
+#if 1
     /* vm3575 */
     unsigned char table[] = {
       0x06, 0x00, 0x02, 0x02, 0x43, 0x00, 0x00, 0x00, 0x20, 0x20,
@@ -506,7 +700,7 @@ teco_identify_scanner (Teco_Scanner * dev)
       0x01, 0x00
     };
 #endif
-#if 1
+#if 0
     /* vm6586 */
     unsigned char table[] = {
       0x06, 0x00, 0x02, 0x02, 0x43, 0x00, 0x00, 0x00, 0x20, 0x20,
@@ -553,13 +747,13 @@ teco_identify_scanner (Teco_Scanner * dev)
   hexdump (DBG_info2, "inquiry", dev->buffer, size);
 
   dev->scsi_type = dev->buffer[0] & 0x1f;
-  strncpy (dev->scsi_vendor, dev->buffer + 0x08, 0x08);
+  memcpy (dev->scsi_vendor, dev->buffer + 0x08, 0x08);
   dev->scsi_vendor[0x08] = 0;
-  strncpy (dev->scsi_product, dev->buffer + 0x10, 0x010);
+  memcpy (dev->scsi_product, dev->buffer + 0x10, 0x010);
   dev->scsi_product[0x10] = 0;
-  strncpy (dev->scsi_version, dev->buffer + 0x20, 0x04);
+  memcpy (dev->scsi_version, dev->buffer + 0x20, 0x04);
   dev->scsi_version[0x04] = 0;
-  strncpy (dev->scsi_teco_name, dev->buffer + 0x2A, 0x0B);
+  memcpy (dev->scsi_teco_name, dev->buffer + 0x2A, 0x0B);
   dev->scsi_teco_name[0x0B] = 0;
 
   DBG (DBG_info, "device is \"%s\" \"%s\" \"%s\" \"%s\"\n",
@@ -570,16 +764,36 @@ teco_identify_scanner (Teco_Scanner * dev)
    * backend supports that one. */
   for (i = 0; i < NELEMS (scanners); i++)
     {
-
       if (dev->scsi_type == scanners[i].scsi_type &&
 	  strcmp (dev->scsi_teco_name, scanners[i].scsi_teco_name) == 0)
 	{
 
 	  DBG (DBG_error, "teco_identify_scanner: scanner supported\n");
 
-	  dev->def = &(scanners[i]);
+	  /*patch for VM356A Jewel or Apollo text in frontend-+ others-------------- */
+
+	  if (strncmp (dev->scsi_vendor, "RELISYS", 1) == 0 ||
+	      strncmp (dev->scsi_vendor, "       ", 1) == 0)
+	    {
+	      DBG (DBG_error,
+		   "teco_identify_scanner: scanner detected with this teco_name and first brand/name entry in table\n");
+
+	      dev->def = &(scanners[i]);
+
+	      return (SANE_TRUE);
+	    }
+	  else
+	    {
+	      i++;
+
+	      DBG (DBG_error,
+		   "teco_identify_scanner: scanner detected with this teco_name and second brand/name entry in table\n");
+
+	      dev->def = &(scanners[i]);
+
+	      return (SANE_TRUE);
+	    }
 
-	  return (SANE_TRUE);
 	}
     }
 
@@ -607,6 +821,8 @@ teco_set_window (Teco_Scanner * dev)
     case TECO_VM3575:
       size = 53;
       break;
+    case TECO_VM3564:
+    case TECO_VM356A:
     case TECO_VM656A:
     case TECO_VM6575:
     case TECO_VM6586:
@@ -878,12 +1094,27 @@ teco_wait_for_data (Teco_Scanner * dev)
 
       hexdump (DBG_info2, "teco_wait_for_data return", dev->buffer, size);
 
-      if (dev->buffer[11] == 0x80)
+      switch (dev->def->tecoref)
 	{
-	  return SANE_STATUS_GOOD;
-	}
+	case TECO_VM3564:
+	case TECO_VM356A:
+	  if (dev->buffer[11] > 0x01)
+	    {
+	      return SANE_STATUS_GOOD;
+	    }
 
-      sleep (1);
+	  sleep (1);
+	  break;
+	  /* For VM3575, VM6575, VM656A, VM6586 until otherwise default value is used */
+	default:
+	  if (dev->buffer[11] == 0x80)
+	    {
+	      return SANE_STATUS_GOOD;
+	    }
+
+	  sleep (1);
+	  break;
+	}
     }
 
   DBG (DBG_proc, "teco_wait_for_data: scanner not ready to send data (%d)\n",
@@ -897,10 +1128,22 @@ teco_wait_for_data (Teco_Scanner * dev)
  * big enough for one line, read the 12 lines of calibration, and do a
  * simple average. The input line is 1 raster for each color. However
  * the output line is interlaced (ie RBG for the first pixel, then RGB
- * for the second, and so on...). The outpur line are the value to use
- * to compensate for the white point. The range goes from 0 to 0xfff,
- * and the average is 0x800. So if the average input is 0x700, the
- * output value for that dot must be 0x1000-0x700=0x900. */
+ * for the second, and so on...). The output line are the value to use
+ * to compensate for the white point. 
+ * There is two algorithms:
+ *
+ *   The range goes from 0 to 0xfff, and the average is 0x800. So if
+ *   the average input is 0x700, the output value for that dot must be
+ *   0x1000-0x700=0x900.
+ * 
+ * and
+ *
+ *   the calibration needs to be a multiplication factor, to
+ *   compensate for the too strong or too weak pixel in the sensor.
+ *
+ * See more info in doc/teco/teco2.txt
+ *
+ **/
 static SANE_Status
 teco_do_calibration (Teco_Scanner * dev)
 {
@@ -909,15 +1152,93 @@ teco_do_calibration (Teco_Scanner * dev)
   size_t size;
   int i;
   int j;
-  int *tmp_buf;			/* hold the temporary calibration */
-  size_t tmp_buf_size;
+  int colsub0, colsub1;
+  int colsub0_0, colsub0_1, colsub0_2;
+  int colsub1_0, colsub1_1, colsub1_2;
+  int *tmp_buf, *tmp_min_buf, *tmp_max_buf;			/* hold the temporary calibration */
+  size_t tmp_buf_size, tmp_min_buf_size, tmp_max_buf_size;
+  const char *calibration_algo;
+  int cal_algo;
 
+  colsub0 = 0;
+  colsub1 = 0;
+  colsub0_0 = 0;
+  colsub0_1 = 0;
+  colsub0_2 = 0;
+  colsub1_0 = 0;
+  colsub1_1 = 0;
+  colsub1_2 = 0;
+	
   DBG (DBG_proc, "teco_do_calibration: enter\n");
 
+  /* Get default calibration algorithm. */
+  cal_algo = dev->def->cal_algo;
+  if ((calibration_algo = getenv ("SANE_TECO2_CAL_ALGO")) != NULL)
+    {
+      cal_algo = atoi (calibration_algo);
+    }
+  if (cal_algo != 0 && cal_algo != 1)
+    {
+      DBG (DBG_error, "Invalid calibration algorithm (%d)\n", cal_algo);
+      cal_algo = 0;
+    }
+
+  switch (dev->def->tecoref)
+    {
+    case TECO_VM3564:
+    case TECO_VM356A:
+      /* white level red, green and blue calibration correction */
+      /* 0x110 or 272 is middle value */
+      colsub0_1 = 240 + (dev->val[OPT_WHITE_LEVEL_R].w);
+      colsub0_2 = 240 + (dev->val[OPT_WHITE_LEVEL_G].w);
+      colsub0_0 = 240 + (dev->val[OPT_WHITE_LEVEL_B].w);
+      /* 14000 is middle value */
+      colsub1_1 = 12720 + (40 * dev->val[OPT_WHITE_LEVEL_R].w);
+      colsub1_2 = 12720 + (40 * dev->val[OPT_WHITE_LEVEL_G].w);
+      colsub1_0 = 12720 + (40 * dev->val[OPT_WHITE_LEVEL_B].w);
+      break;
+    case TECO_VM3575:
+      /* 0x1100 or 4352 is middle value */
+      colsub0_1 = 4096 + (8 * dev->val[OPT_WHITE_LEVEL_R].w);
+      colsub0_2 = 4096 + (8 * dev->val[OPT_WHITE_LEVEL_G].w);
+      colsub0_0 = 4096 + (8 * dev->val[OPT_WHITE_LEVEL_B].w);
+      /* 4206639 is middle value */
+      colsub1_1 = 4078639 + (4000 * dev->val[OPT_WHITE_LEVEL_R].w);
+      colsub1_2 = 4078639 + (4000 * dev->val[OPT_WHITE_LEVEL_G].w);
+      colsub1_0 = 4078639 + (4000 * dev->val[OPT_WHITE_LEVEL_B].w);
+      break;
+      /* For VM6575, 656A, 6586 until otherwise default value is used */
+    default:
+      colsub0 = 0x1000;
+      colsub1 = 4206639;
+      break;
+    }
+
   tmp_buf_size = dev->def->cal_length * 3 * sizeof (int);
+  tmp_min_buf_size = dev->def->cal_length * 3 * sizeof (int);
+  tmp_max_buf_size = dev->def->cal_length * 3 * sizeof (int);
   tmp_buf = malloc (tmp_buf_size);
+  tmp_min_buf = malloc (tmp_min_buf_size);
+  tmp_max_buf = malloc (tmp_max_buf_size);
   memset (tmp_buf, 0, tmp_buf_size);
-  if (tmp_buf == NULL)
+  switch (dev->def->tecoref)
+    {
+    case TECO_VM3564:
+    case TECO_VM356A:
+  	memset (tmp_min_buf, 0xff, tmp_min_buf_size);
+  	memset (tmp_max_buf, 0x00, tmp_max_buf_size);
+      break;
+    case TECO_VM3575:
+  	memset (tmp_min_buf, 0xffff, tmp_min_buf_size);
+  	memset (tmp_max_buf, 0x0000, tmp_max_buf_size);
+      break;
+    default:
+  	memset (tmp_min_buf, 0xff, tmp_min_buf_size);
+  	memset (tmp_max_buf, 0x00, tmp_max_buf_size);
+      break;
+    }
+  
+  if ((tmp_buf == NULL) || (tmp_min_buf == NULL) || (tmp_max_buf == NULL))
     {
       DBG (DBG_proc, "teco_do_calibration: not enough memory (%d bytes)\n",
 	   tmp_buf_size);
@@ -943,8 +1264,8 @@ teco_do_calibration (Teco_Scanner * dev)
 	  break;
 	}
 
-      /* Length of the scanner * 6 bytes */
-      size = dev->def->cal_length * 6;
+      /* Length of the scanner * number of bytes per color */
+      size = dev->def->cal_length * dev->def->cal_col_len;
       cdb.data[3] = (size >> 8) & 0xff;
       cdb.data[4] = (size >> 0) & 0xff;
 
@@ -975,23 +1296,149 @@ teco_do_calibration (Teco_Scanner * dev)
 
       for (j = 0; j < dev->def->cal_length; j++)
 	{
-	  tmp_buf[3 * j + 0] +=
-	    (dev->buffer[6 * j + 1] << 8) + dev->buffer[6 * j + 0];
-	  tmp_buf[3 * j + 1] +=
-	    (dev->buffer[6 * j + 3] << 8) + dev->buffer[6 * j + 2];
-	  tmp_buf[3 * j + 2] +=
-	    (dev->buffer[6 * j + 5] << 8) + dev->buffer[6 * j + 4];
+	  switch (dev->def->tecoref)
+	    {
+	    case TECO_VM6575:
+	    case TECO_VM656A:
+	    case TECO_VM6586:
+	      tmp_buf[3 * j + 0] +=
+		(dev->buffer[6 * j + 1] << 8) + dev->buffer[6 * j + 0];
+	      tmp_buf[3 * j + 1] +=
+		(dev->buffer[6 * j + 3] << 8) + dev->buffer[6 * j + 2];
+	      tmp_buf[3 * j + 2] +=
+		(dev->buffer[6 * j + 5] << 8) + dev->buffer[6 * j + 4];
+	      break;
+	    case TECO_VM3575:
+	      tmp_buf[3 * j + 0] +=
+		(dev->buffer[6 * j + 1] << 8) + dev->buffer[6 * j + 0];
+	      /* get lowest value */
+	      if (tmp_min_buf[3 * j + 0]  >> ((dev->buffer[6 * j + 1] << 8) + dev->buffer[6 * j + 0]))
+	      {
+		      tmp_min_buf[3 * j + 0] = (dev->buffer[6 * j + 1] << 8) + dev->buffer[6 * j + 0];
+	      }
+	      /* get highest value */
+	      if (tmp_max_buf[3 * j + 0]  < ((dev->buffer[6 * j + 1] << 8) + dev->buffer[6 * j + 0]))
+	      {
+		      tmp_max_buf[3 * j + 0] = (dev->buffer[6 * j + 1] << 8) + dev->buffer[6 * j + 0];
+	      }
+	      tmp_buf[3 * j + 1] +=
+		(dev->buffer[6 * j + 3] << 8) + dev->buffer[6 * j + 2];
+	      /* get lowest value */
+	      if (tmp_min_buf[3 * j + 1]  >> ((dev->buffer[6 * j + 3] << 8) + dev->buffer[6 * j + 2]))
+	      {
+		      tmp_min_buf[3 * j + 1] = (dev->buffer[6 * j + 3] << 8) + dev->buffer[6 * j + 2];
+	      }
+	      /* get highest value */
+	      if (tmp_max_buf[3 * j + 1]  < ((dev->buffer[6 * j + 3] << 8) + dev->buffer[6 * j + 2]))
+	      {
+		      tmp_max_buf[3 * j + 1] = (dev->buffer[6 * j + 3] << 8) + dev->buffer[6 * j + 2];
+	      }
+	      tmp_buf[3 * j + 2] +=
+		(dev->buffer[6 * j + 5] << 8) + dev->buffer[6 * j + 4];
+	      /* get lowest value */
+	      if (tmp_min_buf[3 * j + 2]  >> ((dev->buffer[6 * j + 5] << 8) + dev->buffer[6 * j + 4]))
+	      {
+		      tmp_min_buf[3 * j + 2] = (dev->buffer[6 * j + 5] << 8) + dev->buffer[6 * j + 4];
+	      }
+	      /* get highest value */
+	      if (tmp_max_buf[3 * j + 2]  < ((dev->buffer[6 * j + 5] << 8) + dev->buffer[6 * j + 4]))
+	      {
+		      tmp_max_buf[3 * j + 2] = (dev->buffer[6 * j + 5] << 8) + dev->buffer[6 * j + 4];
+	      }
+	      break;
+	    case TECO_VM3564:
+	    case TECO_VM356A:
+	      tmp_buf[3 * j + 0] += dev->buffer[3 * j + 0];
+	      /* get lowest value */
+	      if (tmp_min_buf[3 * j + 0]  >> dev->buffer[3 * j + 0])
+	      {
+		      tmp_min_buf[3 * j + 0] = dev->buffer[3 * j + 0];
+	      }
+	      /* get highest value */
+	      if (tmp_max_buf[3 * j + 0]  < dev->buffer[3 * j + 0]) 
+	      {
+		      tmp_max_buf[3 * j + 0] = dev->buffer[3 * j + 0];
+	      }
+	      tmp_buf[3 * j + 1] += dev->buffer[3 * j + 1];
+	      /* get lowest value */
+	      if (tmp_min_buf[3 * j + 1]  >> dev->buffer[3 * j + 1]) 
+	      {
+		      tmp_min_buf[3 * j + 1]  = dev->buffer[3 * j + 1];
+	      }
+	      /* get hightest value */
+	      if (tmp_max_buf[3 * j + 1]  < dev->buffer[3 * j + 1]) 
+	      {
+		      tmp_max_buf[3 * j + 1]  = dev->buffer[3 * j + 1];
+	      }
+	      tmp_buf[3 * j + 2] += dev->buffer[3 * j + 2];
+              /* get lowest value */
+	      if (tmp_min_buf[3 * j + 2]  >> dev->buffer[3 * j + 2]) 
+	      {
+		      tmp_min_buf[3 * j + 2]  = dev->buffer[3 * j + 2];
+	      }
+	      /* get highest value */
+	      if (tmp_max_buf[3 * j + 2]  < dev->buffer[3 * j + 2]) 
+	      {
+		      tmp_max_buf[3 * j + 2]  = dev->buffer[3 * j + 2];
+	      }
+		break;
+	    }
 	}
     }
 
-  /*hexdump (DBG_info2, "calibration before average:", tmp_buf, tmp_buf_size); */
-
+  /* hexdump (DBG_info2, "calibration before average:", tmp_buf, tmp_buf_size); */
+  /* hexdump (DBG_info2, "calibration before average min value:", tmp_min_buf, tmp_min_buf_size); */
+  /* hexdump (DBG_info2, "calibration before average max value:", tmp_max_buf, tmp_max_buf_size); */
+  
   /* Do the average. Since we got 12 lines, divide all values by 12
    * and create the final calibration value that compensates for the
-   * white values read. */
-  for (j = 0; j < (3 * dev->def->cal_length); j++)
-    {
-      tmp_buf[j] = 0x1000 - (tmp_buf[j] / dev->def->cal_lines);
+   * white values read, for VM3564 and VM356A subtract lowest and highest
+   * value and divide by 10 */
+  switch (dev->def->tecoref)
+  {
+   	case TECO_VM6575:
+	case TECO_VM656A:
+	case TECO_VM6586:
+  		for (j = 0; j < (3 * dev->def->cal_length); j++)
+		{
+      			if (cal_algo == 1)
+	  		tmp_buf[j] = (colsub1 * dev->def->cal_lines) / tmp_buf[j];
+      			else 
+	  		tmp_buf[j] = colsub0 - (tmp_buf[j] / dev->def->cal_lines);
+		}
+	      	break;
+	case TECO_VM3575:
+	case TECO_VM3564:
+	case TECO_VM356A:
+  		for (j = 0; j < dev->def->cal_length; j++)
+		{
+	       /* subtract lowest and highest value */
+			tmp_buf[j] = tmp_buf[j] - (tmp_min_buf[j] + tmp_max_buf[j]); 
+			tmp_buf[j + dev->def->cal_length] = tmp_buf[j + dev->def->cal_length] 
+				- (tmp_min_buf[j + dev->def->cal_length] 
+				+ tmp_max_buf[j + dev->def->cal_length]); 
+			tmp_buf[j + 2 * dev->def->cal_length] = tmp_buf[j + 2 * dev->def->cal_length] 
+				- (tmp_min_buf[j + 2 * dev->def->cal_length] 
+				+ tmp_max_buf[j + 2 *dev->def->cal_length]); 
+		/* sequence colors first color row one then two and last three   */
+      			if (cal_algo == 1) 
+			{
+	  		tmp_buf[j] = (colsub1_0 * (dev->def->cal_lines - 2)) / tmp_buf[j];
+	  		tmp_buf[j + dev->def->cal_length] = (colsub1_1 * (dev->def->cal_lines - 2)) 
+				/ tmp_buf[j + dev->def->cal_length];
+	  		tmp_buf[j + 2 * dev->def->cal_length] = (colsub1_2 * (dev->def->cal_lines - 2)) 
+				/ tmp_buf[j + 2 * dev->def->cal_length];
+			}
+      			else 
+			{
+	  		tmp_buf[j] = colsub0_0 - (tmp_buf[j] / (dev->def->cal_lines - 2));
+	  		tmp_buf[j + dev->def->cal_length] = colsub0_1 - (tmp_buf[j + dev->def->cal_length] 
+								/ (dev->def->cal_lines - 2));
+	  		tmp_buf[j + 2 * dev->def->cal_length] = colsub0_2 
+				- (tmp_buf[j + 2 * dev->def->cal_length] / (dev->def->cal_lines - 2));
+			}
+		}
+	      	break;
     }
 
   /*hexdump (DBG_info2, "calibration after average:", tmp_buf, tmp_buf_size); */
@@ -999,20 +1446,37 @@ teco_do_calibration (Teco_Scanner * dev)
   /* Build the calibration line to send. */
   for (j = 0; j < dev->def->cal_length; j++)
     {
-      dev->buffer[6 * j + 0] =
-	(tmp_buf[0 * dev->def->cal_length + j] >> 0) & 0xff;
-      dev->buffer[6 * j + 1] =
-	(tmp_buf[0 * dev->def->cal_length + j] >> 8) & 0xff;
 
-      dev->buffer[6 * j + 2] =
-	(tmp_buf[1 * dev->def->cal_length + j] >> 0) & 0xff;
-      dev->buffer[6 * j + 3] =
-	(tmp_buf[1 * dev->def->cal_length + j] >> 8) & 0xff;
+      switch (dev->def->tecoref)
+	{
+	case TECO_VM3575:
+	case TECO_VM6575:
+	case TECO_VM656A:
+	case TECO_VM6586:
+	  dev->buffer[6 * j + 0] =
+	    (tmp_buf[0 * dev->def->cal_length + j] >> 0) & 0xff;
+	  dev->buffer[6 * j + 1] =
+	    (tmp_buf[0 * dev->def->cal_length + j] >> 8) & 0xff;
 
-      dev->buffer[6 * j + 4] =
-	(tmp_buf[2 * dev->def->cal_length + j] >> 0) & 0xff;
-      dev->buffer[6 * j + 5] =
-	(tmp_buf[2 * dev->def->cal_length + j] >> 8) & 0xff;
+	  dev->buffer[6 * j + 2] =
+	    (tmp_buf[1 * dev->def->cal_length + j] >> 0) & 0xff;
+	  dev->buffer[6 * j + 3] =
+	    (tmp_buf[1 * dev->def->cal_length + j] >> 8) & 0xff;
+
+	  dev->buffer[6 * j + 4] =
+	    (tmp_buf[2 * dev->def->cal_length + j] >> 0) & 0xff;
+	  dev->buffer[6 * j + 5] =
+	    (tmp_buf[2 * dev->def->cal_length + j] >> 8) & 0xff;
+	  break;
+	case TECO_VM3564:
+	case TECO_VM356A:
+	  dev->buffer[3 * j + 0] = (tmp_buf[3 * j + 0] >> 0) & 0xff;
+
+	  dev->buffer[3 * j + 1] = (tmp_buf[3 * j + 1] >> 0) & 0xff;
+
+	  dev->buffer[3 * j + 2] = (tmp_buf[3 * j + 2] >> 0) & 0xff;
+	  break;
+	}
     }
 
   free (tmp_buf);
@@ -1022,7 +1486,7 @@ teco_do_calibration (Teco_Scanner * dev)
    * one, except for the command. */
 
   cdb.data[0] = 0x0E;
-  size = dev->def->cal_length * 6;
+  size = dev->def->cal_length * dev->def->cal_col_len;
 
   hexdump (DBG_info2, "CDB:", cdb.data, cdb.len);
   /*hexdump (DBG_info2, "calibration line sent:", dev->buffer, size); */
@@ -1041,6 +1505,30 @@ teco_do_calibration (Teco_Scanner * dev)
   return SANE_STATUS_GOOD;
 }
 
+      /*------------request sense command 03----------------*/
+static SANE_Status
+teco_request_sense (Teco_Scanner * dev)
+{
+  CDB cdb;
+  SANE_Status status;
+  size_t size;
+
+
+  MKSCSI_REQUEST_SENSE (cdb, size);
+  size = cdb.data[5];
+
+  hexdump (DBG_info2, "teco_request_sense: enter\n", cdb.data, cdb.len);
+
+  status = sanei_scsi_cmd2 (dev->sfd, cdb.data, cdb.len,
+			    NULL, 0, dev->buffer, &size);
+
+  DBG (DBG_proc, "teco_request_sense: exit, status=%d\n", status);
+
+  return (status);
+}
+
+/*----------------------------------------------------------*/
+
 /* Send the gamma */
 static SANE_Status
 teco_send_gamma (Teco_Scanner * dev)
@@ -1099,9 +1587,16 @@ teco_send_gamma (Teco_Scanner * dev)
     }
 
   hexdump (DBG_info2, "CDB:", cdb.data, cdb.len);
-  status = sanei_scsi_cmd2 (dev->sfd, cdb.data, cdb.len,
-			    &param, size, NULL, NULL);
-
+  switch (dev->def->tecoref)
+    {
+    case TECO_VM3564:
+    case TECO_VM356A:
+      status = 0;
+      break;
+    default:
+      status = sanei_scsi_cmd2 (dev->sfd, cdb.data, cdb.len,
+				&param, size, NULL, NULL);
+    }
   DBG (DBG_proc, "teco_send_gamma: exit, status=%d\n", status);
 
   return (status);
@@ -1547,6 +2042,36 @@ teco_init_options (Teco_Scanner * dev)
   dev->opt[OPT_PREVIEW].cap = SANE_CAP_SOFT_DETECT | SANE_CAP_SOFT_SELECT;
   dev->val[OPT_PREVIEW].w = SANE_FALSE;
 
+  /* red level calibration manual correction */
+  dev->opt[OPT_WHITE_LEVEL_R].name = SANE_NAME_WHITE_LEVEL_R;
+  dev->opt[OPT_WHITE_LEVEL_R].title = SANE_TITLE_WHITE_LEVEL_R;
+  dev->opt[OPT_WHITE_LEVEL_R].desc = SANE_DESC_WHITE_LEVEL_R;
+  dev->opt[OPT_WHITE_LEVEL_R].type = SANE_TYPE_INT;
+  dev->opt[OPT_WHITE_LEVEL_R].unit = SANE_UNIT_NONE;
+  dev->opt[OPT_WHITE_LEVEL_R].constraint_type = SANE_CONSTRAINT_RANGE;
+  dev->opt[OPT_WHITE_LEVEL_R].constraint.range = &red_level_range;
+  dev->val[OPT_WHITE_LEVEL_R].w = 32;	/* to get middle value */
+
+  /* green level calibration manual correction */
+  dev->opt[OPT_WHITE_LEVEL_G].name = SANE_NAME_WHITE_LEVEL_G;
+  dev->opt[OPT_WHITE_LEVEL_G].title = SANE_TITLE_WHITE_LEVEL_G;
+  dev->opt[OPT_WHITE_LEVEL_G].desc = SANE_DESC_WHITE_LEVEL_G;
+  dev->opt[OPT_WHITE_LEVEL_G].type = SANE_TYPE_INT;
+  dev->opt[OPT_WHITE_LEVEL_G].unit = SANE_UNIT_NONE;
+  dev->opt[OPT_WHITE_LEVEL_G].constraint_type = SANE_CONSTRAINT_RANGE;
+  dev->opt[OPT_WHITE_LEVEL_G].constraint.range = &green_level_range;
+  dev->val[OPT_WHITE_LEVEL_G].w = 32;	/* to get middle value */
+  
+  /* blue level calibration manual correction */
+  dev->opt[OPT_WHITE_LEVEL_B].name = SANE_NAME_WHITE_LEVEL_B;
+  dev->opt[OPT_WHITE_LEVEL_B].title = SANE_TITLE_WHITE_LEVEL_B;
+  dev->opt[OPT_WHITE_LEVEL_B].desc = SANE_DESC_WHITE_LEVEL_B;
+  dev->opt[OPT_WHITE_LEVEL_B].type = SANE_TYPE_INT;
+  dev->opt[OPT_WHITE_LEVEL_B].unit = SANE_UNIT_NONE;
+  dev->opt[OPT_WHITE_LEVEL_B].constraint_type = SANE_CONSTRAINT_RANGE;
+  dev->opt[OPT_WHITE_LEVEL_B].constraint.range = &blue_level_range;
+  dev->val[OPT_WHITE_LEVEL_B].w = 32;	/* to get middle value */
+  
   /* Lastly, set the default scan mode. This might change some
    * values previously set here. */
   sane_control_option (dev, OPT_MODE, SANE_ACTION_SET_VALUE,
@@ -2171,6 +2696,9 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	case OPT_CUSTOM_GAMMA:
 	case OPT_PREVIEW:
 	case OPT_THRESHOLD:
+	case OPT_WHITE_LEVEL_R:
+	case OPT_WHITE_LEVEL_G:
+	case OPT_WHITE_LEVEL_B:
 	  *(SANE_Word *) val = dev->val[option].w;
 	  return SANE_STATUS_GOOD;
 
@@ -2219,6 +2747,9 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	case OPT_BR_X:
 	case OPT_THRESHOLD:
 	case OPT_RESOLUTION:
+	case OPT_WHITE_LEVEL_R:
+	case OPT_WHITE_LEVEL_G:
+	case OPT_WHITE_LEVEL_B:
 	  if (info)
 	    {
 	      *info |= SANE_INFO_RELOAD_PARAMS;
@@ -2258,9 +2789,13 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  dev->opt[OPT_DITHER].cap |= SANE_CAP_INACTIVE;
 	  dev->opt[OPT_FILTER_COLOR].cap |= SANE_CAP_INACTIVE;
 	  dev->opt[OPT_THRESHOLD].cap |= SANE_CAP_INACTIVE;
+	  dev->opt[OPT_WHITE_LEVEL_R].cap |= SANE_CAP_INACTIVE;
+	  dev->opt[OPT_WHITE_LEVEL_G].cap |= SANE_CAP_INACTIVE;
+	  dev->opt[OPT_WHITE_LEVEL_B].cap |= SANE_CAP_INACTIVE;
+
 
 	  /* This the default resolution range, except for the
-	   * VM3575 and VM6586 in color mode. */
+	   * VM3575, VM3564, VM356A and VM6586 in color mode. */
 	  dev->opt[OPT_RESOLUTION].constraint_type = SANE_CONSTRAINT_RANGE;
 	  dev->opt[OPT_RESOLUTION].constraint.range = &dev->def->res_range;
 
@@ -2276,10 +2811,36 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	    {
 	      dev->scan_mode = TECO_GRAYSCALE;
 	      dev->depth = 8;
-	      dev->opt[OPT_CUSTOM_GAMMA].cap &= ~SANE_CAP_INACTIVE;
-	      if (dev->val[OPT_CUSTOM_GAMMA].w)
+
+	      switch (dev->def->tecoref)
 		{
-		  dev->opt[OPT_GAMMA_VECTOR_GRAY].cap &= ~SANE_CAP_INACTIVE;
+		case TECO_VM3564:
+		case TECO_VM356A:
+		  dev->opt[OPT_WHITE_LEVEL_R].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_WHITE_LEVEL_G].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_WHITE_LEVEL_B].cap &= ~SANE_CAP_INACTIVE;
+		  break;
+		case TECO_VM3575:
+		  dev->opt[OPT_WHITE_LEVEL_R].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_WHITE_LEVEL_G].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_WHITE_LEVEL_B].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_CUSTOM_GAMMA].cap &= ~SANE_CAP_INACTIVE;
+		  if (dev->val[OPT_CUSTOM_GAMMA].w)
+		    {
+		      dev->opt[OPT_GAMMA_VECTOR_GRAY].cap &=
+			~SANE_CAP_INACTIVE;
+		    }
+		  break;
+		case TECO_VM656A:
+		case TECO_VM6575:
+		case TECO_VM6586:
+		  dev->opt[OPT_CUSTOM_GAMMA].cap &= ~SANE_CAP_INACTIVE;
+		  if (dev->val[OPT_CUSTOM_GAMMA].w)
+		    {
+		      dev->opt[OPT_GAMMA_VECTOR_GRAY].cap &=
+			~SANE_CAP_INACTIVE;
+		    }
+		  break;
 		}
 	      dev->opt[OPT_FILTER_COLOR].cap &= ~SANE_CAP_INACTIVE;
 	    }
@@ -2287,15 +2848,40 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	    {
 	      dev->scan_mode = TECO_COLOR;
 	      dev->depth = 8;
-	      dev->opt[OPT_CUSTOM_GAMMA].cap &= ~SANE_CAP_INACTIVE;
-	      if (dev->val[OPT_CUSTOM_GAMMA].w)
-		{
-		  dev->opt[OPT_GAMMA_VECTOR_R].cap &= ~SANE_CAP_INACTIVE;
-		  dev->opt[OPT_GAMMA_VECTOR_G].cap &= ~SANE_CAP_INACTIVE;
-		  dev->opt[OPT_GAMMA_VECTOR_B].cap &= ~SANE_CAP_INACTIVE;
-		}
 
-	      /* The VM3575 and VM6586 supports only a handful of resolution. Do that here.
+	      switch (dev->def->tecoref)
+		{
+		case TECO_VM3564:
+		case TECO_VM356A:
+		  dev->opt[OPT_WHITE_LEVEL_R].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_WHITE_LEVEL_G].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_WHITE_LEVEL_B].cap &= ~SANE_CAP_INACTIVE;
+		  break;
+		case TECO_VM3575:
+		  dev->opt[OPT_WHITE_LEVEL_R].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_WHITE_LEVEL_G].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_WHITE_LEVEL_B].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_CUSTOM_GAMMA].cap &= ~SANE_CAP_INACTIVE;
+		  if (dev->val[OPT_CUSTOM_GAMMA].w)
+		    {
+		      dev->opt[OPT_GAMMA_VECTOR_R].cap &= ~SANE_CAP_INACTIVE;
+		      dev->opt[OPT_GAMMA_VECTOR_G].cap &= ~SANE_CAP_INACTIVE;
+		      dev->opt[OPT_GAMMA_VECTOR_B].cap &= ~SANE_CAP_INACTIVE;
+		    }
+		  break;
+		case TECO_VM656A:
+		case TECO_VM6575:
+		case TECO_VM6586:
+		  dev->opt[OPT_CUSTOM_GAMMA].cap &= ~SANE_CAP_INACTIVE;
+		  if (dev->val[OPT_CUSTOM_GAMMA].w)
+		    {
+		      dev->opt[OPT_GAMMA_VECTOR_R].cap &= ~SANE_CAP_INACTIVE;
+		      dev->opt[OPT_GAMMA_VECTOR_G].cap &= ~SANE_CAP_INACTIVE;
+		      dev->opt[OPT_GAMMA_VECTOR_B].cap &= ~SANE_CAP_INACTIVE;
+		    }
+		  break;
+		}
+	      /* The VM3575, VM3564, VM356A and VM6586 supports only a handful of resolution. Do that here.
 	       * Ugly! */
 	      if (dev->resolutions_list != NULL)
 		{
@@ -2392,8 +2978,20 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
        * in the SET WINDOWS command. */
       if (dev->val[OPT_PREVIEW].w == SANE_TRUE)
 	{
-	  dev->x_resolution = 50;
-	  dev->y_resolution = 50;
+	  /* for VM356A, no good 50 dpi scan possible, now leave as */
+
+	  switch (dev->def->tecoref)
+	    {
+	    case TECO_VM356A:
+	      dev->x_resolution = 75;
+	      dev->y_resolution = 75;
+	      break;
+	      /* For VM3575, VM6575, VM656A, VM6586 etc until otherwise default value is used */
+	    default:
+	      dev->x_resolution = 50;
+	      dev->y_resolution = 50;
+	      break;
+	    }
 	  dev->x_tl = 0;
 	  dev->y_tl = 0;
 	  dev->x_br = mmToIlu (SANE_UNFIX (dev->def->x_range.max));
@@ -2489,7 +3087,7 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
 
 	      int i;
 	      for (i = 0;
-		   dev->def->color_adjust[i].resolution != dev->x_resolution;
+		   dev->def->color_adjust[i].resolution != dev->y_resolution;
 		   i++);
 
 	      dev->color_adjust = &dev->def->color_adjust[i];
@@ -2594,6 +3192,22 @@ sane_start (SANE_Handle handle)
 	}
 #endif
 
+      switch (dev->def->tecoref)
+	{
+	case TECO_VM3564:
+	case TECO_VM356A:
+/*--------------request sense for  first time loop---*/
+	  status = teco_request_sense (dev);
+	  if (status)
+	    {
+	      teco_close (dev);
+	      return status;
+	    }
+	  break;
+	default:
+	  break;
+	}
+
       status = teco_send_gamma (dev);
       if (status)
 	{
@@ -2607,14 +3221,19 @@ sane_start (SANE_Handle handle)
 	  teco_close (dev);
 	  return status;
 	}
-
-      status = teco_send_vendor_06 (dev);
-      if (status)
+      switch (dev->def->tecoref)
 	{
-	  teco_close (dev);
-	  return status;
+	case TECO_VM3564:
+	case TECO_VM356A:
+	  break;
+	default:
+	  status = teco_send_vendor_06 (dev);
+	  if (status)
+	    {
+	      teco_close (dev);
+	      return status;
+	    }
 	}
-
       status = teco_scan (dev);
       if (status)
 	{
