@@ -42,7 +42,7 @@
 */
 
 /* 
-	$Id: teco2.h,v 1.5 2003/04/24 03:16:50 fzago Exp $
+	$Id: teco2.h,v 1.8 2004/01/15 17:39:30 gekl-guest Exp $
 */
 
 /* Commands supported by the scanner. */
@@ -293,9 +293,11 @@ enum Teco_Option
   OPT_GAMMA_VECTOR_GRAY,	/* Custom Grayscale Gamma table */
 
   OPT_DITHER,
-  OPT_FILTER_COLOR,				/* which color to filter */
-  OPT_THRESHOLD,				/* Threshold */
-
+  OPT_FILTER_COLOR,		/* which color to filter */
+  OPT_THRESHOLD,		/* Threshold */
+  OPT_WHITE_LEVEL_R,		/* white level correction RED */
+  OPT_WHITE_LEVEL_G,            /* white level correction GREEN */
+  OPT_WHITE_LEVEL_B,            /* white level correction BLUE */
 
   OPT_PREVIEW,
 
@@ -303,14 +305,6 @@ enum Teco_Option
   OPT_NUM_OPTIONS
 };
 
-typedef union
-{
-  SANE_Word w;			/* word */
-  SANE_Word *wa;		/* word array */
-  SANE_String s;		/* string */
-}
-Option_Value;
-
 /*--------------------------------------------------------------------------*/
 
 /* 
@@ -322,7 +316,7 @@ struct scanners_supported
   char scsi_teco_name[12];	/* real name of the scanner */
   enum
   {
-    TECO_VM3564,  
+    TECO_VM3564,
     TECO_VM356A,
     TECO_VM3575,
     TECO_VM6575,
@@ -340,8 +334,8 @@ struct scanners_supported
 
   int cal_length;		/* size of a calibration line in pixels */
   int cal_lines;		/* number of calibration lines to read */
-	int cal_col_len;			/* number of byte to code one color */
-	int cal_algo; /* default algo to use to compute calibration line */
+  int cal_col_len;		/* number of byte to code one color */
+  int cal_algo;			/* default algo to use to compute calibration line */
 
   /* Minimum and maximum width and length supported. */
   SANE_Range x_range;
