@@ -1,5 +1,7 @@
 /* sane - Scanner Access Now Easy.
    Copyright (C) 2002, 2003 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Changes according to the sanei_thread usage by
+                                         Gerhard Jaeger <gerhard@gjaeger.de>
    This file is part of the SANE package.
 
    This program is free software; you can redistribute it and/or
@@ -36,12 +38,12 @@
 
    If you write modifications of your own for SANE, it is your choice
    whether to permit this exception to apply to your modifications.
-   If you do not wish that, delete this exception notice. 
+   If you do not wish that, delete this exception notice.
 
    This backend is for testing frontends.
 */
 
-#define BUILD 21
+#define BUILD 27
 
 #include "../include/sane/config.h"
 
@@ -51,7 +53,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
-#include <sys/wait.h>
 #include <time.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -60,9 +61,7 @@
 #include "../include/sane/sanei.h"
 #include "../include/sane/saneopts.h"
 #include "../include/sane/sanei_config.h"
-#ifdef HAVE_OS2_H
 #include "../include/sane/sanei_thread.h"
-#endif
 
 #define BACKEND_NAME	test
 #include "../include/sane/sanei_backend.h"
@@ -114,8 +113,8 @@ static SANE_Range int_constraint_range = {
 };
 
 static SANE_Range fixed_constraint_range = {
-  SANE_FIX (17001.0),
-  SANE_FIX (32767.0),
+  SANE_FIX (-42.17),
+  SANE_FIX (32767.9999),
   SANE_FIX (2.0)
 };
 
@@ -165,7 +164,7 @@ static SANE_String_Const string_constraint_string_list[] = {
 
 static SANE_String_Const string_constraint_long_string_list[] = {
   SANE_I18N ("First entry"), SANE_I18N ("Second entry"), "3", "4", "5", "6",
-    "7", "8", "9", "10",
+  "7", "8", "9", "10",
   "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22",
   "23", "24", "25", "26", "27", "28", "29", "30", "31", "32", "33", "34",
   "35", "36", "37", "38", "39", "40", "41", "42", "43", "44", "45", "46",
@@ -181,7 +180,7 @@ static SANE_Int int_array_constraint_range[] = {
 };
 
 static SANE_Int int_array_constraint_word_list[] = {
-  -42, 0, 9, 17, 42, 42
+  -42, 0, -8, 17, 42, 42
 };
 
 static double random_factor;	/* use for fuzzyness of parameters */
@@ -273,7 +272,7 @@ init_options (Test_Device * test_device)
 {
   SANE_Option_Descriptor *od;
 
-  DBG (2, "init_options: test_device=%p\n", test_device);
+  DBG (2, "init_options: test_device=%p\n", (void *) test_device);
 
   /* opt_num_opts */
   od = &test_device->opt[opt_num_opts];
@@ -686,7 +685,7 @@ init_options (Test_Device * test_device)
   od->type = SANE_TYPE_GROUP;
   od->unit = SANE_UNIT_NONE;
   od->size = 0;
-  od->cap = 0;
+  od->cap = SANE_CAP_ADVANCED;
   od->constraint_type = SANE_CONSTRAINT_NONE;
   od->constraint.range = 0;
   test_device->val[opt_bool_group].w = 0;
@@ -769,7 +768,7 @@ init_options (Test_Device * test_device)
   od->name = "bool-soft-select-soft-detect-emulated";
   od->title = SANE_I18N ("(5/6) Bool soft select soft detect emulated");
   od->desc = SANE_I18N ("(5/6) Bool test option that has soft select, soft "
-			"detect, and emulated (and advanced) capabilities. ");
+			"detect, and emulated (and advanced) capabilities.");
   od->type = SANE_TYPE_BOOL;
   od->unit = SANE_UNIT_NONE;
   od->size = sizeof (SANE_Word);
@@ -807,7 +806,7 @@ init_options (Test_Device * test_device)
   od->type = SANE_TYPE_GROUP;
   od->unit = SANE_UNIT_NONE;
   od->size = 0;
-  od->cap = 0;
+  od->cap = SANE_CAP_ADVANCED;
   od->constraint_type = SANE_CONSTRAINT_NONE;
   od->constraint.range = 0;
   test_device->val[opt_int_group].w = 0;
@@ -850,7 +849,7 @@ init_options (Test_Device * test_device)
   od->name = "int-constraint-word-list";
   od->title = SANE_I18N ("(3/6) Int constraint word list");
   od->desc = SANE_I18N ("(3/6) Int test option with unit bits and "
-			"constraint word list set. ");
+			"constraint word list set.");
   od->type = SANE_TYPE_INT;
   od->unit = SANE_UNIT_BIT;
   od->size = sizeof (SANE_Word);
@@ -881,7 +880,7 @@ init_options (Test_Device * test_device)
   od = &test_device->opt[opt_int_array_constraint_range];
   od->name = "int-constraint-array-constraint-range";
   od->title = SANE_I18N ("(5/6) Int array constraint range");
-  od->desc = SANE_I18N ("(5/6) Int test option with unit mm and using "
+  od->desc = SANE_I18N ("(5/6) Int test option with unit dpi and using "
 			"an array with a range constraint. Minimum is 4, "
 			"maximum 192, and quant is 2.");
   od->type = SANE_TYPE_INT;
@@ -900,7 +899,7 @@ init_options (Test_Device * test_device)
   od->name = "int-constraint-array-constraint-word-list";
   od->title = SANE_I18N ("(6/6) Int array constraint word list");
   od->desc = SANE_I18N ("(6/6) Int test option with unit percent and using "
-			"an array a word list constraint.");
+			"an array with a word list constraint.");
   od->type = SANE_TYPE_INT;
   od->unit = SANE_UNIT_PERCENT;
   od->size = 6 * sizeof (SANE_Word);
@@ -920,7 +919,7 @@ init_options (Test_Device * test_device)
   od->type = SANE_TYPE_GROUP;
   od->unit = SANE_UNIT_NONE;
   od->size = 0;
-  od->cap = 0;
+  od->cap = SANE_CAP_ADVANCED;
   od->constraint_type = SANE_CONSTRAINT_NONE;
   od->constraint.range = 0;
   test_device->val[opt_fixed_group].w = 0;
@@ -956,14 +955,14 @@ init_options (Test_Device * test_device)
     od->cap |= SANE_CAP_INACTIVE;
   od->constraint_type = SANE_CONSTRAINT_RANGE;
   od->constraint.range = &fixed_constraint_range;
-  test_device->val[opt_fixed_constraint_range].w = SANE_FIX (42.0);
+  test_device->val[opt_fixed_constraint_range].w = SANE_FIX (41.83);
 
   /* opt_fixed_constraint_word_list */
   od = &test_device->opt[opt_fixed_constraint_word_list];
   od->name = "fixed-constraint-word-list";
   od->title = SANE_I18N ("(3/3) Fixed constraint word list");
   od->desc = SANE_I18N ("(3/3) Fixed test option with no unit and "
-			"constraint word list set. ");
+			"constraint word list set.");
   od->type = SANE_TYPE_FIXED;
   od->unit = SANE_UNIT_NONE;
   od->size = sizeof (SANE_Word);
@@ -1088,6 +1087,9 @@ read_option (SANE_String line, SANE_String option_string,
 
   cp = sanei_config_get_string (line, &word);
 
+  if (!word)
+    return SANE_STATUS_INVAL;
+
   if (strcmp (word, option_string) != 0)
     return SANE_STATUS_INVAL;
 
@@ -1101,6 +1103,8 @@ read_option (SANE_String line, SANE_String option_string,
     case param_bool:
       {
 	cp = sanei_config_get_string (cp, &word);
+	if (!word)
+	  return SANE_STATUS_INVAL;
 	if (strlen (word) == 0)
 	  {
 	    DBG (3, "read_option: option `%s' requires parameter\n",
@@ -1126,6 +1130,8 @@ read_option (SANE_String line, SANE_String option_string,
 	SANE_Int int_value;
 
 	cp = sanei_config_get_string (cp, &word);
+	if (!word)
+	  return SANE_STATUS_INVAL;
 	errno = 0;
 	int_value = (SANE_Int) strtol (word, &end, 0);
 	if (end == word)
@@ -1154,6 +1160,8 @@ read_option (SANE_String line, SANE_String option_string,
 	SANE_Fixed fixed_value;
 
 	cp = sanei_config_get_string (cp, &word);
+	if (!word)
+	  return SANE_STATUS_INVAL;
 	errno = 0;
 	double_value = strtod (word, &end);
 	if (end == word)
@@ -1180,6 +1188,8 @@ read_option (SANE_String line, SANE_String option_string,
     case param_string:
       {
 	cp = sanei_config_get_string (cp, &word);
+	if (!word)
+	  return SANE_STATUS_INVAL;
 	if (strlen (word) == 0)
 	  {
 	    DBG (3, "read_option: option `%s' requires parameter\n",
@@ -1205,21 +1215,6 @@ read_option (SANE_String line, SANE_String option_string,
   return SANE_STATUS_GOOD;
 }
 
-#ifdef HAVE_OS2_H
-/*
- * reader thread for OS/2: need a wrapper, because threads can have
- * only one parameter.
-*/
-static void
-os2_reader_process (void *data)
-{
-  struct Test_Device *test_device = (struct Test_Device *) data;
-
-  DBG (1, "reader_process thread started\n");
-  reader_process (test_device, test_device->reader_fds);
-}
-#endif
-
 static SANE_Status
 reader_process (Test_Device * test_device, SANE_Int fd)
 {
@@ -1229,7 +1224,8 @@ reader_process (Test_Device * test_device, SANE_Int fd)
   ssize_t bytes_written = 0;
   size_t buffer_size = 0, write_count = 0;
 
-  DBG (2, "(child) reader_process: test_device=%p, fd=%d\n", test_device, fd);
+  DBG (2, "(child) reader_process: test_device=%p, fd=%d\n",
+       (void *) test_device, fd);
 
   bytes_total = test_device->lines * test_device->bytes_per_line;
   status = init_picture_buffer (test_device, &buffer, &buffer_size);
@@ -1264,26 +1260,69 @@ reader_process (Test_Device * test_device, SANE_Int fd)
     }
 
   free (buffer);
-  DBG (4, "(child) reader_process: finished,  wrote %d bytes, expected %d "
+
+  if (sanei_thread_is_forked ())
+    {
+	  DBG (4, "(child) reader_process: finished,  wrote %d bytes, expected %d "
        "bytes, now waiting\n", byte_count, bytes_total);
-  sleep (1000);
-  close (fd);
+	  while (SANE_TRUE)
+	    sleep (10);
+	  DBG (4, "(child) reader_process: this should have never happened...");
+	  close (fd);
+    }
+  else
+    {
+	  DBG (4, "(child) reader_process: finished,  wrote %d bytes, expected %d "
+       "bytes\n", byte_count, bytes_total);
+    }
   return SANE_STATUS_GOOD;
 }
 
+/*
+ * this code either runs in child or thread context...
+ */
+static int
+reader_task (void *data)
+{
+  SANE_Status status;
+  struct SIGACTION act;
+  struct Test_Device *test_device = (struct Test_Device *) data;
+
+  DBG (2, "reader_task started\n");
+  if (sanei_thread_is_forked ())
+    {
+      DBG (3, "reader_task started (forked)\n");
+      close (test_device->pipe);
+      test_device->pipe = -1;
+
+    }
+  else
+    {
+      DBG (3, "reader_task started (as thread)\n");
+    }
+
+  memset (&act, 0, sizeof (act));
+  sigaction (SIGTERM, &act, 0);
+
+  status = reader_process (test_device, test_device->reader_fds);
+  DBG (2, "(child) reader_task: reader_process finished (%s)\n",
+       sane_strstatus (status));
+  return (int) status;
+}
+
 static SANE_Status
 finish_pass (Test_Device * test_device)
 {
   SANE_Status return_status = SANE_STATUS_GOOD;
 
-  DBG (2, "finish_pass: test_device=%p\n", test_device);
+  DBG (2, "finish_pass: test_device=%p\n", (void *) test_device);
   test_device->scanning = SANE_FALSE;
-  if (test_device->pipe > 0)
+  if (test_device->pipe >= 0)
     {
       DBG (2, "finish_pass: closing pipe\n");
       close (test_device->pipe);
       DBG (2, "finish_pass: pipe closed\n");
-      test_device->pipe = 0;
+      test_device->pipe = -1;
     }
   if (test_device->reader_pid > 0)
     {
@@ -1292,32 +1331,29 @@ finish_pass (Test_Device * test_device)
 
       DBG (2, "finish_pass: terminating reader process %d\n",
 	   test_device->reader_pid);
-      kill (test_device->reader_pid, SIGTERM);
-      pid = waitpid (test_device->reader_pid, &status, 0);
+      sanei_thread_kill (test_device->reader_pid);
+      pid = sanei_thread_waitpid (test_device->reader_pid, &status);
       if (pid < 0)
 	{
-	  DBG (1, "finish_pass: waitpid failed, already terminated? (%s)\n",
+	  DBG (1,
+	       "finish_pass: sanei_thread_waitpid failed, already terminated? (%s)\n",
 	       strerror (errno));
 	}
-      else if (WIFEXITED (status))
-	{
-	  DBG (2, "finish_pass: reader process terminated with status %s\n",
-	       sane_strstatus (WEXITSTATUS (status)));
-	  if (WEXITSTATUS (status) != SANE_STATUS_GOOD)
-	    return_status = WEXITSTATUS (status);
-	}
-      else if (WIFSIGNALED (status))
-	{
-	  DBG (2, "finish_pass: reader process was terminated by signal %d\n",
-	       WTERMSIG (status));
-	  if (WTERMSIG (status) != 15)
-	    return_status = SANE_STATUS_IO_ERROR;
-	}
       else
-	DBG (1, "finish_pass: reader process terminated by unknown reason\n");
-
+	{
+	  DBG (2, "finish_pass: reader process terminated with status: %s\n",
+	       sane_strstatus (status));
+	}
       test_device->reader_pid = 0;
     }
+  /* this happens when running in thread context... */
+  if (test_device->reader_fds >= 0)
+    {
+      DBG (2, "finish_pass: closing reader pipe\n");
+      close (test_device->reader_fds);
+      DBG (2, "finish_pass: reader pipe closed\n");
+      test_device->reader_fds = -1;
+    }
   return return_status;
 }
 
@@ -1395,14 +1431,15 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   SANE_Int num;
 
   DBG_INIT ();
+  sanei_thread_init ();
 
   test_device = 0;
   previous_device = 0;
 
-  DBG (2, "sane_init: version_code= %p, authorize=%p\n", version_code,
-       authorize);
-  DBG (1, "sane_init: SANE test backend version %d.%d.%d from %s\n",
-       V_MAJOR, V_MINOR, BUILD, PACKAGE_STRING);
+  DBG (2, "sane_init: version_code= %p, authorize=%p\n",
+       (void *) version_code, (void *) authorize);
+  DBG (1, "sane_init: SANE test backend version %d.%d.%d from %s\n", V_MAJOR,
+       V_MINOR, BUILD, PACKAGE_STRING);
 
   if (version_code)
     *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, BUILD);
@@ -1571,8 +1608,8 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
       test_device->eof = SANE_FALSE;
       test_device->scanning = SANE_FALSE;
       test_device->cancelled = SANE_FALSE;
-      test_device->reader_pid = 0;
-      test_device->pipe = 0;
+      test_device->reader_pid = -1;
+      test_device->pipe = -1;
       DBG (4, "sane_init: new device: `%s' is a %s %s %s\n",
 	   test_device->sane.name, test_device->sane.vendor,
 	   test_device->sane.model, test_device->sane.type);
@@ -1622,7 +1659,7 @@ sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
 {
 
   DBG (2, "sane_get_devices: device_list=%p, local_only=%d\n",
-       device_list, local_only);
+       (void *) device_list, local_only);
   if (!inited)
     {
       DBG (1, "sane_get_devices: not inited, call sane_init() first\n");
@@ -1644,7 +1681,8 @@ sane_open (SANE_String_Const devicename, SANE_Handle * handle)
   Test_Device *test_device = first_test_device;
   SANE_Status status;
 
-  DBG (2, "sane_open: devicename = \"%s\", handle=%p\n", devicename, handle);
+  DBG (2, "sane_open: devicename = \"%s\", handle=%p\n",
+       devicename, (void *) handle);
   if (!inited)
     {
       DBG (1, "sane_open: not inited, call sane_init() first\n");
@@ -1681,7 +1719,7 @@ sane_open (SANE_String_Const devicename, SANE_Handle * handle)
       return SANE_STATUS_DEVICE_BUSY;
     }
   DBG (2, "sane_open: opening device `%s', handle = %p\n", test_device->name,
-       test_device);
+       (void *) test_device);
   test_device->open = SANE_TRUE;
   *handle = test_device;
 
@@ -1704,7 +1742,7 @@ sane_close (SANE_Handle handle)
 {
   Test_Device *test_device = handle;
 
-  DBG (2, "sane_close: handle=%p\n", handle);
+  DBG (2, "sane_close: handle=%p\n", (void *) handle);
   if (!inited)
     {
       DBG (1, "sane_close: not inited, call sane_init() first\n");
@@ -1713,12 +1751,12 @@ sane_close (SANE_Handle handle)
 
   if (!check_handle (handle))
     {
-      DBG (1, "sane_close: handle %p unknown\n", handle);
+      DBG (1, "sane_close: handle %p unknown\n", (void *) handle);
       return;
     }
   if (!test_device->open)
     {
-      DBG (1, "sane_close: handle %p not open\n", handle);
+      DBG (1, "sane_close: handle %p not open\n", (void *) handle);
       return;
     }
   test_device->open = SANE_FALSE;
@@ -1730,8 +1768,8 @@ sane_get_option_descriptor (SANE_Handle handle, SANE_Int option)
 {
   Test_Device *test_device = handle;
 
-  DBG (4, "sane_get_option_descriptor: handle=%p, option = %d\n", handle,
-       option);
+  DBG (4, "sane_get_option_descriptor: handle=%p, option = %d\n",
+       (void *) handle, option);
   if (!inited)
     {
       DBG (1, "sane_get_option_descriptor: not inited, call sane_init() "
@@ -1741,7 +1779,8 @@ sane_get_option_descriptor (SANE_Handle handle, SANE_Int option)
 
   if (!check_handle (handle))
     {
-      DBG (1, "sane_get_option_descriptor: handle %p unknown\n", handle);
+      DBG (1, "sane_get_option_descriptor: handle %p unknown\n",
+	   (void *) handle);
       return 0;
     }
   if (!test_device->open)
@@ -1768,7 +1807,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option, SANE_Action action,
   SANE_Status status;
 
   DBG (4, "sane_control_option: handle=%p, opt=%d, act=%d, val=%p, info=%p\n",
-       handle, option, action, value, info);
+       (void *) handle, option, action, (void *) value, (void *) info);
   if (!inited)
     {
       DBG (1, "sane_control_option: not inited, call sane_init() first\n");
@@ -1777,7 +1816,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option, SANE_Action action,
 
   if (!check_handle (handle))
     {
-      DBG (1, "sane_control_option: handle %p unknown\n", handle);
+      DBG (1, "sane_control_option: handle %p unknown\n", (void *) handle);
       return SANE_STATUS_INVAL;
     }
   if (!test_device->open)
@@ -2249,7 +2288,8 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
   SANE_String text_format, mode;
   SANE_Int channels = 1;
 
-  DBG (2, "sane_get_parameters: handle=%p, params=%p\n", handle, params);
+  DBG (2, "sane_get_parameters: handle=%p, params=%p\n",
+       (void *) handle, (void *) params);
   if (!inited)
     {
       DBG (1, "sane_get_parameters: not inited, call sane_init() first\n");
@@ -2257,12 +2297,12 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
     }
   if (!check_handle (handle))
     {
-      DBG (1, "sane_get_parameters: handle %p unknown\n", handle);
+      DBG (1, "sane_get_parameters: handle %p unknown\n", (void *) handle);
       return SANE_STATUS_INVAL;
     }
   if (!test_device->open)
     {
-      DBG (1, "sane_get_parameters: handle %p not open\n", handle);
+      DBG (1, "sane_get_parameters: handle %p not open\n", (void *) handle);
       return SANE_STATUS_INVAL;
     }
 
@@ -2372,13 +2412,13 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
       break;
     }
 
-  DBG (2, "sane_get_parameters: format=%s\n", text_format);
-  DBG (2, "sane_get_parameters: last_frame=%s\n",
+  DBG (3, "sane_get_parameters: format=%s\n", text_format);
+  DBG (3, "sane_get_parameters: last_frame=%s\n",
        p->last_frame ? "true" : "false");
-  DBG (2, "sane_get_parameters: lines=%d\n", p->lines);
-  DBG (2, "sane_get_parameters: depth=%d\n", p->depth);
-  DBG (2, "sane_get_parameters: pixels_per_line=%d\n", p->pixels_per_line);
-  DBG (2, "sane_get_parameters: bytes_per_line=%d\n", p->bytes_per_line);
+  DBG (3, "sane_get_parameters: lines=%d\n", p->lines);
+  DBG (3, "sane_get_parameters: depth=%d\n", p->depth);
+  DBG (3, "sane_get_parameters: pixels_per_line=%d\n", p->pixels_per_line);
+  DBG (3, "sane_get_parameters: bytes_per_line=%d\n", p->bytes_per_line);
 
   if (params)
     *params = *p;
@@ -2455,41 +2495,24 @@ sane_start (SANE_Handle handle)
       return SANE_STATUS_IO_ERROR;
     }
 
-#ifndef HAVE_OS2_H
-  test_device->reader_pid = fork ();
-#else
   /* create reader routine as new process or thread */
+  test_device->pipe = pipe_descriptor[0];
   test_device->reader_fds = pipe_descriptor[1];
-  test_device->reader_pid = sanei_thread_begin (os2_reader_process, (void *) test_device);
-#endif
-  if (test_device->reader_pid == 0)	/* child */
-    {
-      SANE_Status status;
-      sigset_t ignore_set;
-      struct SIGACTION act;
+  test_device->reader_pid =
+    sanei_thread_begin (reader_task, (void *) test_device);
 
-      /* block all signals but SIGTERM */
-      sigfillset (&ignore_set);
-      sigdelset (&ignore_set, SIGTERM);
-      sigprocmask (SIG_SETMASK, &ignore_set, 0);
-      memset (&act, 0, sizeof (act));
-      sigaction (SIGTERM, &act, 0);
-
-      close (pipe_descriptor[0]);
-      status = reader_process (test_device, pipe_descriptor[1]);
-      DBG (2, "(child) sane_start: reader_process timed out\n");
-      _exit (status);
-    }
-  else if (test_device->reader_pid < 0)
+  if (test_device->reader_pid < 0)
     {
-      DBG (1, "sane_start: fork failed (%s)\n", strerror (errno));
+      DBG (1, "sane_start: sanei_thread_begin failed (%s)\n",
+	   strerror (errno));
       return SANE_STATUS_NO_MEM;
     }
-  /* parent */
-#ifndef HAVE_OS2_H
-  close (pipe_descriptor[1]);
-#endif
-  test_device->pipe = pipe_descriptor[0];
+
+  if (sanei_thread_is_forked ())
+    {
+      close (test_device->reader_fds);
+      test_device->reader_fds = -1;
+    }
 
   return SANE_STATUS_GOOD;
 }
@@ -2507,7 +2530,7 @@ sane_read (SANE_Handle handle, SANE_Byte * data,
 
 
   DBG (4, "sane_read: handle=%p, data=%p, max_length = %d, length=%p\n",
-       handle, data, max_length, length);
+       handle, data, max_length, (void *) length);
   if (!inited)
     {
       DBG (1, "sane_read: not inited, call sane_init() first\n");
@@ -2591,7 +2614,7 @@ sane_read (SANE_Handle handle, SANE_Byte * data,
   read_count = max_scan_length;
 
   bytes_read = read (test_device->pipe, data, read_count);
-  if (bytes_read == 0  
+  if (bytes_read == 0
       || (bytes_read + test_device->bytes_total >= bytes_total))
     {
       SANE_Status status;
@@ -2631,7 +2654,7 @@ sane_read (SANE_Handle handle, SANE_Byte * data,
   test_device->bytes_total += bytes_read;
 
   DBG (2, "sane_read: read %d bytes of %d, total %d\n", bytes_read,
-       max_length, test_device->bytes_total);
+       max_scan_length, test_device->bytes_total);
   return SANE_STATUS_GOOD;
 }
 
@@ -2658,12 +2681,12 @@ sane_cancel (SANE_Handle handle)
     }
   if (test_device->cancelled)
     {
-      DBG (1, "sane_read: scan already cancelled\n");
+      DBG (1, "sane_cancel: scan already cancelled\n");
       return;
     }
   if (!test_device->scanning)
     {
-      DBG (1, "sane_read: not scanning\n");
+      DBG (2, "sane_cancel: scan is already finished\n");
       return;
     }
   finish_pass (test_device);
