@@ -93,14 +93,6 @@ enum Umax_PP_Option
   NUM_OPTIONS
 };
 
-typedef union
-{
-  SANE_Word w;
-  SANE_Word *wa;		/* word array */
-  SANE_String s;
-}
-Option_Value;
-
 
 typedef struct Umax_PP_Descriptor
 {
@@ -192,17 +184,4 @@ Umax_PP_Device;
 			  __PRETTY_FUNCTION__, V_MAJOR, V_MINOR,	\
 			  UMAX_PP_BUILD, UMAX_PP_STATE, __LINE__)
 
-#define ASSERT(cond)	if (!(cond))					\
-			  {						\
-                            DEBUG();					\
-			    DBG(1, "ASSERT(%s) failed\n", STRINGIFY(cond)); \
-			    DBG(1, "expect disaster...\n");\
-			  }
-
-/* Please note: ASSERT won't go away if you define NDEBUG, it just won't
- * output a message when ASSERT failes. So if "cond" does anything, it will
- * be executed, even if NDEBUG is defined... 
- */
-
-
 #endif /* umax_pp_h */
