@@ -2,7 +2,7 @@
    Uses the SANE library.
    Copyright (C) 1996, 1997, 1998 Andreas Beck and David Mosberger
 
-   You can contact me at becka@sunserver1.rz.uni-duesseldorf.de
+   You can contact me at becka@uni-duesseldorf.de
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -70,6 +70,7 @@ Image;
 #define OPTION_BATCH_START_AT	1004
 #define OPTION_BATCH_DOUBLE	1005
 #define OPTION_BATCH_INCREMENT	1006
+#define OPTION_BATCH_PROMPT    1007
 
 #define BATCH_COUNT_UNLIMITED -1
 
@@ -86,6 +87,7 @@ static struct option basic_options[] = {
   {"batch-start", required_argument, NULL, OPTION_BATCH_START_AT},
   {"batch-double", no_argument, NULL, OPTION_BATCH_DOUBLE},
   {"batch-increment", required_argument, NULL, OPTION_BATCH_INCREMENT},
+  {"batch-prompt", no_argument, NULL, OPTION_BATCH_PROMPT},
   {"format", required_argument, NULL, OPTION_FORMAT},
   {"accept-md5-only", no_argument, NULL, OPTION_MD5},
   {"icc-profile", required_argument, NULL, 'i'},
@@ -445,12 +447,12 @@ print_option (SANE_Device * device, int opt_num, char short_name)
   if (opt->type == SANE_TYPE_STRING || opt->size == sizeof (SANE_Word))
     {
       /* print current option value */
-      fputs (" [", stdout);
       if (SANE_OPTION_IS_ACTIVE (opt->cap))
 	{
 	  void *val = alloca (opt->size);
 	  sane_control_option (device, opt_num, SANE_ACTION_GET_VALUE, val,
 			       0);
+	  fputs (" [", stdout);
 	  switch (opt->type)
 	    {
 	    case SANE_TYPE_BOOL:
@@ -472,11 +474,13 @@ print_option (SANE_Device * device, int opt_num, char short_name)
 	    default:
 	      break;
 	    }
+	  fputc (']', stdout);
 	}
-      else
-	fputs ("inactive", stdout);
-      fputc (']', stdout);
     }
+
+  if (!SANE_OPTION_IS_ACTIVE (opt->cap))
+    fputs (" [inactive]", stdout);
+
   fputs ("\n        ", stdout);
 
   switch (short_name)
@@ -931,7 +935,14 @@ process_backend_option (SANE_Handle device, int optnum, const char *optarg)
       break;
 
     case SANE_TYPE_STRING:
-      valuep = (void *) optarg;
+      valuep = malloc (opt->size);
+      if (!valuep)
+	{
+	  fprintf (stderr, "%s: out of memory\n", prog_name);
+	  exit (1);
+	}
+      strncpy (valuep, optarg, opt->size);
+      ((char *) valuep)[opt->size - 1] = 0;
       break;
 
     case SANE_TYPE_BUTTON:
@@ -1457,7 +1468,10 @@ main (int argc, char **argv)
   const char *devname = 0;
   const char *defdevname = 0;
   const char *format = 0;
+  char readbuf[2];
+  char *readbuf2;
   int batch = 0;
+  int batch_prompt = 0;
   int batch_count = BATCH_COUNT_UNLIMITED;
   int batch_start_at = 1;
   int batch_increment = 1;
@@ -1511,6 +1525,9 @@ main (int argc, char **argv)
 	case 'n':
 	  dont_scan = 1;
 	  break;
+	case OPTION_BATCH_PROMPT:
+          batch_prompt = 1;
+          break;
 	case OPTION_BATCH_INCREMENT:
 	  batch_increment = atoi (optarg);
 	  break;
@@ -1683,6 +1700,7 @@ standard output.\n\
     --batch-increment=#    increase number in filename by an amount of #\n\
     --batch-double         increment page number by two for 2sided originals\n\
                            being scanned in a single sided scanner\n\
+    --batch-prompt         ask for pressing a key before scanning a page\n\
     --accept-md5-only      only accept authorization requests using md5\n\
 -n, --dont-scan            only set options, don't actually scan\n\
 -T, --test                 test backend thoroughly\n\
@@ -1961,6 +1979,12 @@ List of available devices:", prog_name);
 
 	  if (batch)
 	    {
+	      if (batch_prompt)
+                {
+                  fprintf (stderr, "Place document no. %d on the scanner.\n", n);
+                  fprintf (stderr, "Press <RETURN> to continue.\n");
+                  readbuf2=fgets(readbuf, 2, stdin);
+                }
 	      fprintf (stderr, "Scanning page %d\n", n);
 	    }
 	  status = scan_it ();
